/*
 * Gasell: Copyright 2005-2006 Linux-Nantes Association <contact@linux-nantes.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Poulard Fabien (GrdScarabe) <grdscarabe@grdscarabe.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 */

/**
 * Ce fichier contient les fonctions de manipulation des formulaires.
 */

/**
 * Fonction de gestion des erreurs sur les formulaires
 *
 * @author Fabien Poulard <grandscarabe@free.fr>
 * @version $Revision: 1048 $
 * @package gasell
 * 
 * @param mesg le message qui s'affiche dans la boite d'alerte
 * @param field l'identifiant du champs input (pas de type hidden) concerne par l'erreur  [facultatif]
 */
function BoxFormError(mesg , field)
{
	/* affichage du message d'erreur */
	alert(mesg);
	
	/* si le champs est specifie, on donne le focus sur le champs */
	if ( (field != null) && (field != undefined) && (field.type == "text") )
	{
	        /* on recupere le champs  */
		var inp = document.getElementById(field);

		/* on verifie que le champs a ete effectivement recupere */
		if ( (inp != null) && (inp != undefined) )
		  {
		    /* on recupere le champs d'origine */
		    var f = document.getElementById(field).form;
		    
		    /* on remet les fonds des champs en blanc */
		    for(var j=0 ; j<f.elements.length ; j++)
		      if(f.elements[j].type == "text") f.elements[j].style.backgroundColor="white";
		    /* on met le champs fautif en valeur */
		    inp.style.backgroundColor="#FA5555";
		    inp.focus();
		  }
		else
		  {
		    document.write("<div class=\"warning\">");
		    document.write("Erreur dans le javascript, pr&eacute;venir <a href=\"mailto:grandscarabe@free.fr\">le mainteneur</a>");
		    document.write("</div>");
		  }
	}
}


/**
 * Fonction qui verifie que deux passwords sont identiques
 *
 * @author Fabien Poulard <grandscarabe@free.fr>
 * @version $Revision: 1048 $
 * @package gasell
 *
 * @param string pw1 l'identifiant du premier champs password
 * @param string pw2 l'identifiant du deuxieme champs password
 */
function CheckPassword(pw1, pw2)
{
		/* on recupere les passwords */
	 	var p1 = document.getElementById(pw1).value;
	 	var p2 = document.getElementById(pw2).value;
	 	
		if(p1 === p2)
	 		var res = true;
	 	else
	 	{
			 var res = false;
			 
			 /* on vide les champs */
			 document.getElementById(pw1).value = "";
 			 document.getElementById(pw2).value = "";
 			 /* on notifie de l'erreur */
			 BoxFormError("Attention! les deux mots de passe sont diffÃ©rents",pw1);
	 	}
	
		return res;
 }

/**
 * Fonction qui verifie que les champs en parametre ne sont pas vides
 *
 * @author Fabien Poulard <grandscarabe@free.fr>
 * @version $Revision: 1048 $
 * @package gasell
 *
 * @param string_array fields les id des champs a verifier
 */
function CheckNoEmptyFields(fields)
{
	var res = true;

	if (fields.length >= 0)
	{
		/* On verifie chaque element texte pour savoir si il est vide ou non */
		var i = -1;
		while ( ( i  < fields.length ) && res )
		{
			i++;
			/* on verifie seulement les input de type texte */
			var f = document.getElementById(fields[i]);
			if ( f.type == "text" )
			{
				var txt = f.value;
				if ( txt == null || txt == undefined || txt == "") { res = false; }
			}
		}	
	
		/* des qu'une erreur est trouvee, on la gere */
		if (res != true) 	BoxFormError("Un des champs est vide !", fields[i]);
	}

	return res;
}

	
/**
 * Fonction qui verifie qu'aucun des champs est vide
 *
 * @author Fabien Poulard <grandscarabe@free.fr>
 * @version $Revision: 1048 $
 * @package gasell
 *
 * @param fname le nom du formulaire a analyser
 */
function CheckNoEmptyForm(fname)
{
	var res = true;
	var f = document.forms[fname];

	/* on verifie qu'il y a des elements sur lesquels travailler */
	if ( f.elements.length >= 0 )
	{
		/* On verifie chaque element texte pour savoir si il est vide ou non */
		var i = -1;
		while ( ( i  < f.elements.length ) && res )
		{
			i++;
			
			if (f.elements[i].type == "text")
			{
				var txt = f.elements[i].value;
				if ( txt == null || txt == undefined || txt == "") { res = false; }
			}
		}	

		/* des qu'une erreur est trouvee, on la gere */
		if (res != true) 	BoxFormError("Un des champs est vide !", f.elements[i].id);
	}

	return res;
}

/**
 * Permet changer l'etat d'un champ suivant l'etat d'une checkbox
 *
 * @author Damien Raude-Morvan <drazzib@drazzib.com>
 * @version $Revision: 1048 $
 * @package gasell
 *
 * @param fields 'array' : liste des champs pour lequels on doit changer l'etat
 * @param stateField 'string' : la checkbox qui va servir a trouver l'etat
 */
function switchFieldState(fields, stateField)
{
	/* Il y a des fields a traiter et le browser est compatible */
	if (fields != null && fields.length > 0 && compatible == true)
	 {
		/* Recuperation de la valeur du champ d'etat */
		var oStateField = document.getElementById(stateField);

		/* Pour chaque champ... */
		for (var i=0; i < fields.length; i++)
		 {
			/* On recupere le champ dans l'arbre DOM */
			var oField = document.getElementById(fields[i]);
			/* On verifie que le champ existe bien */
			if (oField != null)
			 {
				if (oField.disabled != undefined
					&& oStateField.checked != undefined)
				 {
					/* Permutation de l'etat du champ */
					oField.disabled = oStateField.checked;
				 }
			 }
		 }
	 }

	return true;
}