<?php
/*
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 *
 * Histoire
 *  * 14/01/05 Hooligan0 : Modifications des appels a la classe
 *                         Bdd pour utiliser l'objet Gasell global.
 *  * 19/06/05 DrazziB : Implementation des specifications de
 *                       la partie activite / participation
 */

/**
 * Classe Activite : Gestion des activites gerees par
 * l'association.
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @author Damien Raude-Morvan <drazzib@drazzib.com>
 * @since 0.6
 * @package gasell
 */
class Activite
{

	/** Identifiant de l'activite. */
	var $id;

	/** Libelle de l'activite */
	var $label;

	/** Date de debut de l'activite
	 * @type date
	 */
	var $startDate;

	/** Date de fin de l'activite
	 * @type date
	 */
	var $endDate;

	/** Bilan textuel de cette activite
	 * @type string
	 */
	var $bilan;

	/**
	 * Recuperation de la liste des Activites (depuis une racine).
	 *
	 * Par defaut, il y a recuperation de l'ensemble des activites
	 * de niveau 0 (sans parents).
	 *
	 * TODO Ajouter un parametre pour l'affichage des activites
	 * cloturees !
	 *
	 * @access public
	 * @type static
	 * @param $racine : Code de l'Activite racine de la liste.
	 */
	function GetListActivite($racine = NULL, $showAll = false)
	{
		global $Gasell;
		
		$query = "SELECT actCode, id_parent FROM activite";
		if (!$showAll)
		 {
			/* Les activites closes ne sont pas affichees */
			$query .= " WHERE (actEndDate > '".date("Y-m-d")."' OR actEndDate IS NULL)";
		 }
		if ($racine != NULL)
		 {
			if (!$showAll)
			 {
				$query .= " AND ";
			 }
			else
			 {
				$query .= " WHERE ";
			 }
			$query .= " id_parent='".$racine."'";
		 }
		$query .= " ORDER BY actCode";

		/* Envoi de la requete a la base de donnee */
		$result = $Gasell->db->Requette($query, "ALL");

		/* Si il existe des elements dans le resultat de la base */
		if (!empty($result))
		 {
			/* Creation du tableau des activites pour le retour */
			$tabActivites = NULL;
			$i = 0;
			foreach ($result as $tupleActivite)
			 {
				/* Pour chaque ligne de resultat on recupere le premier attribut */
				//echo $tupleActivite[0]."</br>";
				$tabActivites[$i] = new Activite($tupleActivite[0]);
				$i++;
			 }
 			return($tabActivites);
		 }
		else
		 {
			return(NULL);
		 }
	}

	/**
	 * Recuperation d'une Activite (en utilisant son code)
	 * depuis la base de donnees.
	 *
	 * @constructor
	 * @access public
	 * @param $actcode : Code de l'Activite en base de donnees (champ actCode)
	 */
	function Activite($actcode = 0)
	{
		global $Gasell;
		
		$query = "SELECT actCode, actLib, actStartDate, actEndDate, id_parent, actBilan FROM activite";
		$query .= " WHERE actCode='".$actcode."'";
		$result = $Gasell->db->Requette($query, "ALL");

		if (!empty($result))
		 {
			/* Affectation de l'identifiant de la participation */
			$this->id = $result[0][0];
			/* Affectation du type de la participation */
			$this->label = $result[0][1];
			/* Affectation des dates de l'Activite */
			$this->startDate = $result[0][2];
			$this->endDate = $result[0][3];
			/* Affectation du bilan de l'activite */
			$this->bilan = $result[0][5];

			return($this);
		 }
		else
		 {
			return(NULL);
		 }
	}
  
	/**
	 * Creation d'une nouvelle activite
	 *
	 * @access public
	 * @type static
	 * @param integer $base      : Code de l'activite parente
	 * @param string  $libelle   : Libelle de la nouvelle activite
	 * @param date    $startDate : Date de debut de l'activite
	 * @param date    $endDate   : Date de cloture de l'activite
	 */
	function Ajoute($parent_code, $libelle, $startDate = "", $endDate = "")
	{
		global $Gasell;

		$query  = "INSERT INTO activite(actLib, actStartDate, actEndDate, id_parent)";
		$query .= " VALUES('".$libelle."', ";
		/* Gestion de la partie SQL pour la date de debut de l'activite */
		if ($startDate != "")
		 {
			/* Une date de fin est fournie, on l'encadre par des '' */
			$query .= "'".$startDate."', ";
		 }
		else
		 {
			/* Aucune date de fin n'est fournie, on insert NULL */
			$query .= "NULL, ";
		 }
		/* Gestion de la partie SQL pour la date de fin de l'activite */
		if ($endDate != "")
		 {
			/* Une date de fin est fournie, on l'encadre par des '' */
			$query .= "'".$endDate."', ";
		 }
		else
		 {
			/* Aucune date de fin n'est fournie, on insert NULL */
			$query .= "NULL, ";
		 }
		$query .= "'".$parent_code."')";

		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * Suppression d'une activite
	 *
	 * WARN --> Preferer le changement de la date de cloture !
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de l'activite
	 */
	function Delete($code)
	{
		global $Gasell;

		/* On supprime d'abord les fils de l'activite */
		$listChilds = Activite::GetListActivite($code, true);
		if ($listChilds != NULL)
		 {
			foreach ($listChilds as $child)
			 {
				Activite::Delete($child->id);
			 }
		 }

		/* ... ensuite les participation liees */
		$listParticipations = Participation::ListParticipationForActivity($code);
		if ($listParticipations != NULL)
		 {
			foreach ($listParticipations as $participation)
			 {
				Participation::Delete($participation->id);
			 }
		 }

		$query  = "DELETE FROM activite";
		$query .= " WHERE actCode='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * Cloture d'une activite en changeant la date de cloture
	 * a la date courante.
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de l'activite
	 */
	function Cloturer($code)
	{
		global $Gasell;

		$endDate = date("Y-m-d");

		/* On desactive d'abord les fils de l'activite */
		$listChilds = Activite::GetListActivite($code, true);
		if ($listChilds != NULL)
		 {
			foreach ($listChilds as $child)
			 {
				Activite::Cloturer($child->id);
			 }
		 }

		/* ... ensuite les participation liees */
		$listParticipations = Participation::ListParticipationForActivity($code);
		if ($listParticipations != NULL)
		 {
			foreach ($listParticipations as $participation)
			 {
				Participation::Cloturer($participation->id);
			 }
		 }

		$query  = "UPDATE activite";
		$query .= " SET actEndDate='".$endDate."'";
		$query .= " WHERE actCode='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * Definit le texte du rapport de l'activite.
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de l'activite
	 * @param string  $report  : Texte du rapport de l'activite
	 */
	function SaveReport($code, $report)
	{
		global $Gasell;

		$query  = "UPDATE activite";
		$query .= " SET actBilan='".$report."'";
		$query .= " WHERE actCode='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}

}

/* EOF */
?>