<?php
/*
 * Gasell: Copyright 2004-2006 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors : Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *           Poulard Fabien (GrdScarabe) <grdscarabe@grdscarabe.net>
 */

/*
 * Historique :
 *    16/01/2005 : GrdScarabe : ajout d'un controle sur l'ajout d'un type
 *    d'adhesion deja enregistre ou vide
 */

include_once 'Date.php';

/**
 * Classe de gestion des types d'adhesion
 *
 * @version $Revision: 1154 $
 * @package gasell
 */
class TypeAdhesion
{
	/* Declaration des variables membre de la classe                     */
	var $num;
	var $exenum;
	var $libelle;

	/**
	 * Constructeur de la classe "TypeAdhesion"
	 *
	 * @access public
	 * @param integer $tadhnum : numero du type d'adhesion
	 */
	function TypeAdhesion($tadhnum = NULL)
	{
		global $Gasell;

		if ($tadhnum == NULL)
			return;

		/* Recherche du type d'adhesion dans la BDD                          */
		$query  = "SELECT typadhnum,typadhlib,exenum ";
		$query .= "FROM typeadhesion WHERE typadhnum='" . $tadhnum . "'";
		$result = $Gasell->db->Requette($query, "ALL");

		if (count($result) > 0)
		{
			$this->num     = $result[0][0];
			$this->libelle = $result[0][1];
			$this->exenum  = $result[0][2];
		}
	}

	/**
	 * Fonction de recherche de tous les types d'adhesion par exercice
	 *
	 * @access public
	 * @param  integer $exenum : Numero d'exercice de la recherche
	 * @return array           : Enssemble des types d'adhesion de l'exe
	 */
	function searchAll($exenum = NULL)
	{
		global $Gasell;
		/* Si aucun numero d'exercice particulier n'est precise en parametre */
		if (! $exenum)
		{
			/* Si l'objet courrant est deja ratache a un exercice            */
			if ($this->exenum != NULL)
			 {
				/* Alors, recherche dans cet exercice                        */
				$exenum = $this->exenum;
			 }
			else
			 {
				/* Sinon, recherche l'exercice courrant de l'asso            */
				$exenum = $Gasell->CfgGet("ASSO", "Exercice");
			 }
		}
		
		/* Recherche tous les types d'adhesion dans la BDD                   */
		$query  = "SELECT typadhnum,typadhlib ";
		$query .= "FROM typeadhesion WHERE exenum='".$exenum."'";
		$result = $Gasell->db->Requette($query, "ALL");
		/* Retour a l'appelant avec un tableau des types trouves             */
		return($result);
	}
  
	/**
	 * Creation d'un nouveau type d'adhesion
	 *
	 * @access public
	 * @param  string $libele : Libele du type a creer
	 * @return boolean        : Resultat de l'operation de creation
	 */
	function create($libele)
	{
		global $Gasell;
		
		/* Le libelle ne peut pas etre vide...une verification est faites plus */
		/* haut si jamais le libelle est tout de meme vide, c'est une erreur */
		if( empty($libele) )
		{
			/* Petit message d'erreur indiquant qu'on a voulu inserer un type */
			/* d'adhesion vide */
			echo "<div class=\"error\">";
			echo "Erreur: Une erreur inattendue s'est produite.<br>";
			echo "N'auriez vous pas appeler le script directement ?</div>";
			$ret = 0; /* erreur */
		}
		else
		{
			/* Si l'objet courant est deja lie a un exercice                    */
			if ($this->exenum != NULL)
			 {
				/* Alors, utilisation de cet exercice pour le nouveau type       */
				$exenum = $this->exenum;
			 }
			else
			 {
				/* Sinon, utilisation du numero d'exercice courant de l'asso    */
				$exenum = $this->Gasell->CfgGet("ASSO", "Exercice");
			 }
		
			/* On verifie que le type d'adhesion n'existe pas deja */
			$query = "SELECT COUNT(*) FROM typeadhesion ";
			$query .= "WHERE typadhlib='".$libele."' AND exenum='".$exenum."'";
			$result = $Gasell->db->Requette($query,"ALL");
		
			if( ($result != NULL) && (count($result) == 1) )
			{
				/* Si la requete SQL s'est bien deroulee */
				if($result[0][0] != 0)
				{
					/* Ce type d'adhesion existe deja...comment on gere ça ? */
// FIXME Gestion des erreurs
					echo "<div class=\"warning\">";
					echo "Attention! Type d'adh&eacute;sion existant!</div>";
					/* ce n'est pas une erreur non plus */
					$ret = 1;
				}
				else
				{
					/* Construction de la requette SQL et envoi a la BDD                 */
					$query  = "INSERT INTO typeadhesion(typadhlib,exenum) ";
					$query .= "VALUES('".$libele."','".$exenum."')";
					$result = $Gasell->db->Requette($query);
		
					if ($result != NULL)
						/* resultat OK */
		  			$ret = 1;
		  		else
		  			/* resultat KO */
		  			$ret = 0;
		  	}
		  }
		  else
		  {
		  	/* Un probleme durant la requête ? */
// FIXME Gestion des erreurs
				echo "<div class=\"error\">";
				echo "Erreur durant la requ&ecirc;te d'ajout de l'adh&eacute;rent.";
				echo "</div>";
		  	$ret = 0;
		  }
			
		  return $ret;
		}
	}
  
	/**
	 * Suppression d'un type d'adhesion
	 *
	 * @access public
	 * @param  ???????????????
	 */
	function Supprime($num)
	{
		global $Gasell;
		if ($this->exenum != NULL)
		 {
			$exenum = $this->exenum;
		 }
		else
		 {
			$exenum = $this->Gasell->CfgGet("ASSO", "Exercice");
		 }

		$query  = "DELETE FROM typeadhesion ";
		$query .= "WHERE exenum='".$exenum."' AND typadhnum='".$num."'";
		$result = $Gasell->db->Requette($query);
		return($result);
	}

}

/**
 * Classe de gestion des reglements
 *
 * @version $Revision: 1154 $
 * @package gasell
 */
class Reglement
{
	/* Declaration des variables membre de la classe                           */
	var $Gasell;
	var $regnum;
	var $date;
	var $montant;

	/**
	 * Constructeur : Cree un objet gerant un Reglement
 	 *
 	 * @param integer $reg : Numero de reglement a charger (optionnel)
 	 */
	function Reglement($regnum = NULL)
	{
		global $Gasell;
		/* Si aucun numero de reglement n'est precise au constructeur*/
		if ($regnum == NULL)
			/* Alors, plus de traitements, retour appelant       */
			return;
		
		/* Construction d'une requette d'extraction du reglement     */
		$query  = "SELECT regdatech,regmont,regvalid ";
		$query .= "FROM reglement WHERE regnum='".$regnum."'";
		/* Recherche dans la BDD a partir de cette requette          */
		$dbresult = $Gasell->db->Requette($query, "ALL");
		/* Si le nombre d'enregistrements resultat est nul           */
	
		if (count($dbresult) == 0)
			/* Alors, plus de traitement, retour appelant        */
			return;
		
		/* Renseigne les car membres a partir des donnees lues en bdd*/
		$this->regnum  = $regnum;
		$this->date    = new Date($dbresult[0][0]);
		$this->valide  = $dbresult[0][2];
		$this->montant = $dbresult[0][1];
	}

	/**
	 * Ajoute : Cree un nouveau reglement
	 *
	 * @param  integer   $adhnum  : Numero de l'adhesion qui recoit un reglement
	 * @param  integer   $montant : Montant percu pour ce reglement
	 * @param  date      $date    : Date de l'operation de reglement
	 * @return booleen : NULL si l'operation est reussie
	 */
	function Ajoute($adhnum, $montant, $date)
	{
		global $Gasell;
		$q_cols = "adhnum,regmont,regdatech";
		$q_vals = "'".$adhnum."','".$montant."','".$date."'";
		$query = "INSERT INTO reglement(".$q_cols.") VALUES(".$q_vals.")";
		$dbresult = $Gasell->db->Requette($query);
		return($dbresult);
	}

	/**
	 * ListeParAdhesion : Rechercher tous les reglements relatifs a une adhesion
	 *
	 * @return tab_integer : Tableau contenant les numeros de reglement
	 */
	function ListeParAdhesion($adhnum)
	{
		global $Gasell;
		$query = "SELECT regnum FROM reglement WHERE adhnum='".$adhnum."' ";
		$query .= "ORDER BY regdatech";
		$dbresult = $Gasell->db->Requette($query, "ALL");
		return($dbresult);
	}

	/**
	 * Valide : Marque un reglement comme percu
	 *
	 */
	function Valide()
	{
		$query = "UPDATE reglement SET regvalid='1' ";
		$query .= "WHERE regnum='" . $this->regnum . "'";
		$dbresult = $GLOBALS['GGasell']->db->Requette($query);
		return($dbresult);
	}
}

/**
 * Classe de gestion des adhesions
 *
 * @version $Revision: 1154 $
 * @package gasell
 */
class Adhesion
{
	/* Declaration des variables membre                                        */
	var $adhNum;     /* Numero de l'adhesion                       */
	var $memNum;     /* Numero de la personne a qui est l'adhesion */
	var $adhDat;     /* Date du debut de l'adhesion                */
	var $typAdhNum;  /* Type d'adhesion                            */
	var $adhMontant; /* Montant de l'adhesion                      */
	var $reglements; /* Tableau des reglements de l'adhesion       */

	/**
	 * Constructeur d'un objet representant une Adhesion
	 *
	 * @param integer $adhnum : Numero de l'adhesion a charger (opt)
	 * @param integer $memnum : Numero de la personne liee a l'adhesion (opt)
	 */
	function Adhesion($adhnum = NULL, $memnum = NULL)
	{
		global $Gasell;
    
		/* Creation d'un tableau (vide) des reglements de cette adhesion         */
		$this->reglements = array();
    
		/* Si un numero de personne a ete passe en argument au constructeur      */
		if ($memnum != NULL)
			/* Alors, sauvegarde de ce numero dans l'objet                         */
			$this->memNum = $memnum;
    
		/* Si aucun numero d'adhesion n'est precise                              */
		if (!$adhnum)
			/* Alors, il n'y a rien a initialiser, retour immediat a l'appelant    */
			return;
    
		/* Sauvegarde le numero d'adhesion dans l'objet                          */
		$this->adhNum = $adhnum;
		$q_cols = "adhnum,memnum,adhdate,typadhnum,adhmontant";
		$query = "SELECT ".$q_cols." FROM adhesion WHERE adhnum='".$adhnum."'";
		$dbresult = $Gasell->db->Requette($query, "ALL");
    
		$this->memNum     = $dbresult[0][1];
		$this->adhDat     = $dbresult[0][2];
		$this->typAdhNum  = $dbresult[0][3];
		$this->adhMontant = $dbresult[0][4];
    
		// TODO: charger les reglements de l'adhesion
	}
  
	function LibeleDuType()
	{
		global $Gasell;
    
		$query = "SELECT typadhlib FROM typeadhesion ";
		$query .= "WHERE typadhnum='".$this->typAdhNum."'";
		$dbresult = $Gasell->db->Requette($query, "ALL");
		return($dbresult[0][0]);
	}

	function ExerciceDuType()
	{
		$query = "SELECT exenum FROM typeadhesion ";
		$query .= "WHERE typadhnum='".$this->typAdhNum."'";
		$dbresult = $GLOBALS['GGasell']->db->Requette($query, "ALL");
		return($dbresult[0][0]);
	}

	/**
	 * Ajoute : Cree une nouvelle adhesion
	 *
	 * @param integer $memnum  : Numero de personne a qui l'adhesion est ajoutee
	 * @param date    $adhdate : Date de debut de l'adhesion
	 * @param integer $typeadhnum : Code du type d'adhesion
	 * @param integer $montant : Montant de la cotisation pour l'adhesion
	 */
	function Ajoute($memnum, $adhdate, $typeadhnum, $montant)
	{
		$isPremiere = count(Adhesion::ListeByMemnum($memnum));

		$date = new Date($adhdate);
		/* Construction de la requette a partir du numero de personne            */
		$q_cols = "memnum";
		$q_vals = "'" . $memnum . "'";
		/* Ajout du type d'adhesion a la requette                                */
		$q_cols .= ",typadhnum";
		$q_vals .= ",'" . $typeadhnum . "'";
		/* Ajout de la date de debut de l'adhesion a la requette                 */
		$q_cols .= ",adhdate";
		$q_vals .= ",'" . $adhdate . "'";
		/* Ajout du montant de cotisation a la requette                          */
		$q_cols .= ",adhmontant";
		$q_vals .= ",'" . $montant . "'";

		/* */
		$query = "INSERT INTO adhesion(".$q_cols.") VALUES (" . $q_vals . ")";
		$ret = $GLOBALS['GGasell']->db->Requette($query);

		/* Mise a jour de la date de premiere adhesion */
		if ($isPremiere == 0)
		{
			$query = "UPDATE personne SET memdatadh1='".$adhdate."' ";
			$query .= "WHERE memnum='".$memnum."'";
      $GLOBALS['GGasell']->db->Requette($query);
		}
		return($ret);
	}
  
	function Reglements()
	{
		/* Creation du tableau des resultats (pour eviter de renvoyer NULL)      */
		$reglements = array();
		/* Recherche la liste des reglements pour cette adhesion                 */
		$regnums = Reglement::ListeParAdhesion($this->adhNum);
		/* Pour chaque reglement lie a cette adhesion                            */
		foreach($regnums as $regnum)
		{
			/* Creation d'un objet Reglement pour le reglement trouve              */
			$reglement = new Reglement($regnum[0]);
			/* Ajoute l'objet reglement dans le tableau des resultats renvoyes     */
			array_push($reglements, $reglement);
		}
		/* Renvoi le tableau des objets reglements pour cette adhesion           */
		return($reglements);
	}

	function addReglement()
	{
		$newReglement = new Reglement;
		array_push($this->reglements, $newReglement);
		return($newReglement);
	}
  
	function CotisationAJour()
	{
		$TotalReglements = 0;
		$reglements = $this->Reglements();
		foreach($reglements as $reglement)
		{
			if ($reglement->valide != "1")
			{
				if (Date::compare($reglement->date, new Date()) == -1)
					return(0);
			}
			$TotalReglements += $reglement->montant;
		}
    
		if ($TotalReglements < $this->adhMontant)
			return(0);
		
		return(1);
	}

	function ListeByMemnum($memnum)
	{
		$query = "SELECT adhnum FROM adhesion WHERE memnum='".$memnum."'";
		$result = $GLOBALS['GGasell']->db->Requette($query, "ALL");
		return($result);
	}

	function ChercheParExercice($exenum)
	{
		$q_cont  = "adhesion.typadhnum=typeadhesion.typadhnum";
		$q_cont .= " AND typeadhesion.exenum='" . $exenum . "'";
		$query = "SELECT adhnum FROM adhesion,typeadhesion WHERE ".$q_cont;
		
		// TODO: Fonction non terminee, interet ? a finir ?
	}
}
/* EOF */
?>
