<?php
/*
 * Gasell: Copyright 2004-2006 Linux-Nantes Association <contact@linux-nantes.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 *
 * Histoire:
 *  * 25/06/2006 DrazziB : lancement de la methode de connection
 *       a la fin du constructeur de Bdd afin d'etre informe tres tot
 *       des problemes de connections a la base.
 *  * 09/06/2006 DrazziB : _escapeString() qui permet d'echapper les chaines
 *       suivant la configuration actuelle de PHP.
 *  * 10/01/2006 DrazziB : Correction pour compatibilite PHP5 (la methode
 *      get_class() est desormais sensible a la casse).
 *  * 20/10/2005 Hooligan0 : Ajout du necessaire pour appeler une base mysql
 *      Ce support ne garanti pas que l'appli soit compatible avec mysql !!
 *  * 18/10/2005 Hooligan0 : Abandon du module MDB, utilisation des fonctions
 *      postgresql natives de php.
 *  * 14/07/2005 Hooligan0 : Deplacement des fonctions "Config*" vers la classe
 *      Gasell. Correction de la methode Select.
 */

require_once GASELL_BASE . '/lib/Exceptions.php';

define('ERR_BDD_connect_01',          10101);
define('ERR_BDD_TableCreate_01',      10301);
define('ERR_BDD_TableDrop_01',        10401);
define('ERR_BDD_TableAddCol_01',      10501);
define('ERR_BDD_TableDropCol_01',     10601);

/**
 * Gasell Bdd Class.
 *
 * Cette classe permet l'acces aux donnees presentes dans le SGBD
 * @version $Revision: 1157 $
 * @package gasell
 */
class Bdd
{
	var $dbType;
	var $dbUser;
	var $dbPass;
	var $dbHost;
	var $dbDb;
	var $dbPrefix;
	var $dbLink;
	var $isConnected;
	
	/**
	 * Constructeur de la classe Bdd
	 *
	 */
	function Bdd()
	{
		global $conf;
		$this->dbType = $conf['bdd']['type'];
		$this->dbUser = $conf['bdd']['user'];
		$this->dbPass = $conf['bdd']['pass'];
		$this->dbHost = $conf['bdd']['host'];
		$this->dbDb = $conf['bdd']['db'];

		$this->isConnected = FALSE;

		/* Lancement de la connection apres parametrage */
		$this->_connect();
	}
	
	function reconnectDB($name)
	{
		$this->dbDb = $name;
		$this->dbLink = NULL;
		$this->dbLink = $this->_connect();
	}
	
	/**
	 * Connection a la base de donnee
	 *
	 * @access public
	 *
	 * @return Identifiant de la ressource
	 */
	function _connect($report = 1)
	{
		global $conf;
		global $Gasell;

		/* Dans PHP5, la fonction get_class respecte maintenant la casse du nom de la classe 
		   on utilise donc la fonction strtolower() afin de s'assurer que la comparaison
		   fonctionne sous PHP4 et PHP5 */
		if ((! isset($this)) || (strtolower(get_class($this)) != 'bdd'))
			return;

		if ($this->dbLink)
			return($this->dbLink);
		
		switch ($this->dbType)
		{
			case "pgsql": $this->_connect_pgsql(); break;
			case "mysql": $this->_connect_mysql(); break;
		}

		/* Si une erreur est survenue                                        */
		if (! $this->dbLink)
		{
			if ($report == 1)
			  {
				/* Alors, arret des traitements et affichage 
					d'une erreur    */
				$msg = "Erreur de connexion &agrave; la base ";
				$msg .= "de donn&eacute;es <br/> Merci de vous";
				$msg .= " assurer que le serveur de la base de";
				$msg .= " donn&eacute;es est accessible et ";
				$msg .= "que l'application est ";
				$msg .= "autoris&eacute;e &agrave; s'y ";
				$msg .= "connecter.";
				Except::throwFailure("BDD", $msg);
			  }
			else
				return(NULL);
		}
		$Gasell->dblink    = $this->dbLink;
		$this->isConnected = TRUE;

		/* Retourne l'id de la ressource de la connection                */
		return($this->dbLink);
	}

	function _connect_pgsql()
	{
		$connHost = "host="     . $this->dbHost . " ";
		$connUser = "user="     . $this->dbUser . " ";
		$connPass = "password=" . $this->dbPass . " ";
		$conndb   = "dbname="   . $this->dbDb   . " ";

		$connString = $connHost . $connUser . $connPass . $conndb;

		/* Connection a la base                                              */
		$db = @pg_connect($connString);
		/* Si une erreur est survenue                                        */
		if (! $db)
			return(NULL);

		// Set the client encoding to UNICODE.  Data will be automatically
		// converted from the backend encoding to the frontend.
		@pg_set_client_encoding($db, 'LATIN9');

		$this->dbLink      = $db;

		/* Retourne l'id de la ressource de la connection                */
		return($db);
	}

	function _connect_mysql()
	{
		$db = @ mysql_connect($this->dbHost, $this->dbUser, $this->dbPass);
		if (! $db)
			return(NULL);

		mysql_select_db($this->dbDb);

		$this->dbLink = $db;
		return($db);
	}

	function TableDrop($nom)
	{
		$db = $this->_connect();
		/* Construction de la requette SQL de creation de la table           */
		$query = 'DROP TABLE ' . $nom;
		$result = @pg_query($query);
		if (! $result)
			/* Alors, appel le module de gestion d'erreur                    */
			Except::throwError("BDD", "Code ".ERR_BDD_TableDrop_01);
		return;
	}
  
	function TableAddCol($table, $newcol, $type)
	{
		$db = $this->_connect();
		$query  = 'ALTER TABLE ' . $table;
		$query .= ' ADD COLUMN ' . $newcol . ' ' . $type;
		$result = $this->Requette($query);
		return;
	}
  
	function TableDropCol($table, $col)
	{
		$db = $this->_connect();
		$query  = 'ALTER TABLE ' . $table;
		$query .= ' DROP COLUMN ' . $col;
		$result = @pg_query($query);
		if (! $result)
			/* Alors, appel le module de gestion d'erreur                    */
			Except::throwError("BDD", "Code ".ERR_BDD_TableDropCol_01);
		return;
	}

	/**
	 * Fonction pour realiser une requette SQL quelconque sur la base
	 *
	 * @param string $requette : Requette SQL a executer
	 * @param string $type     : La requette ou non un resultat
	 */
	function Requette($requette, $type='')
	{

		$db = $this->_connect();
		switch($this->dbType)
		{
			case "pgsql": $result = pg_query($requette); break;
			case "mysql": $result = mysql_query($requette); break;
		}

		if ($type == '')
			/* return a boolean */
			return ($result === false) ? false : true;
		else if ($type == 'ALL')
		{
			if (! $result)
				return(NULL);
			else
			{
				$glbR = array();
				switch($this->dbType)
				{
					case "pgsql":
						while($row = pg_fetch_row($result))
							array_push($glbR, $row);
						break;
					case "mysql":
						while($row = mysql_fetch_row($result))
							array_push($glbR, $row);
						break;
				}
				return($glbR);
			}
		}
		return(NULL);
	}

	/**
	 * Fonction de selection de donnees depuis la base
	 *
	 * @access public
	 * @param  string $table : Nom de la table dans laquelle realiser la selection
	 * @param  string $cols  : Tableau des attributs a selectionner
	 * @param  string $contraintes : Tableau des arguments complementaires
	 */
	function Select($table, $cols = '*', $contraintes = NULL)
	{
		/* Connexion a la base                                               */
		$db = $this->_connect();

		/* Si aucune liste d'attributs n'a ete precise                       */
		if ($cols == "*")
			/* Alors, prise en compte de tous les attributs                  */
			$q_cols = "*";
		/* Sinon, un tableau des attributs a ete passe en argument           */
		else
		{
			$q_cols = "";
			$flagContinue = 0;
			/* Boucle d'ajout de chacun des attributs dans une chaine        */
			foreach($cols as $col)
			{
				/* Si plusieurs attributs sont places dans la chaine         */
				if ($flagContinue)
					/* Alors, il faut ajouter des separateurs                */
					$q_cols .= ",";
				/* Sauf pour le premier :)                                   */
				else
					$flagContinue  = 1;

				$q_cols .= $col;
			}
		}

		/* Construction de la requete de selection                           */
		$query = "SELECT  " . $q_cols . " FROM " . $table;

		/* S'il existe des contraintes a appliquer                           */
		if ($contraintes != NULL)
		{
			$query .= " WHERE ";

			$flagContinue = 0;
			/* Boucle d'ajout des contraintes dans une chaine                */
			foreach($contraintes as $contrainte)
			{
				if ($flagContinue ==   1)
					$query .= " ";
				else
					$flagContinue = 1;
				$query .= $contrainte;
			}
		}

		/* Envoi de la requete a la base                                     */
		switch($this->dbType)
		{
			case "pgsql": $resultDB = @pg_query($query); break;
			case "mysql": $resultDB = mysql_query($query); break;
		}

		/* Verification de la bonne execution                                */
		if ($resultDB == false)
			return(NULL);
		else
		{
			$result = array();
			switch ($this->dbType)
			{
				case "pgsql":
					if (pg_num_rows($resultDB) > 0)
					{
						/* Lecture du resultat et renvoi a l'appelant  */
						while($row = pg_fetch_row($resultDB))
							array_push($result, $row);
					}
					break;

				case "mysql":
					if (mysql_num_rows($resultDB) > 0)
					{
						while($row = mysql_fetch_row($resultDB))
							array_push($result, $row);
					}
			}
			return($result);
		}
	}

	/**
	 * Fonction d'ajout de donnees dans la base
	 *
	 * @access public
	 * @param  string $table : Nom de la table dans laquelle realiser l'ajout
	 * @param  string $cols  : Les colones a renseigner
	 * @param  string $id    : the serial field we want
	 * @param  tab_string
	 */
	function Insert($table, $valeurs, $id=NULL)
	{
		if (! isset($this))
			die("MAIN ERROR - Bdd Insert without object");

		if($id != NULL)
		{
			/* prepare the id of the sequence */
			$q = "SELECT nextval('".$table."_".$id."_seq');";
			$res = pg_query($q);
			$id_inserted = pg_fetch_result($res, 0, 0);
		}

		/* Initialisation de la liste des colones et des valeurs             */
		if($id != NULL)
		{
			$q_cols = "$id";
			$q_vals = "$id_inserted";
		} else {
			$q_cols = "";
			$q_vals = "";
		}
    
		/* Pour chacune des valeurs a prendre en compte dans l'ajout         */
		foreach($valeurs as $clef => $val)
		{
			/* Si l'option "Magic Quotes" de php n'est pas activee           */
			//if (get_magic_quotes_gpc() != 1)
			//	/* Alors, convertion du format (sequences d'echapement)      */
			//	$val = addslashes($val);

			/* Ajout du nom de la colone dans la requette                    */
			if ($q_cols == "")
				$q_cols = $clef;
			else
				$q_cols .= "," . $clef;

			/* Ajout de la valeurs de cette colone dans la requette          */
			if ($q_vals == "")
				$q_vals = "'" . Bdd::_escapeString($val) . "'";
			else
				$q_vals .= ",'" . Bdd::_escapeString($val) . "'";
		}
		/* Construction de la requette SQL apartir des colones et valeurs    */
		$query = "INSERT INTO " . $table . "(" . $q_cols . ") ";
		$query .= "VALUES (" . $q_vals . ")";
		/* Envoi de la requette SQL au moteur de la base                     */
		//$result = Bdd::Requette($query);
		if ($this->isConnected != TRUE)
			$this->_connect();

		$result = pg_query($query);	

		if($id != NULL)
			return $id_inserted;
		else
			return ($result === false) ? false : true;
	}

	function TableCreate($nom, $cols)
	{
		$db = Bdd::_connect();
		
		/* Construction de la requette SQL de creation de la table           */
		$query = 'CREATE TABLE ' . $nom . ' (';
		/* Mise a zero du compteur de colones de la table                    */
		$nbparam = 0;
		/* Pour chaque colone a creer                                        */
		foreach($cols as $col)
		{
			/* Si le nombre de colone deja specifie est non nul              */
			if ($nbparam > 0)
				/* Alors, il faut ajouter un separateur entre les definitions*/
				$query = $query . ', ';
			/* Ajoute une colone dans la requette SQL de creation de la table*/
			$query = $query . $col[0] . ' ' . $col[1];
			/* Incremente le compteur de colones                             */
			$nbparam++;
		}
		/* La requette SQL est fermee apres la declaration des colones       */
		$query = $query . ')';

		$result = $this->Requette($query);
		return(0);
	}

	/***************************************************************
	 * @method Update                                              *
	 *                                                             *
	 * @brief update a value in a table                            *
	 *                                                             *
	 * @param $table : le nom de la table a mettre a jour          *
	 * @param $col   : tableau colonnes/valeurs a mettre a jour    *
	 * @param $const : tableau des contraintes                     *
	 *                                                             *
	 * @result bool : booleen indiquant si tout s'est bien passe   *
	 ***************************************************************/
	function Update($table, $cols, $const)
	{
		$sql = "UPDATE $table SET ";

		/* Add the columns data */
		$first = true;
		foreach($cols as $name => $value)
		{
			/* hack pour proteger les dates */
			if(preg_match('/^\'(.+)\'$/', $value, $m))
			{
				$val = "'".Bdd::_escapeString($m[1])."'";
			} else if (is_numeric($value)) {
				$val = Bdd::_escapeString($value);
			} else if (is_null($value)) {
				$val = "NULL";
			} else {
				$val = "'".Bdd::_escapeString($value)."'";
			}
		
			if($first)
			{
				$sql .= $name.'='.$val;
				$first = false;
			} else
				$sql .= ', '.$name.'='.$val;
		}

		/* Add the constraint data */
		$first = true;
		foreach($const as $val)
		{
			if($first)
			{
				$sql .= ' WHERE '.$val;
				$first = false;
			} else
				$sql .= ' '.$val;
		}
		
		/* Envoi de la requette SQL au moteur de la base                     */
		//$result = Bdd::Requette($query);
		if ($this->isConnected != TRUE)
			$this->_connect();

		$result = pg_query($sql);	

		// FIXME : check the request is ok
		return true;
	}

	/**
	 * @param string $string
	 */
	function _escapeString($string)
	{
		/* Par defaut, on retourne la chaine elle-mme */
		$return = $string;

		if (!get_magic_quotes_gpc())
		{
			switch ($this->dbType)
			{
				case "pgsql": $return = pg_escape_string($string); break;
				case "mysql": $return = stripslashes($string); break;
			}
		}

		return $return;
	}
	
 }
/* EOF */
?>
