<?php
/*
 * Gasell: Copyright 2004-2006 Linux-Nantes Association <contact@linux-nantes.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Poulard Fabien (GrdScarabe) <grdscarabe@free.fr>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 */

/* Historique :
 *    * 10/01/2006 GrdScarabe : Insertion des methodes de manipulations des comptes
 *       en provenance de la classe Gasell.
 *    * 08/01/2006 GrdScarabe : retrait du mot Horde et changement 
 *       du nom de fichier (suppression Horde)
 *    * 05/11/2004 Hooligan0 : Creation du fichier.
 */

@define('COMPTESGASELL_status_EMPTY',   0);
@define('COMPTESGASELL_status_VALID',   1);
@define('COMPTESGASELL_status_INVALID', 4);
@define('COMPTESGASELL_status_CHANGED', 2);
@define('COMPTESGASELL_status_NEW',     3);

/**
 * Classe de gestion des comptes de connexion a Gasell
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 980 $
 * @package gasell
 */
class ComptesGasell
{
	var $uid    = 0;
	var $memnum = NULL;
	var $status = COMPTESGASELL_status_EMPTY;

	/****************************************************
	 * @method ComptesGasell                            *
	 *                                                  *
	 * @brief constructeur de "ComptesGasell"           *
	 *                                                  *
	 * @param $uid    : identifiant du compte           *
	 * @param $memnum : id du membre                    *
	 *                                                  *
	 * @access public                                   *
	 ****************************************************/
	 function ComptesGasell($uid = NULL, $memnum = NULL)
	 {
	 	global $Gasell;

		/* Si aucun numero de personne n'a ete precise au constructeur       */
		if (($uid == NULL) && ($memnum == NULL))
			/* Alors, plus de traitements, retour a l'appelant               */
			return;

		$db = new Bdd;
		
		// prepare the columns to be selected
		$sels  = array( 'user_uid', 'level' );
		$const = array( 'memnum=\''.$memnum.'\'' );

		// add the asso if known
		// FIXME : not portable for other database but pg_escape_string is 
		// necessary to prevent a bug with asso with special chars
		if ($Gasell->user['asso'] != "")
			array_push($const, 'AND asso=\''.pg_escape_string($Gasell->user['asso']).'\'');

		$resultat = $db->Select('comptes_gasell', $sels, $const);

		if (count($resultat) == 1)
		{
    			$this->uid = $resultat[0][0];
    			$this->status = COMPTESGASELL_status_VALID;
		} else {
			$this->status = COMPTESGASELL_status_INVALID;
		}
	}
  
  /**
   * Acesseur de la variable membre "memnum" pour sa mise a jour.
   *
   * @access public
   */
  function setPersonne($memnum)
   {
    /* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
    if (! is_object($this))
      /* Alors, le traitement est impossible, retour a l'appelant (+err)    */
      return(0);
    
    /* Mise a jour du numero de personne pour l'objet courant */
    $this->memnum = $memnum;
    /* Si le status de l'objet indique qu'une personne etait deja chargee   */
    if (($this->status == COMPTESGASELL_status_VALID) ||
        ($this->status == COMPTESGASELL_status_CHANGED))
      /* Alors, l'objet passe en etat "personne modifiee"                   */
      $this->status = COMPTESGASELL_status_CHANGED;
    /* Sinon */
    else
      /* L'objet passe en etat "nouvelle personne" (non sauvegardee)        */
      $this->status = COMPTESGASELL_status_NEW;
    
    /* Retour a l'appelant avec le status de l'objet                        */
    return($this->status);
   }
  
  /**
   * Acesseur de la variable de status de l'objet (lecture)
   *
   * @access public
   */
  function getStatus()
   {
    /* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
    if (! is_object($this))
      /* Alors, le traitement est impossible, retour a l'appelant (+err)    */
      return(0);
    
    return($this->status);
   }
  
  /**
   * Accesseur pour l'attribut uid de l'objet
   *
   * @access public
   */
  function getUID()
   {
    /* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
    if (! is_object($this))
      /* Alors, le traitement est impossible, retour a l'appelant (+err)    */
      return(0);
    
    return($this->uid);
   }
  
  /**
   * Recherche de compte par numero de personne
   *
   * @access public
   */
  function search_memnum($memnum)
   {
    global $Gasell;
    
    // prepare the query
    $cols  = array( 'user_uid','level' );
    $const = array( 'memnum='.$memnum );
    $resultat = $Gasell->db->Select('comptes_gasell', $cols, $const);
    if (count($resultat) > 0)
      $status = COMPTESGASELL_status_VALID;
    else
      $status = COMPTESGASELL_status_EMPTY;
    
    return($status);
   }

   /**
    * @method ComptesGasell::listAll
    *
    * @brief fonction qui permet de lister les comptes
    *  d'utilisateurs pour une asso donnee ou non
    *
    * @noparam
    *
    * @result un tableau indexe avec la liste des comptes
    *  avec les indexes :
    *    - user_uid
    *    - memnum
    *    - level
    **/
   function listAll()
   {
     global $Gasell;
     
     $db = new Bdd();
     
     /* prepare la liste des colonnes */
     $cols = array(
		   "user_uid",
		   "memnum",
		   "level");

     /* get the list of users */
     if ($Gasell->user['asso'] != "")
       $resultat = $db->Select("comptes_gasell", $cols, 
			       array("asso='" . $Gasell->user['asso'] . "'"));
     else
       $resultat = $db->Select("comptes_gasell", $cols);
     
     return($resultat);
   }

	/**
	 * comptes_ajoute : Creation d'une nouvelle entree dans les comptes
	 *
	 * @access public
	 * @param string $nom    Nom/login du compte
	 * @param int    $level  Niveau de droits a appliquer dans Gasell
	 * @param int    $memnum Numero de personne
	 * @param string $pass   Mot de passe de connexion
	 */
	function comptes_ajoute($nom, $level, $memnum, $pass)
	{
		global $conf, $Gasell;

		/* Sauvegarde la configuration actuelle                              */
		$conf_save = $conf;
		/* Rechargement de la configuration (pour les parametres de bdd)     */
		include GASELL_BASE . '/config/conf.php';

		$db = new Bdd();
		
		// prepare the query
		$q_cols = array( 
				'user_uid' => $nom,
				'level'    => $level);
		if ($memnum != NULL)
		{
		  $q_cols['memnum'] = $memnum;
		}
		$q_cols['db']   = $Gasell->user['db'];
		$q_cols['asso'] = $Gasell->user['asso'];

		// launch the request
		$db->Insert('comptes_gasell', $q_cols);
    
		unset($q_cols);

		// prepare the new request
		$q_cols['user_uid']  = $nom;
		$q_cols['user_pass'] = md5($pass);

		// launch the request
		$db->Insert('gasell_users', $q_cols);

		/* Restauration de la configuration                                  */
		$conf = $conf_save;
	}


	/**
	 * comptes_chpass : Change le mot de passe d'un compte
	 *
	 * @access public
	 * @param string $nom    Nom/login du compte
	 * @param string $pass   Nouveau mot de passe de connexion
	 */
	function comptes_chpass($nom, $pass)
	{
		global $conf;
		/* Sauvegarde la configuration actuelle                              */
		$conf_save = $conf;
		/* Rechargement de la configuration (pour les parametres de bdd)     */
		include GASELL_BASE . '/config/conf.php';

		$db = new Bdd;
		$query = "UPDATE gasell_users SET user_pass='".md5($pass)."' WHERE user_uid='".$nom."'";
		$db->Requette($query);

		/* Restauration de la configuration                                  */
		$conf = $conf_save;
	}

	/**
	 * Recherche la liste des groupes d'utilisateurs
	 *
	 * FIXME Table groupesusers inexistante !
	 *
	 * @access public
	 * @return tab : Tableau des groupes trouves et des utilisateurs inscrits
	 */
	function groupes()
	{
		global $Gasell;

		/* Initialisation de la variable resultat avec un tableau vide       */
		$groupes = array();

		/* Recherche dans la base tous les groupes                           */
		$q_cols = array("grpnum", "nom", "description");
		$dbresult = $Gasell->db->Select("groupes", $q_cols);

		/* Si l'enssemble resultat est vide                                  */
		if ($dbresult == NULL)
			/* Alors, retour immediat a la fonction appelante avec un tableau vide*/
			return($groupes);

		/* Pour chaque groupe d'utilisateur existant pour l'asso             */
		foreach($dbresult as $resultat)
		{
			$grpnum = $resultat[0];
			/* Ajout du nom de ce groupe dans l'enssemble resultat          */
			$groupes[$grpnum]['nom']  = $resultat[1];
			/* Ajout de la description du groupe dans l'enssemble resultat  */
			$groupes[$grpnum]['desc'] = $resultat[2];
			/* Ajout du numero identifiant du groupe dans l'enssemble resultat */
			$groupes[$grpnum]['num']  = $grpnum;

			/* Recherche dans la base les utilisateurs inscrits dans ce groupe */
			$q_cols = array("user_uid");
			$q_cont = array("grpnum='" . $grpnum . "'");
			$dbresult2 = $Gasell->db->Select("groupesusers", $q_cols, $q_cont);

			/* Initialisation du tableau des utilisateurs du groupe         */
			$groupes[$grpnum]['users'] = array();
			/* Pour chaque utilisateur reconnus comme apartenant a ce groupe*/
			foreach($dbresult2 as $user)
				/* Ajout l'identifiant de l'utilisateur dans le resultat    */
				$groupes[$grpnum]['users'][$user[0]] = "1";
		}
		/* Retourne l'enssemble resultat de la recherche a la fonction appelante*/
		return($groupes);
	}

	/**
	 * FIXME Table groupesusers inexistante !
	 */
	function groupes_ajouter($nom, $desc)
	{
		global $Gasell;

		// prepare the query
		$cols['nom']         = $nom;
		$cols['description'] = $desc;
		$Gasell->db->Insert('groupes', $cols);
	}

	/**
	 * FIXME Table groupesusers inexistante !
	 */
	function groupes_NumParNom($nom)
	{
		global $Gasell;

		$q_cols = array("grpnum");
		$q_cont = array("nom='".$nom."'");
		$dbresult = $Gasell->db->Select("groupes", $q_cols, $q_cols);
		if ($dbresult == NULL)
			return(NULL);
		return($dbresult[0][0]);
	}

	/**
	 * FIXME Table groupesusers inexistante !
	 */
	function groupes_insert($groupe, $user)
	{
		global $Gasell;

		$vals['grpnum'] = ComptesGasell::groupes_NumParNom($groupe);
		$vals['user_uid'] = $user;
		$Gasell->db->Insert("groupesusers", $vals);
	}

	/**
	 * FIXME Table groupesusers inexistante !
	 */
	function groupes_suppr($groupe, $user)
	{
		global $Gasell;

		$query = "DELETE FROM groupesusers WHERE user_uid='".$user."'";
		$Gasell->db->Requette($query);
	}

}
/* EOF */
?>
