<?php
/*
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *
 * Histoire:
 *  * 28/01/05 Hooligan0 La fonction ExerciceGet est deplacee de la classe Bdd
 *     vers cette classe Exercice.
 *  * 21/01/05 Hooligan0 Correction de la fonction d'ajout d'un exercice pour
 *     remplacer l'appel d'une fonction dediee dans la classe Bdd en une
 *     requette SQL ici.
 */

include_once "lib/Adhesion.php";
include_once "lib/Bdd.php";

define('ERR_BDD_ExerciceGet_01',      10701);

/**
 * Classe definissant un exercice
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 1145 $
 * @package gasell
 */
class Exercice
{
	var $TypeAdhesion;  /* Objet TypeAdhesion (fichier : Adhesion.php) */
	var $Num;
	var $Libelle;
	var $Status;
	var $DateDebut;
	var $DateFin;

	/******************************************************************
	 * @method Exercice                                               *
	 *                                                                *
	 * @brief constructeur de la classe "Exercice"                    *
	 *                                                                *
	 * @param Gasell $GasellSession : Session Gasell de reference     *
	 * @param string $exercice      : Ref de l'exercice a instancier  *
	 *                                                                *
	 * @result Exerice : une instance de la classe                    *
	 ******************************************************************/
	function Exercice($exercice='')
	{
		global $Gasell;

		/* Sauvegarde la reference a l'objet Gasell de la session            */
		$this->TypeAdhesion  = new TypeAdhesion();

		/* Si un numero d'exercice a charge a ete precise  */
		// SECURITE: protection de l'exerice en forcant un entier
		if (isset($exercice) && (is_integer($exercice)))
		 {
			/* Alors, sauvegarde dans l'objet du numero d'exercice demande  */
			$this->Num = $exercice;
		 /* Sinon, utilisation de l'exercice courrant                        */
		 } else {
			/* Sauvegarde du numero d'exercice pris dans la config */
			$this->Num = $Gasell->CfgGet("ASSO", "Exercice");
		 }

		/* Sauvegarde le numero de l'exercice dans les Types d'Adhesions*/
		$this->TypeAdhesion->exenum = $this->Num;
		$exedata = $this->ExerciceGet($this->Num);

		// FIXME : et si plusieurs exercices sont retournes ?
		if (count($exedata) > 0)
		{
			$DateDebut = $exedata[0][1];
			$DateFin   = $exedata[0][2];
			$Libelle   = $exedata[0][3];
		}

		/* Recuperation du status de l'exercice */
		$tmp = $Gasell->CfgGet("ASSO", "ExerciceStatus");
		switch(strtoupper($tmp))
		{
			case 'OUVERT':
				$this->Status = 'ouvert';
				break;

			case 'FERME':
				$this->Status = 'ferme';
				break;

			// considere ferme par defaut
			default:
				$this->Status = 'ferme';
				break;
		}
//		$ENum = $Gasell->CfgGet("ASSO", "Exercice");
//		if ($this->Num == $ENum) $this->Status = 'Present';
//		else if ($this->Num < $ENum) $this->Status = 'Passe';
//		else if ($this->Num > $ENum) $this->Status = 'Futur';
	}
  
	/*************************************************************
	 * @method ExerciceGet                                       *
	 *                                                           *
	 * @brief retourne un exercice ou bien tous les exercices si *
	 *  aucun numero d'exercice n'est precise                    *
	 *                                                           *
	 * @param integer $num : le numero d'exercice a recuperer    *
	 *                                                           *
	 * @result array : liste des exercices retounres             *
	 *************************************************************/
	function ExerciceGet($num="")
	{
		global $Gasell;
		// TODO : retourne toutes les informations en meme temps !
		$q_cols = array("exenum", "datedebut", "datefin", "libelle");
		$q_cont = array("type IS NULL", "OR type='asso'");

		/* SECURITE: prise en compte de num seulement si int */
		// TODO loguer un warning lorsque $num n'est pas integer */
		if ( ($num != "") && (is_integer($num)) )
			array_push($q_cont, "AND exenum='" . $num . "'");
		
		$result = $Gasell->db->Select("exercice", $q_cols, $q_cont);
		
		// TODO: Verification de bonne execution
		return($result);
	}

	/*************************************************************
	 * @method GetAll                                            *
	 *                                                           *
	 * @brief retourne tous les exercices sauf les exerices nuls *
	 *  si le flag est a faux                                    *
	 *                                                           *
	 * @param bool filter : filtrage des exercices nuls ou pas   *
	 *                                                           *
	 * @result array : tableau des exercices                     *
	 *************************************************************/
	function GetAll($filter = true)
	{
		global $Gasell;
		$result = array();
		
		/* Recuperation de tous les exercices dans la base                   */
		if (isset($this) && (get_class($this) == "Exercice"))
			$exedata = $this->ExerciceGet();
		else
			$exedata = Exercice::ExerciceGet();
			
		/* Pour chaque exercice trouve dans la base                          */
		foreach($exedata as $data)
		{
			if( ($filter && ($data[1] != NULL)) || !$filter )
			{
				$result[$data[0]]['libelle']   = $data[3];
				/* Sauvegarde dans la reponse la date de debut de l'exercice */
				$result[$data[0]]['datedebut'] = $data[1];
				/* Sauvegarde dans la reponse de la date de fun de l'exercice */
				$result[$data[0]]['datefin']   = $data[2];
			}
		}
		
		/* Renvoi a l'appelant un tableau des exercices trouves              */
		return($result);
	}
  
	/**
	 * Creation d'un nouvel exercice
	 *
	 * @param $nom       : Identifiant du nouvel exercice
	 * @param $datedebut : Date de debut de l'exercice
	 * @param $datefin   : Date de fin de l'exercice
	 */
	function Create($nom, $datedebut, $datefin)
	{
		global $Gasell;
    
		/* Ajout d'un exercice dans la table des exercices */

		// prepare the request
		$q_cols['libelle']   = $nom;
		$q_cols['type']      = 'asso';
		$q_cols['datedebut'] = $datedebut;
		$q_cols['datefin']   = $datefin;
		$index = $Gasell->db->Insert('exercice', $q_cols, 'exenum');
		
		/* Mise a jour de la config : Numero de l'exercice en cours */
		$Gasell->CfgSet("ASSO", "Exercice", $index);
		/* Mise a jour de la config : Status de l'exercice en cours */
		$Gasell->CfgSet("ASSO", "ExerciceStatus", "Ouvert");
		return;
	}

	// quel est son role ?
	function Status($stat)
	{
		global $Gasell;
		$Gasell->CfgSet("ASSO", "ExerciceStatus", $stat);
		return;
	}
  
  	// quel est son role ?
	function StatusGet()
	{
		$now = getdate();
		echo $this->DateDebut;
		return;
	}

	/*********************************************************
	 * @method nextStatus                                    *
	 *                                                       *
	 * @brief permet de retourner le prochain statut dispo   *
	 *  pour l'exercice courant                              *
	 *                                                       *
	 * @noparam                                              *
	 *                                                       *
	 * @result string : le prochain statut disponible ou     *
	 *   false si il n'est plus possible de changer le statut*
   	 *********************************************************/
  	function nextStatus()
  	{
    	switch(strtoupper($this->Status))
      	{
      		case "OUVERT":
				return "FERME";
		
      		case "FERME":
	  			return false;
		
      		default:
	  			return "OUVERT";
      	}
  	}

	/*********************************************************
	 * @method openExercice                                  *
	 *                                                       *
	 * @brief permet d'ouvrir un exercice qui est non ouvert *
	 *                                                       *
	 * @noparam                                              *
	 *                                                       *
	 * @result bool : renvoie vrai si tout s'est bien deroule*
	 *********************************************************/
	 function openExercice()
	 {
	 	// FIXME : check that we can do it
	 	global $Gasell;
		
		/* mise a jour de la date de debut */
		$today = sprintf("'%s-%s-%s'", date('Y'), date('m'), date('j')); 

		$cols  = array( 'datedebut' => $today );
		$const = array( 'exenum=\''.$this->Num.'\'' );
		$Gasell->db->Update('exercice', $cols, $const);
		
		/* mise a jour dans la config si il s'agit de l'exercice courant */
		if($this->Num == $Gasell->CfgGet('ASSO', 'Exercice'))
			$Gasell->CfgSet('ASSO', 'ExerciceStatus', 'Ouvert');

		// FIXME : check that it is ok
		return true;
	 }

	/*********************************************************
	 * @method closeExercice                                 *
	 *                                                       *
	 * @brief permet de fermer un exercice                   *
	 *                                                       *
	 * @noparam                                              *
	 *                                                       *
	 * @result bool : renvoie vrai si tout s'est bien deroule*
	 *********************************************************/
	 function closeExercice()
	 {
	 	// FIXME : check that we can do it
	 	global $Gasell;
		
		/* mise a jour de la date de fin */
		$today = sprintf("'%s-%s-%s'", date('Y'), date('m'), date('j')); 
		
		$cols  = array( 'datefin' => (string)$today);
		$const = array( 'exenum=\''.$this->Num.'\'' );
		$Gasell->db->Update('exercice', $cols, $const);
		
		/* mise a jour dans la config si il s'agit de l'exercice courant */
		if($this->Num == $Gasell->CfgGet('ASSO', 'Exercice'))
			$Gasell->CfgSet('ASSO', 'ExerciceStatus', 'Ferme');

		// FIXME : check that it is ok
		return true;
	 }


}
/* EOF */
?>
