<?php
/*
 * Gasell: Copyright 2005-2006 Linux-Nantes Association <contact@linux-nantes.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 */

/* Historique :
 *  * 25/06/2006 DrazziB : Utilisation des Exceptions a la place de la methode Erreur()
 *  * 10/01/2006 DrazziB : Deplacement des methodes de manipulation des comptes
 *      vers la classe ComptesGasell.
 *  * 14/07/2005 Hooligan0 : Remplacement de toutes les requetes SELECT par un
 *      appel a la methode dediee de Bdd. Deplacement des fonctions de config.
 *  * 09/07/2005 Hooligan0 : Ajout a GasellPage des methodes pour l'inclusion de
 *      fichiers et la gestion de CSS optionelles.
 *  * 26/05/2004 Hooligan0 : La gestion des parametre 'db' et 'prefix' pour
 *      chaque utilisateur est ajoute au constructeur. Les fonctions de
 *      manipulation horde prennent egalement ce changement en compte.
 *  * 25/05/2004 Hooligan0 : Prise en compte du mot de passe pour la creation
 *      d'un utilisateur horde.
 */

require_once GASELL_BASE . "/lib/Bdd.php";
require_once GASELL_BASE . "/lib/ComptesGasell.php";

/**
 * GasellPage : Gestion d'une page web de l'appli
 *
 * @version $Revision: 1157 $
 * @package gasell
 */
class GasellPage
{
	var $name;
	var $args_get;
	var $args_post;
	var $CSSlist;
	var $redirectFrom;
	var $errorMsgs; /* tableau contenant les messages d'erreur */
	
	/**
	 * Constructeur d'un onjet GasellPage
	 *
	 */
	function GasellPage()
	{
		/* Initialisation de la liste des CSS optionelles                    */
		$this->CSSlist = array();
		/* Initialisation du nom de la page et de (l'eventuelle) redirection */
		$this->name         = NULL;
		$this->redirectFrom = NULL;
		/* Initialisation des messages d'erreur */
		$this->errorMsgs = array();
	}
	
	/**
	 * Chargement d'une page
	 *
	 * @param $page : Nom de la page a charger
	 */
	function load($page, $args_get = NULL, $args_post = NULL)
	{
		global $PAGE;
		global $Gasell;
		
		/* Sauvegarde le nom de la page chargee dans l'objet                 */
		$this->name = $page;
		
		/* Sauvegarde les arguments passes a la page dans l'objet            */
		if ($args_get != NULL)
			$this->args_get  = $args_get;
		if ($args_post != NULL)
			$this->args_post = $args_post;
		
		// TODO: Ajouter ici la gestion des droits sur la page
		
		/* affichage des messages d'erreur en tete de page  */
		if(count($this->errorMsgs) > 0){
		  foreach($this->errorMsgs as $msg)
		    echo "<div class='error'>".$msg."</div>\n";
		}
		  

		$filename = GASELL_TEMPLATES . '/' . $PAGE[$page];
		if (file_exists($filename))
		{
			/* Inclusion du fichier de template pour la page demandee        */
			include_once $filename;
		} else {
			/* Log pour trouver les liens vers le fichiers casses !! */
			if(file_exists('/tmp') && is_dir('/tmp'))
			{
				$fhd = fopen("/tmp/GASELL.log", "a");
				fwrite($fhd, "Page $filename called from ");
				fwrite($fhd, $_SERVER["PHP_SELF"]."?".$_SERVER["QUERY_STRING"]);
				fwrite($fhd, " is broken !\n");
				fclose($fhd);
			}
			/* inclusion du template 404 si il existe */
			$filename = GASELL_TEMPLATES . '/' . $PAGE['_404_'];
			if(file_exists($filename) && is_readable($filename))
				include_once $filename;
			else
			{
				echo "Un fichier d'action est manquant et le template";
				echo "_404_ ne peut &ecirc;tre atteint !";
			}
		}
	}
	
	/**
	 * Demande l'inclusion d'un fichier depuis une page
	 *
	 * @param $page : Reference (dans _pages_defs) du fichier a inclure
	 * @param $args : Tableau optionnel de parametres passes au fichier
	 */
	function loadInclude($page, $args = NULL)
	{
		global $IPAGE;
		global $Gasell;

		// TODO Controle d'acces
		
		/* Verification de l'existance de l'identifiant et du fichier        */
		$filename = GASELL_TEMPLATES . '/' . $IPAGE[$page];
		if (file_exists($filename))
			/* Tout semble Ok, inclusion du fichier                          */
			include_once GASELL_TEMPLATES . '/' . $IPAGE[$page];
		else
			echo "Un fichier a inclure est manquant !";
	}
	
	/**
	 * Permet a un fichier template d'ajouter la ref d'une CSS a inclure
	 *
	 * @param $name : Reference de la CSS a inclure
	 */
	function addCSS($name)
	{
		array_push($this->CSSlist, $name);
	}
	
	/**
	 * Lecture de la liste des CSS optionelles
	 *
	 * @return : La liste des ref de CSS
	 */
	function getCSS()
	{
		return($this->CSSlist);
	}
	
	function getPageName()
	{
		return($this->name);
	}
	
	/**
	 * Realise une action sur une page
	 *
	 * @access public
	 * @param  $group  : Section de l'action (ou module)
	 * @param  $nom    : Nom de l'action a realiser
	 * @return // depend de l'action //
	 */
	function doAction($group, $name, $args = NULL)
	{
		/* Construction du nom du fichier de l'action                       */
		$a_name = '/actions/';
		if ($group != NULL)
			$a_name .= $group . '/';
		$a_name .= $name;
		
		// TODO : controle des droits d'acces
		
		// NOTE : Cherche les fichiers avec ou sans extension pour
		//	preserver la compatibilite avec les actions existantes.
		//	Modif suite a discution securite avec DRM, FP, GS
		
		/* Construction du nom de fichier pour l'action                     */
		$filename = GASELL_BASE . $a_name;
		/* Si un fichier d'action ".PHP" existe                             */
		if (file_exists($filename . '.php'))
		{
			/* Inclue le fichier ".PHP" de l'action demandee                */
			require $filename . '.php';
		}
		/* Sinon, si un fichier d'action sans extension existe              */
		else if (file_exists($filename))
		{
			/* Inclue le fichier de l'action demandee                       */
			require $filename;
		}
		/* Sinon, aucun fichier d'action valide n'est trouve                */
		else
		{
			/* Il s'agit d'une erreur fatale !!                             */
			echo "Fichier d'action $filename introuvable !<br/>";
			echo "MAIN SECURITY ERROR ! Please contact admin";
			exit(-1);
		}
		
		return($result);
	}

	/**
	 * Recherche et renvoi la valeur d'un parametre passe a la page
	 *
	 * @param $name : Nom du parametre recherche
	 * @return Valeur du parametre ou NULL si non trouve
	 */
	function getArg($name)
	{
		/* Si le parametre existe dans les donnees POST de la page           */
		if (isset($this->args_post[$name]))
			/* Alors, renvoi en priorite cette valeur POST                   */
			return($this->args_post[$name]);
		/* Sinon, si le parametre existe dans les donnees GET de la page     */
		else if (isset($this->args_get[$name]))
			/* Alors, renvoi de cette valeur GET                             */
			return($this->args_get[$name]);
		/* Sinon, le parametre n'existe pas, renvoi NULL                     */
		else
			return (NULL);
	}
	
	/**
	 * Cette methode permet de recuperer la totalite des parametres
	 *    L'interet est de simplifier les gros formulaires (bcp params)
	 *    mais le risque est de faire trop de traitements en templates...
	 *
	 * @return tableau de l'enssemble des datas du formulaire
	 */
	function getAllArgs()
	{
		$result = array();
		foreach ($this->args_get as $k => $v)
			$result[$k] = $v;
		foreach ($this->args_post as $k => $v)
			$result[$k] = $v;
		return($result);
	}
	
	/**
	 * Permet la redirection vers une autre page
	 *
	 * @param $page : Identifiant de la page cible
	 */
	function doRedirect($page)
	{
		global $PAGE;
		
		/* Verification de l'existance de la page de renvoi                  */
		if (! isset($PAGE['_REDIRECT_']))
			die("Page de redirection manquante !");
		$filename = GASELL_TEMPLATES . '/' . $PAGE['_REDIRECT_'];
		if (! file_exists($filename))
			die("Fichier de redirection introuvable !");
		
		/* Sauvegarde du nom de la page avant redirection                    */
		$this->redirectFrom = $this->name;

		/* Calcul de l'URL complete */
		$url = "index.php?page=" . $page;
		
		/* Redirection vers une nouvelle page                                */
		$this->load('_REDIRECT_', array("url" => $url));
		exit();
	}

   /**
	 * @brief fonction qui ajoute un message d'erreur a la page
     *
	 * @param $err string le message a ajouter
	 **/
	function errorMessage($err)
	 {
		$this->errorMsgs[] = $err;
	 }
}

/**
 * Gasell Base Class.
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 1157 $
 * @package gasell
 */
class Gasell
{
	var $dblink;
	var $db;          /* Objet de type Bdd */
	var $asso;
	var $user;
	var $config;
	var $Comptes;
	var $Page;
  
	/**
	 * Constructeur de "Gasell"
	 *
	 * @access public
	 */
	function Gasell($user = NULL)
	{
		global $conf;
	
		$this->db = new Bdd();

		/* Si un login utilisateur a ete passe au constructeur               */
		if ($user != NULL)
		{
			$q_cols = array("user_uid","memnum","level","asso","template","db","dbprefix");
			$q_cont = array("user_uid='" . $user . "'");
			if (isset($_SESSION['gasell']['user']['asso']))
			{
				$this->asso = $_SESSION['gasell']['user']['asso'];
				$q = "AND asso='".$_SESSION['gasell']['user']['asso']."'";
				array_push($q_cont, $q);
			}
			$result = $this->db->Select("comptes_gasell", $q_cols, $q_cont);
			/* Si l'utilisateur n'apartient a aucune asso                    */
			if (empty($result))
			{
				/* Alors, aucun droit d'acces a l'appli, retour immediat     */
				$this->user = NULL;
				return;
			}
			/* Si l'utilisateur fait parti de plusieurs assos                */
			if (count($result) > 1)
			{
				$this->user['asso'] = array();
				foreach ($result as $Element)
					array_push($this->user['asso'], $Element[3]);
				return;
			}
			$result = $result[0];
			$this->user['login']    = $user;
			$this->user['level']    = $result[2];
			$this->user['asso']     = $result[3];
			$this->user['memnum']   = $result[1];
			$this->user['template'] = $result[4];
			$this->user['db']       = $result[5];
			$this->user['dbprefix'] = $result[6];

			$this->db->reconnectDB($this->user['db']);
		}

		/* Est ce qu'une session de Gasell est deja ouverte ?                */
		if (isset($_SESSION['gasell']['config']))
		{
			/* Oui :) La config peut etre chargee a partir de la session     */
			$this->config = $_SESSION['gasell']['config'];
			$this->config['SESSION'] = $_SESSION['gasell']['session'];
		}
		else
		{
			/* Non :( Il faut donc lire la config depuis la base             */
			$Config = $this->configInit();
			/* Si la config mini n'a pas pu etre chargee                     */
			if (!$Config)
			{
				/* Alors, l'installation n'est pas terminee !                */
				$ecode = "0002";
				$emsg  = "La configuration de base n'est pas termin&eacute;e. Utilisez le module d'<a href=\"setup/install.php\">installation</a> !";
				/* Affichage d'une page pour cette erreur                    */
				@define('GASELL_TEMPLATES', 'templates');
				require GASELL_TEMPLATES . '/erreur.php';
				/* L'execution ne peut se poursuivre ! Sortie                */
				exit;
			}
		}
    
		/* Cree un objet d'acces aux fonctions specifiques des comptes       */
		$this->Comptes = new ComptesGasell();

		/* Si aucun nom d'utilisateur n'a ete specifie                       */
		if ($user == NULL)
		{
			$this->user = NULL;
			return;
		}

		global $conf;
		/* Sauvegarde la configuration actuelle                              */
		$conf_save = $conf;
		include GASELL_BASE . '/config/conf.php';

		$this->Comptes->setPersonne($this->user['memnum']);

		/* Restauration de la configuration                                     */
		$conf = $conf_save;
		return;
	}

	/**
	 * Initialisation d'un objet Gasell Page (avant utilisation template)
	 *
	 */
	function PageInit()
	{
		$this->Page = new GasellPage();
	}
  
	/**
	 * Chargement de la configuration depuis la BDD
	 *
	 * @param  string $exenum : Numero de l'exercice a charger
	 * @return Tableau du contenu de la table de config
	 */
	function configInit($exenum = '')
	{
		$resultat = array();

		$data = $this->db->Select("config", array("cfgparam","cfgvaleur"), array("cfgsection='ASSO'"));
		/* Si une erreur se produit pendant la requette                      */
		if ($data == NULL)
			/* Alors, appel le module de gestion d'erreur                    */
			Except::throwError("Gasell", "Code 10201");

		/* Placement des donnees de la section ASSO dans le resultat         */
		$resultat['ASSO'] = $data;
		/* Si le numero de l'exercice charge n'a pas ete passe en parametre  */
		if (!$exenum)
		{
			/* Pour chaque ligne trouvee dans la section ASSO                */
			foreach($data as $entree)
			/* Si le parametre de la ligne est le numero de l'exercice actuel*/
			if ($entree[0] == "Exercice")
			/* Alors, sauvegarde sa valeur                                   */
				$exenum = $entree[1];
		}

		$q_cols = array("cfgsection", "cfgparam", "cfgvaleur");
		$q_cont = array("exenum='".$exenum. "'", "OR exenum='0'");
		$data = $this->db->Select("config", $q_cols, $q_cont);
		/* Si une erreur se produit pendant la requette                      */
		if ($data == NULL)
		/* Alors, appel le module de gestion d'erreur                        */
			Except::throwError("Gasell", "Code 10203");

		/* Placement des donnees de l'exercice demande dans le resultat      */
		$resultat['exe'] = $data;

		$this->config = $resultat;

		/* Est-ce que la config doit etre placee en cache dans la session ?  */
		$iscached = $this->CfgGet("CONFIG", "CacheConfig");
		if ($iscached == "OUI")
		{
			/* La config est sauvee dans la session pour etre reutilisable   */
			$_SESSION['gasell']['config'] = $Config;
		}
		else
		{
			if (isset($_SESSION['gasell']['config']))
				unset($_SESSION['gasell']['config']);
		}
      
		/* Creation de l'entree pour la pseudo-config SESSIOn                */
		if (! isset($_SESSION['gasell']['session']))
		{
			$_SESSION['gasell']['session'] = array();
			$this->config['session'] = array();
		}
		else
			$this->config['session'] = $_SESSION['gasell']['session'];

		/* Retourne le tableau resultat a l'appelant                         */
		return($resultat);
	}

	/**
	 * Recherche d'un parametre dans la configuration
	 *
	 * @access public
	 * @param string $Section  : Section de la config
	 * @param string $param    : Nom du parametre
	 * @param string $exercice : Numero de l'exercice dans lequel chercher
	 * @return string Valeur du parametre ou NULL si non trouve
	 */
	function CfgGet($Section, $param, $exercice='')
	{
		/* Si la section de config demandee est la pseudo-config SESSION     */
		if ($Section == "SESSION")
		{
			$config = $this->config;
			if (isset($config['session'][$param]))
				return($config['session'][$param]);
			return(NULL);
		}

		/* Si un numero d'exercice particulier a ete precise                 */
		if ($exercice)
			/* */
			$config = $this->configInit($exercice);
		/* Sinon, aucun numero d'exercice n'a ete precise                    */
		else
			/* Alors, utilisation de l'exercice actuel                       */
			$config = $this->config;
    
		/* Si la recherche porte sur la section ASSO                         */
		if ($Section == "ASSO")
		{
			/* Pour chaque element de la section ASSO de la config           */
			foreach($config['ASSO'] as $configEntry)
			 {
				/* Si le champ 'param' concorde avec la recherche            */
				if ($configEntry[0] == $param)
				 {
					/* Alors, la valeur du parametre est renvoyee a l'appelant */
					return $configEntry[1];
				 }
			 }
		}
		/* Sinon, la recherche porte sur une autre section que ASSO          */
		else
		{
			/* Pour chaque element de la configuration                       */
			foreach($config['exe'] as $configEntry)
				/* Si le champ 'section' concorde avec la recherche ...      */
				if ($configEntry[0] == $Section)
					/* ... et si le champ 'param' concorde avec la recherche */
					if ($configEntry[1] == $param)
						/* Alors, la valeur du parametre est renvoyee a l'appelant */
						return $configEntry[2];
		}
		/* Si aucun element de config correspond a la requette, retourne NULL*/
		return(NULL);
	}

	/**
	 * Mise a jour d'un parametre de configuration
	 *
	 * @param string $Section  : Section de la config
	 * @param string $Param    : Nom du parametre
	 * @param string $Valeur   : Nouvelle valeur du parametre
	 */
	function CfgSet($Section, $Param, $Valeur)
	{
		/* Si la section recherchee est "session" : cas particulier          */
		if ($Section == "SESSION")
		{
			$this->config['session'][$Param] = $Valeur;
			$_SESSION['gasell']['session'][$Param] = $Valeur;
			return;
		}
    
		$old = $this->CfgGet($Section, $Param);
//		if ($old)
//		{
			$query =  "UPDATE config SET cfgValeur='" . $Valeur . "' ";
			$query .= "WHERE cfgSection='" . $Section . "' AND cfgParam='" . $Param . "'";
			$result = $this->db->Requette($query);
//		}
//		else
//			Bdd::ConfigCreate($Section, $Param, $Valeur);

		// TODO : Mettre la lecture de la config dans une fonction

		$Config = $this->configInit();
		$this->config =  $Config;
		/* La config est sauvee dans la session pour etre reutilisable       */
		$_SESSION['gasell']['config'] = $Config;
		session_register('gasell');
	}
}
/* EOF */
?>
