<?php
/*
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 *
 * Histoire
 *  * 19/06/2005 DrazziB : Implementation des specifications de
 *                       la partie activite / participation
 *  * 18/12/2005 DrazziB : Fonction de cloture de l'activite
 */

// Benevolat
@define('GSL_PARTICIPATION_ptype_VolontaryHelp',   0);
// Volontariat
@define('GSL_PARTICIPATION_ptype_Volontariate', 1);

/**
 * Classe Participation : Gestion des participations
 * aux differentes activites de l'association.
 *
 * --> Utilisation de la table INTERVENTION
 *
 * @author Damien Raude-Morvan <drazzib@drazzib.com>
 * @since 0.6
 * @package gasell
 */
class Participation
{

	/** Identifiant de l'activite
	(i.e. en base de donnee */
	var $id;

	/** Type de participation. Pour le moment, on gere
	 *	deux types : O = benevolat (classique), 1 = volontariat
	 * @type integer
	 */
	var $type;

	/** Si cette variable est == true, la personne lie a cette participation
	 *	est le responsable de l'activite
	 * @type boolean
	 */
	var $manager;
	
	/** Date de debut de la participation
	 * @type date
	 */
	var $startDate;

	/** Date de fin de la participation
	 * @type date
	 */
	var $endDate;

	/** Objet Personne (GASELL) lie a cette participation
	 * @type Personne
	 */
	var $person;

	/** identifiant de l'adhesion.
	 * @type Adhesion
	 */
	var $adhesion;

	/** Activite lie a cette participation
	 * @type Activite
	 */
	var $activity;

	/** Bilan textuel de cette participation
	 * @type string
	 */
	var $bilan;

	/**
	 * Recuperation de la liste complete des Participations
	 *
	 * @access public
	 */
	function ListParticipation()
	{
		return Participation::ListParticipationForAdhesionAndPersonne();
	}

	/**
	 * Recuperation de la liste des Participations
	 * affectees a une adhesion ou a un membre.
	 *
	 * Si les deux parametres sont vides, on recupere l'ensemble
	 * des participations, peu importe leur type ou leur etat.
	 *
	 * TODO Ajout d'un nouveau parametre qui permet de selectionner
	 *  les participations sur leur date.
	 *
	 * @access public
	 */
	function ListParticipationForAdhesionAndPersonne($adhesion_id = "", $personne_id = "")
	{
		global $Gasell;
		
		$query  = "SELECT intNum";
		$query .= " FROM intervention";

		/* On effecture donc une selection */
		if ($adhesion_id != "" || $personne_id != "")
		 {
			/* Debut de selection via la clause WHERE */
			$query .= " WHERE ";
			
			if ($personne_id != "")
			 {
				/* On selectionne sur une personne particuliere */
				$query .= " memNum='".$personne_id."'";
		 	 }
			else if ($adhesion_id != "")
			 {
				if ($personne_id != "")
				 {
					/* On ajoute un critere sur l'adhesion */
					/* Donc on selectionne adhesion + personne */
					$query .= " AND ";
				 }
				/* On selectionne sur une adhesion particuliere */
				$query .= " adhNum='".$adhesion_id."'";
			 }
		 }
		else
		 {
			/* Si il n'y a pas d'adhesion selectionnes, on selectionne
			l'ensemble des participations lies a une adhesion. */
			$query .= " WHERE memNum IS NOT NULL";
		 }
		$query .= " ORDER BY intnum";

		/* Envoi de la requete a la base de donnee */
		$result = $Gasell->db->Requette($query, "ALL");

		/* Si il existe des elements dans le resultat de la base */
		if (!empty($result))
		 {
			/* Creation du tableau des activites pour le retour */
			$tabParticipations = NULL;
			$i = 0;
			foreach ($result as $tupleParticipation)
			 {
				/* Pour chaque ligne de resultat on recupere le premier attribut */
				$tabParticipations[$i] = new Participation($tupleParticipation[0]);
				$i++;
			 }
			return($tabParticipations);
		 }
		else
		 {
			return(NULL);
		 }
	}

	/**
	 * Recuperation de la liste des Participations
	 * affectees a une activite.
	 *
	 * TODO Ajout d'un nouveau parametre qui permet de selectionner
	 *  les participations sur leur date.
	 *
	 * @access public
	 */
	function ListParticipationForActivity($activity_id)
	{
		global $Gasell;
		
		$query  = "SELECT intNum";
		$query .= " FROM intervention";
		/* On selectionne sur une activite particuliere */
		$query .= " WHERE actCode = '".$activity_id."'";
		$query .= " ORDER BY intnum";

		/* Envoi de la requete a la base de donnee */
		$result = $Gasell->db->Requette($query, "ALL");

		/* Si il existe des elements dans le resultat de la base */
		if (!empty($result))
		 {
			/* Creation du tableau des activites pour le retour */
			$tabParticipations = NULL;
			$i = 0;
			foreach ($result as $tupleParticipation)
			 {
				/* Pour chaque ligne de resultat on recupere le premier attribut */
				$tabParticipations[$i] = new Participation($tupleParticipation[0]);
				$i++;
			 }
			return($tabParticipations);
		 }
		else
		 {
			return(NULL);
		 }
	}

	/**
	 * Recuperation d'une Participation (en utilisant son ID)
	 * --> Constructor ???
	 *
	 * @access public
	 */
	function Participation($part_id)
	{
		global $Gasell;
		
		$query = "SELECT intNum, adhNum, memNum, actCode, intStartDate, intEndDate, intType, intBilan";
		$query .= " FROM intervention";
		$query .= " WHERE intNum='".$part_id."'";
		$result = $Gasell->db->Requette($query, "ALL");
		if (!empty($result))
		 {
			/* Affectation de l'identifiant de la participation */
			$this->id = $result[0][0];
			/* Affectation du type de la participation */
			$this->type = $result[0][6];
			/* TODO FIXME XXX */
			$this->manager = false;
			/* Affectation de la date de la participation */
			$this->startDate = $result[0][4];
			$this->endDate = $result[0][5];
			/* Affectation du bilan de l'activite */
			$this->bilan = $result[0][7];

			/* Recuperation de la personne afin de pouvoir afficher son nom */
			if ($result[0][2] != NULL)
			 {
				$this->personne = new Personne($result[0][2]);
			 }

			/* Recuperation de l'Adhesion afin de pouvoir afficher son identifiant */
			if ($result[0][1] != NULL)
			 {
				$this->adhesion = new Adhesion($result[0][1]);
			 }

			/* Recuperation de l'activite afin de pourvoir afficher son libelle */
			if ($result[0][3] != NULL)
			 {
				$this->activity = new Activite($result[0][3]);
			 }
	
			return($this);
		 }
		else
		 {
			return(NULL);
		 }
	}

	/**
	 * @access private
	 *
	 * @param string  $person_id   : identifiant de la personne.
	 * @param string  $member_id   : identifiant du membre (adhesion).
	 * @param string  $activity_id : identifiant de l'activite.
	 * @param integer $type        : O = benevolat, 1 = volontariat.
	 * @param date    $startdate   : date de debut (Y-m-d).
	 * @param date    $enddate     : date de fin (Y-m-d).
	 */
	function _Create($person_id, $member_id, $activity_id, $type, $startdate, $enddate)
	{
		global $Gasell;

		/* Generation des champs utilises dans la requete */
		$query  = "INSERT INTO intervention(actCode,";
		if ($person_id != NULL)
		 {
			$query .= " memNum, ";
		 }
		if ($member_id != NULL)
		 {
			$query .= " adhNum, ";
		 }
		if ($startdate != NULL)
		 {
			$query .= " intStartDate, ";
		 }
		if ($enddate != NULL)
		 {
			$query .= " intEndDate, ";
		 }
		$query .= " intType)";
		/* Generation des valeurs de la requete */
		$query .= " VALUES(";
		$query .= "'".$activity_id."', ";
		if ($person_id != NULL)
		 {
			$query .= "'".$person_id."', ";
		 }
		if ($member_id != NULL)
		 {
			$query .= "'".$member_id."', ";
		 }
		if ($startdate != NULL)
		 {
			$query .= "'".$startdate."', ";
		 }
		if ($enddate != NULL)
		 {
			$query .= "'".$enddate."', ";
		 }
		$query .= "'".$type."')";

		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * @access protected
	 *
	 * @param string  $member_id   : identifiant du membre (adhesion).
	 * @param string  $activity_id : identifiant de l'activite.
	 * @param integer $type        : O = benevolat, 1 = volontariat.
	 * @param date    $startdate   : date de debut (Y-m-d).
	 * @param date    $enddate     : date de fin (Y-m-d).
	 */
	function CreateForAdhesion($member_id, $activity_id, $type = 0, $startdate = NULL, $enddate = NULL)
	{
		return Participation::_Create(NULL, $member_id, $activity_id, $type, $startdate, $enddate);
	}

	/**
	 * @access protected
	 *
	 * @param string  $person_id   : identifiant de la personne.
	 * @param string  $activity_id : identifiant de l'activite.
	 * @param integer $type        : O = benevolat, 1 = volontariat.
	 * @param date    $startdate   : date de debut (Y-m-d).
	 * @param date    $enddate     : date de fin (Y-m-d).
	 */
	function CreateForPerson($person_id, $activity_id, $type = 0, $startdate = NULL, $enddate = NULL)
	{
		return Participation::_Create($person_id, NULL, $activity_id, $type, $startdate, $enddate);
	}

	/**
	 * Suppression d'une participation.
	 *
	 * WARN --> Preferer le changement de la date de cloture !
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de la participation
	 */
	function Delete($code)
	{
		global $Gasell;

		$query  = "DELETE FROM intervention";
		$query .= " WHERE intNum='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * Cloture d'une participation en changeant la date de cloture
	 * a la date courante.
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de la participation
	 */
	function Cloturer($code)
	{
		global $Gasell;

		$endDate = date("Y-m-d");

		$query  = "UPDATE intervention";
		$query .= " SET intEndDate='".$endDate."'";
		$query .= " WHERE intNum='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}

	/**
	 * Definit le texte du rapport de la participation.
	 *
	 * @access public
	 * @type static
	 * @param string  $code    : Code de la participation
	 * @param string  $report  : Texte du rapport de la participation
	 */
	function SaveReport($code, $report)
	{
		global $Gasell;

		$query  = "UPDATE intervention";
		$query .= " SET intBilan='".$report."'";
		$query .= " WHERE intNum='".$code."'";
		return ($Gasell->db->Requette($query) != 0);
	}
}


class Volontariate extends Participation
{

	/**
	 * Creation d'une nouvelle participation (Volontariat)
	 * pour une personne.
	 *
	 * @access public
	 * @param string  $person_id    : Identifiant de la personne
	 * @param string  $activity_id  : Identifiant de la participation
	 * @param date    $startdate    : date de debut (Y-m-d).
	 * @param date    $enddate      : date de fin (Y-m-d).
	 */
	function CreateVolontariateForPerson($person_id, $activity_id, $startdate = NULL, $enddate = NULL)
	{
		return Participation::CreateForPerson($person_id, $activity_id, GSL_PARTICIPATION_ptype_Volontariate, $startdate, $enddate);
	}

	/**
	 * Creation d'une nouvelle participation (Volontariat)
	 * pour un adherent dans le cadre d'une adhesion.
	 *
	 * @access public
	 * @param string  $member_id    : Identifiant de l'adhesion
	 * @param string  $activity_id  : Identifiant de la participation
	 * @param date    $startdate    : date de debut (Y-m-d).
	 * @param date    $enddate      : date de fin (Y-m-d).
	 */
	function CreateVolontariateForAdhesion($member_id, $activity_id, $startdate = NULL, $enddate = NULL)
	{
		return Participation::CreateForAdhesion($member_id, $activity_id, GSL_PARTICIPATION_ptype_Volontariate, $startdate, $enddate);
	}

}

class VolontaryHelp extends Participation
{

	/**
	 * Creation d'une nouvelle participation (Benevolat)
	 * pour une personne.
	 *
	 * @access public
	 * @param string  $person_id    : Identifiant de la personne
	 * @param string  $activity_id  : Identifiant de la participation
	 * @param date    $startdate    : date de debut (Y-m-d).
	 * @param date    $enddate      : date de fin (Y-m-d).
	 */
	function CreateVolontaryHelpForPerson($person_id, $activity_id, $startdate = NULL, $enddate = NULL)
	{
		return Participation::CreateForPerson($person_id, $activity_id, GSL_PARTICIPATION_ptype_VolontaryHelp, $startdate, $enddate);
	}

	/**
	 * Creation d'une nouvelle participation (Benevolat)
	 * pour un adherent dans le cadre d'une adhesion.
	 *
	 * @access public
	 * @param string  $member_id    : Identifiant du l'adhesion
	 * @param string  $activity_id  : Identifiant de la participation
	 * @param date    $startdate    : date de debut (Y-m-d).
	 * @param date    $enddate      : date de fin (Y-m-d).
	 */
	function CreateVolontaryHelpForAdhesion($member_id, $activity_id, $startdate = NULL, $enddate = NULL)
	{
		return Participation::CreateForAdhesion($member_id, $activity_id, GSL_PARTICIPATION_ptype_VolontaryHelp, $startdate, $enddate);
	}

}

/* EOF */
?>