<?php
/*
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *
 * Histoire
 *  * 09/06/2006 DrazziB : Utilisation des methodes Bdd::Insert() et Bdd::Update()
 *  * 13/07/2005 Hooligan0 : Creation d'une classe de gestion des civilites (les
 *      appels depuis Personne sont encore a transformer)
 *  * 21/01/2005 Hooligan0 : Ajout d'une fonction de suppression de civilite
 *  * 19/01/2005 Hooligan0 : Ajout de fonctions de recherche de personnes (par
 *      annee d'adhesion, ou toutes les personnes).
 *  * 27/10/2004 Hooligan0 : Mise en place d'accesseurs pour les variables de
 *      l'objet.
*/
include_once GASELL_BASE . '/lib/Adhesion.php';
include_once GASELL_BASE . '/lib/Objet.php';

include_once 'Date.php';

//define('MEM_ERROR_BADmemNom',    103);
//define('MEM_ERROR_BADmemPrenom', 104);

@define('PERSONNE_pstatus_EMPTY',   -1);
@define('PERSONNE_pstatus_EXIST',   0);
@define('PERSONNE_pstatus_CHANGED', 1);
@define('PERSONNE_pstatus_NEW',     2);
@define('MEM_ERROR_BADmemPrenom', 3);
@define('MEM_ERROR_BADmemNom', 4);

/**
 * Classe Civilite : Gestion des civilites des personnes
 *
 */
class Civilite
{
	var $code;
	var $abr;
	var $libelle;

	function Civilite($code = NULL)
	{
		global $Gasell;
		if ($code != NULL)
		{
			$query = "SELECT civabr,civlib FROM civilite WHERE civcode='".$code."'";
			$result = $Gasell->db->Requette($query, "ALL");
			$this->code    = $code;
			$this->abr     = $result[0][0];
			$this->libelle = $result[0][1];
		}
	}

	/**
	 * Fonction renvoyant la liste de toutes les civilite
	 *
	 */
	function getAll()
	{
		global $Gasell;

		$query = "SELECT civcode,civabr,civlib FROM civilite";
		$results = $Gasell->db->Requette($query, "ALL");

		$civilites = array();
		foreach($results as $result)
		{
			$civilites[$result[0]]['abr'] = $result[1];
			$civilites[$result[0]]['lib'] = $result[2];
		}
		return($civilites);
	}

  	/**
  	 * Fonction d'ajout d'une civilite
  	 *
  	 * @param string $abr : Abreviation, forme courte
  	 * @param string $lib : Denomination de la civilite
  	 */
	function ajoute($abr, $lib)
	{
		global $Gasell;
		
		/* Construction de la requette d'ajout d'une civilite        */
		$query  = "INSERT INTO civilite(civabr, civlib) ";
		$query .= "VALUES('".$abr."','".$lib."')";
		/* Envoi de la requette a la base                            */
		$Gasell->db->Requette($query);
	}

	function delete($code)
	{
		global $Gasell;
		
		$query = "DELETE FROM civilite WHERE civcode='".$code."'";
		$Gasell->db->Requette($query);
	}
}

/**
 * Classe Personne
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 1208 $
 * @package gasell
 */
class Personne
{
	var $Gasell;
	var $pstatus = -1;
	
	var $phascontacts = 0;
	var $phasadhesions = 0;
	
	var $memNum;
	var $memCiv;
	var $memNom;
	var $memPrenom;
	var $memSurnom;
	var $memAdr1;
	var $memAdr2;
	var $memCP;
	var $memVille;
	var $memDatNai;  /* Date au format PEAR-Date */
	var $memSexe;
	var $memDatAdh1;
	var $memDatMaj;
	var $memObs;
	var $civcode;
	
	var $Attributs;
	var $Objets;
	var $Adhesions;

	/**
	 * Constructeur : Charge le detail d'une personne au besoin
	 *
	 * @param  integer $num : Numero de personne a charger
	 */
	function Personne($num = 0)
	{
		global $Gasell;
		$this->Gasell = $Gasell;
		/* Si aucun numero de personne n'est precise                         */
		if ($num == 0)
			/* Alors, pas de chargement a effectuer, retour immediat         */
			return;
		
		$aliste = Adhesion::ListeByMemnum($num);
		$this->Adhesions = array();
		foreach($aliste as $adh)
			array_push($this->Adhesions, new Adhesion($adh[0]));
		
		$q_cols  = "memnum, civcode, memnom, memprenom, memsurnom, adr1, ";
		$q_cols .= "adr2, adrcp, adrville, memdatnai, memsexe, memdatadh1, ";
		$q_cols .= "memdatmaj, memobs";
		$query ='SELECT ' . $q_cols . ' FROM personne WHERE memnum=' . $num;
		$result = $this->Gasell->db->Requette($query, "ALL");
		/* Si le resultat est vide                                           */
		if (! isset($result[0]))
			/* Alors, aucune personne trouvee pour ce numero, retour         */
			return;
		$this->memNum     = $result[0][0];
		$this->memCiv     = $result[0][1];
		$this->memNom     = $result[0][2];
		$this->memPrenom  = $result[0][3];
		$this->memSurnom  = $result[0][4];
		$this->memAdr1    = $result[0][5];
		$this->memAdr2    = $result[0][6];
		$this->memCP      = $result[0][7];
		$this->memVille   = $result[0][8];
		$this->memDatNai  = new Date($result[0][9]);
		$this->memSexe    = $result[0][10];
		$this->memDatAdh1 = $result[0][11];
		$this->memDatMaj  = $result[0][12];
		$this->memObs     = $result[0][13];
		
		/* Mise a jour de l'indicateur d'etat : personne existante chargee   */
		$this->pstatus = PERSONNE_pstatus_EXIST;
		
		/* Construction d'un objet pour acceder aux objets de la personne    */
		$this->Objets = new Objet($this->memNum);
		
		$query = "SELECT attcode,attvalcode FROM attributmembre WHERE memnum='".$this->memNum."'";
		$dbresult = $this->Gasell->db->Requette($query, "ALL");
		$Attributs = array();
		foreach ($dbresult as $attr)
		{
			$query = "SELECT attlib FROM attribut WHERE attcode='".$attr[0]."'";
			$code   = $this->Gasell->db->Requette($query, "ALL");
			
			$query = "SELECT attvallib FROM attributvaleur WHERE attvalcode='".$attr[1]."'";
			$valeur = $this->Gasell->db->Requette($query, "ALL");
			
			$Attributs[$code[0][0]] = $valeur[0][0];
		}
		
		/* */
		return;
	}
  
	function get_num()
	{
		return($this->memNum);
	}
  
	/**
	 * set_num : Change la valeur du numero de personne pour l'objet
	 *
	 * @param  integer : Nouveau numero de personne
	 * @return boolean : Resultat de la demande de modification
	 */
	function set_num($value)
	{
		/* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
		if (! is_object($this))
			/* Alors, le traitement est impossible, retour a l'appelant (+err)    */
			return(0);

		/* Mise a jour de la valeur du numero de personne dans l'objet          */
		$this->memNum = $value;
		/* Mise a jour du flag d'etat pour indiquer que l'objet est modifie     */
		$this->pstatus = PERSONNE_pstatus_NEW;
	}

	function getNom()
	{
		return($this->memNom);
	}

	/**
	 * set_nom : Change la valeur du nom de la personne pour l'objet
	 *
	 * @param  string  : Nouveau nom de personne
	 * @return boolean : Resultat de la demande de modification
	 */
	function set_nom($value)
	{
		/* Si l'appel courrant n'est pas sur la fonction membre d'un objet  */
		if (! is_object($this))
			/* Alors, le traitement est impossible, retour a l'appelant     */
			return(0);
		
		/* Mise a jour de la valeur du nom de la personne dans l'objet      */
		$this->memNom = $value;
		/* Mise a jour du flag d'etat pour indiquer que l'objet est modifie */
		$this->pstatus = PERSONNE_pstatus_NEW;
	}

	function getPrenom()
	{
		return($this->memPrenom);
	}

	/**
	 * set_prenom : Change la valeur du prenom de la personne pour l'objet
	 *
	 * @param  string  : Nouveau prenom de personne
	 * @return boolean : Resultat de la demande de modification
	 */
	function set_prenom($value)
	{
		/* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
		if (! is_object($this))
			/* Alors, le traitement est impossible, retour a l'appelant (+err)    */
			return(0);

		/* Mise a jour de la valeur du nom de la personne dans l'objet          */
		$this->memPrenom = $value;
		/* Mise a jour du flag d'etat pour indiquer que l'objet est modifie     */
		$this->pstatus = PERSONNE_pstatus_NEW;
	}

	/**
	 * getSurnom : Accesseur du membre Surnom de la personne
	 *
	 * @return string Surnom de la personne
	 */
	function getSurnom()
	{
		return($this->memSurnom);
	}
  
	function set_surnom()
	{
		/* Not available yet :) */
	}

	function getCivilite()
	{
		return($this->memCiv);
	}
  
	function set_civilite($value)
	{
		/* Si l'appel courrant n'est pas sur la fonction membre d'un objet      */
		if (! is_object($this))
			/* Alors, le traitement est impossible, retour a l'appelant (+err)    */
			return(0);

		// TODO: controle de coherence de la valeur du code civilite

		/* Mise a jour de la valeur de civilite pour la personne dans l'objet   */
		$this->memCiv = $value;
		/* Mise a jour du flag d'etat pour indiquer que l'objet est modifie     */
		$this->pstatus = PERSONNE_pstatus_CHANGED;
	}

	function getSexe()
	{
		return($this->memSexe);
	}
  
	function set_sexe($value)
	{
		$this->memSexe = $value;
	}
  
	/**
	 * Renvoi la date de la premiere adhesion de la personne
	 *
	 * @return : Date de premiere adhesion
	 */
	function getAdhesion1()
	{
		return($this->memDatAdh1);
	}
  
	function set_adhesion1($value)
	{
		$this->memDatAdh1 = $value;
	}
  
	function getAdresse()
	{
		$result['adresse1'] = $this->memAdr1;
		$result['adresse2'] = $this->memAdr2;
		$result['cp']       = $this->memCP;
		$result['ville']    = $this->memVille;
		return($result);
	}
  
  function set_adresse($value1, $value2=NULL, $valuecp=NULL, $valueville=NULL)
   {
    if ($value1 != NULL)
      $this->memAdr1 = $value1;
    
    if ($value2 != NULL)
      $this->memAdr2 = $value2;
    
    if ($valuecp != NULL)
      $this->memCP = $valuecp;
    
    if ($valueville != NULL)
      $this->memVille = $valueville;
   }
  
	/**
	 * getNaissance : Accesseur du membre "date de naissance" de la personne
	 */
	function getNaissance()
	{
		return($this->memDatNai);
	}
	
  function set_naissance($value)
   {
    $this->memDatNai = new Date($value);
   }
  
  function set_maj($value)
   {
    $this->memMaj = $value;
   }
  
	/**
	 * getObservations : Accesseur du membre "Observations"
	 *
	 * @return string 
	 */
	function getObservations()
	{
		return($this->memObs);
	}

	function set_observations($value)
	{
		$this->memObs = $value;
	}
  
  /**
   * MoyensContact : Recherche les moyens de contact des personnes
   *
   * @return tab_string : Tableau contenant les codes et moyens de contact
   */
  function MoyensContact()
   {
    global $Gasell;
    
    $Moyens = array();
    
    $dbresult = $Gasell->db->Requette("SELECT moyconcode,moyconlib FROM moyencontact", "ALL");
    foreach($dbresult as $data)
      $Moyens[$data[0]] = $data[1];
    return($Moyens);
   }
  
  /**
   * Contacts : Extrait tous les contacts de la personne
   *
   * @return tab_string : Tableau contenant les contacts
   */
  function Contacts()
   {
    global $Gasell;
    $result = array();
    
    $Moyens = $this->MoyensContact();
    
    $query = "SELECT moyconcode,comval FROM contact WHERE memnum='".$this->memNum."'";
    $dbresult = $Gasell->db->Requette($query, "ALL");
    foreach($dbresult as $data)
      $result[$Moyens[$data[0]]] = $data[1];
    return($result);
   }
  
  function ContactsSave($tableau)
   {
    global $Gasell;
    
    $Actuels = $this->Contacts();
    $Moyens = $this->MoyensContact();
    
    foreach($tableau as $code => $val)
     {
      foreach($Moyens as $MCode => $Moyen)
        if($Moyen == $code)
          $moyconcode = $MCode;
      if (!isset($Actuels[$code]))
       {
        if (!empty($val))
         {
          $q_vals = "'".$this->memNum."','".$moyconcode."','".$val."'";
          $query = "INSERT INTO contact(memnum,moyconcode,comval) VALUES(".$q_vals.")";
          $dbresult = $Gasell->db->Requette($query);
         }
       }
      else if ($Actuels[$code] != $val)
       {
        $q_where = "memnum='".$this->memNum."' AND moyconcode='".$moyconcode."'";
        $query = "UPDATE contact SET comval='".$val."' WHERE ".$q_where;
        $Gasell->db-> Requette($query);
       }
     }
    return;
   }
  
  /**
   * Competences : Extraire la liste des competences de la personne
   *
   * @return tab_string : Tableau contenant les competences
   */
  function Competences()
   {
    global $Gasell;
    
    $result = array();

    $query = "SELECT competence.compnum,competence.designation FROM competence, competencemembre WHERE competence.compnum = competencemembre.compnum AND memnum=".$this->memNum;
    $dbresults = $Gasell->db->Requette($query, "ALL");
    
    foreach ($dbresults as $dbresult)
      $result[$dbresult[0]] = $dbresult[1];
    return($result);
   }

  /**
   * Competences_all : Extraire la liste de toutes les competences
   *
   * @return tab_string : Tableau contenant les competences
   */
  function Competences_all()
   {
    global $Gasell;
    
    $result = array();
    $query = "SELECT competence.compnum, competence.designation FROM competence";
    $dbresults = $Gasell->db->Requette($query, "ALL");
    
    foreach ($dbresults as $dbresult)
      $result[$dbresult[0]] = $dbresult[1];
    return($result);
   }

  function Competence_add($compnum)
   {
    global $Gasell;
    
    $query = "INSERT INTO competencemembre(memnum,compnum) VALUES ('".$this->memNum."', '".$compnum."')";
    $Gasell->db->Requette($query);
    return;
   }
  
  function Competence_del($compnum)
   {
    global $Gasell;
    $query = "DELETE FROM competencemembre WHERE memnum='".$this->memNum."' AND compnum='".$compnum."'";
    $Gasell->db->Requette($query);
    return;
   }
  
  /**
   * Objets : Extraire la liste des objets que possede la personne
   *
   * @return tab_string : Tableau contenant les objets
   */
  function Objets()
   {
    global $Gasell;
    
    $result = array();
    $query = "SELECT objnum,objlib FROM objets WHERE memnum=".$this->memNum;
    $dbresults = $Gasell->db->Requette($query, "ALL");
    
    foreach ($dbresults as $dbresult)
      $result[$dbresult[0]] = $dbresult[1];
    return($result);
   }
  
	/**
	 * Save : Sauvegarder l'enregistrement de la personne
	 *
	 */
	function Save()
	 {
		global $Gasell;
		
		$date = date('Y-m-d', time());
		
		$fields = array();
		$fields['memnom'] = $this->memNom;
		$fields['memdatmaj'] = $date;
		if(!empty($this->memPrenom))
		 {
			$fields['memprenom'] = $this->memPrenom;
		 }
		$fields['memsurnom'] = $this->memSurnom;
		$fields['adr1'] = $this->memAdr1;
		$fields['adr2'] = $this->memAdr2;
		/* Pour etre valide, le code postal doit etre un entier */
		if (is_numeric($this->memCP))
		 {
			$fields['adrcp'] = (int) $this->memCP;
		 }
		else
		 {
			/* Sinon, le code postal est efface (NULL) */
			$fields['adrcp'] = NULL;
		 }
		$fields['memsexe'] = $this->memSexe;
		if(!empty($this->memDatNai))
		 {
			$fields['memdatnai'] = $this->memDatNai;
		 }
		$fields['adrville'] = $this->memVille;
		$fields['memobs'] = $this->memObs;
		
		/* Clause de selection a partir du numero de membre */
		$const = array('memnum = ' . $this->memNum);
		
		$Gasell->db->Update('personne', $fields, $const);
	 }
  
  /**
   * Check if the object is valid
   *
   * @access public
   *
   * @return boolean
   */
  function ValidCheck()
   {
    if (empty($this->memNom))
      return(MEM_ERROR_BADmemNom);
    if (empty($this->memPrenom))
      return(MEM_ERROR_BADmemPrenom);
    return (0);
   }

	function Ajoute()
	 {
		global $Gasell;
		/* Construction de la liste des champs a inserer */
		$fields = array('memnom' => $this->memNom);
		/* Envoi des donnees a la base */
		$dbresult = $Gasell->db->Insert('personne', $fields, 'memnum');
		if (!$dbresult)
		 {
			return(NULL);
		 }
		else
		 {
			$this->memNum = $dbresult;
			/* Appel la fonction de maj pour les autres infos */
			$this->Save();
			return($this->memNum);
		 }
	 }

  function Cherche($arguments)
   {
    global $Gasell;
    
    $resultat = array();;
    $q = '';
    foreach ($arguments as $anom => $aval)
      if ($q == '')
        $q = "(" . $anom . " ILIKE('%" . $aval . "%'))";
      else
        $q .= " AND (" . $anom . " ILIKE('%" . $aval . "%'))";
    $query = 'SELECT memnum FROM personne WHERE '.$q;
    $result = $Gasell->db->Requette($query, 'ALL');
    if (is_array($result))
      foreach($result as $pers)
        $resultat[$pers[0]] = new Personne($pers[0]);
    return($resultat);
   }
   
   	/**
   	 * Recherche la liste de *toutes* les personnes
   	 *
   	 * @return tab Tous les numeros de personnes
   	 */
	function searchAll()
	{
		global $Gasell;
		
		/* Construction de la requette de recherche en fonction de l'exercice*/
		$query = 'SELECT DISTINCT personne.memnum FROM personne';
		
		/* Envoi de la requette a la base de donnees                         */
		$dbresult = $Gasell->db->Requette($query, "ALL");
		
		/* Renvoi l'enssemble resultat a l'appelant                          */
		return($dbresult);
	}
	
   	/**
   	 * Recherche des personnes en fonction de leur adhesion a un exercice
   	 *
   	 * @param integer $exenum : Numero de l'exercice
   	 * @return tab : Enssemble des numeros de personnes pour l'exercice
   	 */
	function searchByExenum($exenum)
	{
		global $Gasell;
		
		/* Construction de la requette de recherche en fonction de l'exercice*/
		$q_tables = "personne,adhesion,typeadhesion ";
		$q_cont   = "WHERE adhesion.memnum=personne.memnum ";
		$q_cont  .= "AND adhesion.typadhnum=typeadhesion.typadhnum ";
		$q_cont  .= "AND typeadhesion.exenum='" . $exenum . "'";
		$query = 'SELECT DISTINCT personne.memnum FROM ' . $q_tables . $q_cont;
		
		/* Envoi de la requette a la base de donnees                         */
		$dbresult = $Gasell->db->Requette($query, "ALL");
		
		/* Renvoi l'enssemble resultat a l'appelant                          */
		return($dbresult);
	}
  
  /**
   * Search the error message from the error code
   *
   * @access public
   *
   * @return string : Error Message
   */
  function errorMessage($ecode)
   {
    $errorMessages = array(
      MEM_ERROR_BADmemNom     => 'Champ "nom" incorrect',
      MEM_ERROR_BADmemPrenom  => 'Champ "prenom" incorrect');
    return($errorMessages[$ecode]);
   }

 }
/* EOF */
?>
