<?php
/*
 * Gasell: Copyright 2003-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *          Raude-Morvan Damien (DrazziB) <drazzib@drazzib.com>
 *
 * Histoire:
 *  * 25/06/2006 DrazziB : Utilisation des Exceptions a la place de la methode Erreur()
 *  * 28/01/2005 Hooligan0 Toutes les fonctions de BddTarif sont maintenant gere dans
 *      la classe Tarif. Corrections diverses pour etre compatible avec le nouveau
 *      format d'utilisation de Bdd.
 */

define('ERR_BDD_TarifGet_01',         20101);

require_once GASELL_BASE . '/lib/Bdd.php';

/**
 * Gasell Tarif class : 
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 1157 $
 * @package gasell
 */
class Tarif
{
	var $GasellSession; /* Pointeur sur l'objet Gasell de la session          */
	var $TarifParam;    /* */
	var $TarifTab;

	/**
	 * Constructeur de "Tarif"
	 *
	 * @param Gasell $Session  : Session Gasell de reference
	 * @param string $exercice : Numero de l'exercice a charger
	 */
	function Tarif($Session, $exercice = '')
	{
		/* Sauvegarde la le pointeur sur l'objet Gasell de la session        */
		$this->GasellSession = $Session;

		/* Initialise le tableau des parametres de tarif                     */
		$this->TarifParam = array();
		/* Si un numero d'exercice a ete precise                             */
		if ($exercice)
			/* Recherche les parametres de tarif pour cet exercice           */
			$tables = $this->GasellSession->CfgGet("TARIF", "Params", $exercice);
		/* Sinon, utilisation de l'exercice actuel                           */
		else
		{
			/* Recherche dans la config les parametres de tarif              */
			$tables = $this->GasellSession->CfgGet("TARIF", "Params");
			/* */
			$exercice = $this->GasellSession->CfgGet("ASSO", "Exercice");
		}

		if ($tables != '')
		{
			/* Pour chaque parametre de tarif declare dans la config         */
			foreach(explode("*", $tables) as $param)
			{
				$newparam = array();
				$val = explode(",", $param);
				$newparam['nom']    = $val[0];
				$newparam['col']    = $val[1];
				$newparam['table']  = $val[2];
				$newparam['libele'] = $this->_subTableGet($val[2], $exercice);
				/* Si la table des parametres ne contient aucune entree      */
				if(count($newparam['libele']) == 0)
					/* Alors, ajout d'une entree virtuelle pour eviter des erreurs    */
					$newparam['libele']['n/a'] = 'n/a';

				array_push($this->TarifParam, $newparam);
			}
		}
		/* Lecture de toute la table des tarifs depuis la BDD                */
		$this->TarifTab = $this->_get($this->TarifParam, $exercice);
	}
  
	/**
	 * Modification d'une entree du tarif
	 *
	 * @access public
	 * @param string $ids    : Tableau des colones des parametres tarif
	 * @param string $valeur : Nouvelle valeur pour le tarif
	 */
	function Modif($ids, $valeur, $exenum)
	{
		$this->_modify($ids, $valeur, $exenum);
		return;
	}
  
	/**
	 * Creation d'un nouveau parametre de tarif
	 *
	 * @access public
	 * @param string $nom    : Nom du nouveau parametre
	 * @param string $nbcols : Nombre de colones de la table associee
	 */
	function ParamCreate($nom, $nbcols, $exercice = '')
	{
		global $Gasell;
		
		/* Si le numero de l'exercice n'a pas ete precise                   */
		if ($exercice == '')
		/* Alors, utilisation de l'exercice actuel                          */
		$exercice = $this->GasellSession->CfgGet("ASSO", "Exercice");
    
		/* Creation du nom de la table avec le prefix 'tr' et le nom en min */
        // FIXME SECURITY : Attention, risque d'injection de données dans la BDD
		$tablenom = 'tr'.strtolower($nom);
		/* Creation du tableau de definition des colones de la table        */
		$tablecols = array();
		array_push($tablecols, array("exenum",             "text"));
		array_push($tablecols, array('T'.$tablenom.'code', "text"));
		array_push($tablecols, array($tablenom.'lib',      "text"));
		/* Creation d'une nouvelle table pour le parametre de tarif         */
		$resultNT = $Gasell->db->TableCreate($tablenom, $tablecols);

		/* Ajoute le lien a ce nouveau parametre dans la table des tarifs       */
		$resultAC = $Gasell->db->TableAddCol("Tarif", $tablenom.'code', "text");

		/* Lecture de la config actuelle des parametres de tarif                */
		$old = $this->GasellSession->CfgGet("TARIF", "Params");
		/* Si un ou des parametres de config existent deja                      */
		if ($old)
			/* Alors, ajout d'un separateur dans la ligne de config               */
			$new = $old . '*';
		else
			/* Sinon, La chaine de config actuelle est vide                       */
			$new = "";
		/* Ajoute la reference du nouveau parametre dans la chaine de config    */
		$new = $new . $nom.','.$tablenom.'code,'.$tablenom;
		/* Mise a jour de la config dans l'appli (et la base)                   */
		$this->GasellSession->CfgSet("TARIF", "Params", $new);
    
		return;
	}
  
	/**
	 * Suppression d'un parametre de tarif
	 *
	 * @param string $nom    : Nom du parametre a supprimer
	 */
	function ParamDrop($nom)
	{
		global $Gasell;
		$tablenom = 'tr'.strtolower($nom);
		
		/* Supprime le lien a ce parametre dans la table des tarifs  */
		$resultAC = $Gasell->db->TableDropCol("Tarif", $tablenom.'code');
		/* TODO : Tester si le parametre est utilise sur un autre exercice ! */
		/* !!!! */
		$resultNT = $Gasell->db->TableDrop($tablenom);

		/* Lecture de la config actuelle des parametres de tarif                */
		$old = $this->GasellSession->CfgGet("TARIF", "Params");
		$oldparams = explode('*', $old);
		foreach($oldparams as $key => $param)
		{
			$datas = explode(',', $param);
			if ($nom == $datas[0])
				unset($oldparams[$key]);
		}
		$new = "";
		foreach($oldparams as $param)
		{
			if ($new)
				$new = $new.'*';
			$new = $new . $param;
		}

		/* Mise a jour de la config dans l'appli (et la base)                   */
		$this->GasellSession->CfgSet("TARIF", "Params", $new);
		return;
	}

	/**
	 * Ajout d'une entree dans un parametre de tarif
	 *
	 * @access public
	 * @param string $param    : Nom du parametre dans lequel se fait l'ajout
	 * @param string $nom      : Code de la nouvelle entree dans le param
	 * @param string $libele   : Libele de la nouvelle entree dans le param
	 * @param string $exercice : Numero de l'exercice
	 *
	 * @result bool renvoie un flag indiquant la reussite ou l'echec
	 */
	function ParamAdd($param, $code, $libele, $exercice='')
	{

		/* Si le numero de l'exercice n'a pas ete precise                       */
		if ($exercice == '')
			/* Alors, utilisation de l'exercice actuel                            */
			$exercice = $this->GasellSession->CfgGet("ASSO", "Exercice");

		$datas[0] = $code;
		$datas[1] = $libele;
		return $this->_subTableIns(strtolower($param), $datas, $exercice);
	}

	/**
	 * Suppression d'une entree dans un parametre de tarif
	 *
	 * @access public
	 * @param string $param    : Nom du parametre concerne par la suppression
	 * @param string $nom      : code de l'entree a supprimer
	 * @param string $exercice : le numero d'exercice concerne
	 *
	 * @result bool renvoie un flag indiquant la reussite ou l'echec
	 **/
	function ParamDel($param, $nom, $exercice='')
	{
		/* TODO : Rechercher les elements de tarifs qui dependent de l'element */
		/* Si le numero de l'exercice n'a pas ete precise                       */
		if ($exercice == '')
		/* Alors, utilisation de l'exercice actuel                            */
			$exercice = $this->GasellSession->CfgGet("ASSO", "Exercice");

		return $this->_subTableDel(strtolower($param), $nom, $exercice);
	}

	/**
	 */
	function _get($Params, $exercice)
	{
		global $Gasell;
		
		/* preparation des colonnes */
		$cols  = array( 'tarnum', 'tarmontant');
		$cons = array($cons[] = "tarif.exenum='".$exercice."'");
		$tables = "tarif";

		foreach($Params as $Param)
		{
			$cols[]  = $Param['col'];
			$tables .= ','.$Param['table'];
			$cons[]  = ' AND '.$Param['table'].'.T'.$Param['col'].'=Tarif.'.$Param['col'];
		}

		$dbresult = $Gasell->db->Select($tables, $cols, $cons);
		//$dbresult = $Gasell->db->Requette($query, "ALL");
		/* Si une erreur s'est produite lors de l'acces a la base            */
		if (!is_array($dbresult))
			/* Alors, arret des traitements et affichage d'une erreur        */
			Except::throwError("Tarifs", "Code ".ERR_BDD_TarifGet_01);
		return($dbresult);
	}
	
	function _subTableGet($NomTable, $exercice)
	{
		global $Gasell;
		
		$cols = array( 'T'.$NomTable.'code', $NomTable.'lib' );
		$cons = array( "exenum='".$exercice."'");
		$datas = $Gasell->db->Select($NomTable, $cols, $cons);

		$result = array();

		if (count($datas) > 0) {
			foreach($datas as $data) {
				$result[$data[0]] = $data[1];
			}
		}
		return($result);
	}

	function _subTableIns($table, $data, $exercice)
	{
		global $Gasell;
		$values = array('exenum' => $exercice,
				'T'.$table.'code'   => $data[0],
				$table.'lib'    => $data[1]);
		$dbresult = $Gasell->db->Insert($table, $values);
		return $dbresult;
	}
	
	function _modify($ids, $valeur, $exenum)
	{
		global $Gasell;
		
		$query = 'UPDATE Tarif SET tarMontant=' . $valeur . " WHERE exenum='".$exenum."' ";
		foreach($ids as $pk => $param)
			$query = $query . ' AND ' . $pk . "='" . $param . "'";

		$dbresult = $Gasell->db->Requette($query);

//		if ($db->affectedRows())
//			return;
    
		$queryP='';
		foreach ($ids as $pkey => $pval)
		{
			if ($queryP)
				$queryP = $queryP.','.$pkey;
			else
				$queryP = $pkey;
		}

		$query = 'INSERT INTO Tarif (tarMontant,exenum,'.$queryP.') ';
		$query = $query . 'VALUES ('.$valeur.','."'".$exenum."'";
		foreach($ids as $param)
			$query = $query . ", '" . $param . "'";
		$query = $query . ')';
		
		$dbresult = $Gasell->db->Requette($query);

		return;
	}

	function _subTableDel($table, $data, $exercice)
	{
		global $Gasell;
		$query = 'DELETE FROM '.$table.' WHERE exenum=\''.$exercice.'\' AND T'.$table.'code=\''.$data.'\'';
		$dbresult = $Gasell->db->Requette($query);
		return $dbresult;
	}

	/**
	 * Construction d'une representation plane du tarif (en 2D)
	 *
	 * @param $ncols
	 * @param $toffs
	 * @params $tafft
	 **/
	function _flatTarif(&$ncols, &$toffs, &$tafft)
	{
		$toffs['table'] = array();
	    
		/* Il faut determiner le nombre de colones du tableau a plat */
		$ncols = 1;
		reset($this->TarifParam);

		/* 1er parametre ignore => traitement a part (lignes) */
		$k = each($this->TarifParam);
		/* Pour tous les parametres de tarif de 2 a N  */
		while(list($k, $ttab) = each($this->TarifParam))
		{
			$nb = count($ttab['libele']);
			$ncols = $ncols * $nb;
		}

		/* Calcul les offsets de chaque parametre dans le tableau a plat */
		$nbc  = 1;
		/* Reset le pointeur d'elt courrant du tableau des parametres tarif */
		reset($this->TarifParam);
		/* 1er parametre ignore => traitement est a part (lignes) */
		$k = each($this->TarifParam);
		/* Pour tous les parametres de tarif de 2 a N */
		while(list($k, $ttab) = each($this->TarifParam))
		{
			$nb = count($ttab['libele']);
			$nspan = ($ncols/$nbc)/$nb;
			$nbx = 0;
			$toffs['table'][$ttab['nom']] = $nspan;
			foreach($ttab['libele'] as $libk => $libv)
			{
				$toffs['cols'][$libk] = $nbx;
				$nbx += $nspan;
			}
			$nbc = $nbc * $nb;
		}

		if (!count($this->TarifParam))
		{
			echo '<!-- Pas assez de parametres de tarifs -->';
			return;
		}

		/* Creation d'un tableau plat des montants de tarifs */
		foreach($this->TarifParam[0]['libele'] as $tkey => $tlib)
			$tafft[$tkey] = array_pad(array(), $ncols, '.');
		
		/* Pour chaque entree de tarif valide */
		foreach($this->TarifTab as $ttrf)
		{
			$cnum = 0;
			reset($ttrf);
			list($numk,     $num)     = each($ttrf);
			list($montantk, $montant) = each($ttrf);
			list($tabk,     $tabv)    = each($ttrf);
			
			while(list($k, $v) = each($ttrf))
				$cnum += $toffs['cols'][$v];

			$tafft[$tabv][$cnum] = $montant;
		}
		return;
	}
}
/* EOF */
?>
