<?php
/*
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@linux-nantes.fr.eu.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *
 * Histoire:
 *  * 13/07/05 Hooligan0 : Ajout du traitement des objets adhesion et civilite
 *  * 11/02/05 Hooligan0 : Modification de comportement de la fct searchSection
 *  *   et suppression de la dep sur PEAR XML/Tree.
 *  * 18/11/04 Hooligan0 : Un peu de remise en forme
 *  * 26/10/04 Hooligan0 : Creation du fichier.
 */

include_once GASELL_BASE . '/lib/Personne.php';

@define('gXML_ERR_NONE', 0);
@define('gXML_ERR_FILE', 1);
@define('gXML_ERR_ROOT', 2);

/**
 * Gasell XML
 *
 * @author Gwenael Saint-Genest <gwen@hooligan0.net>
 * @version $Revision: 405 $
 * @package gasell
 */
class gXML
{
	var $file = NULL;
	var $type;
	var $erreur = 0;
	var $root;
	var $doc;
	var $currentPos;

	/**
	 * Constructeur de "gXML"
	 *
	 * @access public
	 */
	function gXML($file = NULL)
	{
		/* Si aucun fichier n'a ete passe au constructeur                    */
		if ($file == NULL)
			/* Alors, aucune autre initialisation, retour a l'appelant       */
			return;

		/* Ouverture du fichier XML passe en parametre                       */
		$dxml = @domxml_open_file($file);
		/* Si une erreur est survenue lors de l'ouverture                    */
		if ($dxml == NULL)
		{
			/* Alors, enregistrement de celle-ci et retour a l'appelant      */
			$this->erreur = gXML_ERR_FILE;
			return;
		}
		/* Sauvegarde dans l'objet le nom du fichier qui vient d'etre ouvert*/
		$this->file = $file;
		/* Chargement de la racine du document XML                          */
		$root = $dxml->document_element();
		/* Si la racine du document n'est pas un element 'gasell'           */
		if ($root->node_name() != 'gasell')
		{
			/* Alors, il s'agit d'une erreur fatale, retour a l'appelant    */
			$this->erreur = gXML_ERR_ROOT;
			return;
		}
		/* Sauvegarde dans l'objet la reference a l'element racine          */
		$this->root = $root; 
		/* Sauvegarde dans l'objet de l'attribut 'type' du document         */
		$this->type = $root->get_attribute('type');
		/* Fin normale des initialisation, retour a l'appelant              */
		return;
	}

	function init()
	{
		/* Creation d'un document XML vide */
		$this->doc = domxml_new_doc("1.0");
		$this->root = $this->doc->create_element("gasell");
		$this->root = $this->doc->append_child($this->root);
	}

	/**
	 * 
	 */
	function dump()
	{
		return($this->doc->dump_mem(true, "UTF-8"));
	}
	
	/**
	 * get_type : Accesseur pour le parametre 'type' du document
	 *
	 * @return string : Type du document detecte lors de l'ouverture
	 */
	function get_type()
	{
		/* Controle si l'appel est bien sur une une fonction membre d'un objet  */
		if (! is_object($this))
			/* Ce n'est pas une fonction membre d'un objet instancie, sortie */
			return(NULL);

		/* Renvoi le code de type du document XML qui a ete charge           */
		return($this->type);
	}

	/**
	 * searchSection : Rechercher une section en particulier
	 *
	 * @param  string $section    : Nom de la section a rechercher
	 * @param  string $searchroot : Racine de recherche
	 * @return string             : DOM sur la section recherchee ou NULL
	 */
	function searchSection($section, $searchroot = NULL)
	{
		/* Controle si l'appel est bien sur une une fonction membre d'un objet*/
		if (! is_object($this))
			/* Ce n'est pas le cas, retour immediat a l'appelant             */
			return;

		/* Si aucune racine de recherche n'a ete precisee lors de l'appel    */
		if ($searchroot == NULL)
			/* Alors, la racine du document XML est utilisee                 */
			$searchroot = $this->root;

		/* Pour chaque element present immediatement sous l'element racine   */
		foreach($searchroot->child_nodes() as $node)
		{
			/* Si l'element est un champ de texte                            */
			if ($node->node_type() == XML_TEXT_NODE)
				/* Alors il doit etre ignore, renvoi en debut de boucle      */
				continue;
			/* Extraction du nom de l'element courrant                       */
			$name = $node->node_name();
			/* Si le nom de l'element correspond a la section recherchee     */
			if ($name == $section)
			{
				/* Alors, recherche terminee, sauvegarde la nouvelle position*/
				$this->currentPos = $node;
				/* Renvoi cet element a l'appelant                           */
				return($node);
			}
		}
		/* Retour a l'appelant avec le tableau de l'enssemble resultat       */
		return(NULL);
	}

	/**
	 * get_object : Creer un objet de gasell a partir d'un element XML
	 *
	 * @param  string : Reference a l'element XML racine des donnees
	 * @return objet  : Un objet gasell correspondant au type de donnees
	 */
	function get_object($xref)
	{
		/* Extrait le nom de l'element racine pour determiner le type        */
		$localtype = $xref->node_name();
		/* Si l'element definit une 'personne'                               */
		if ($localtype == 'personne')
			/* Alors, appelle la fonction particuliere de traitement des personnes */
			return($this->get_object_personne($xref));
		elseif ($localtype == 'adhesion')
			return($this->get_object_adhesion($xref));
		print "gXML ".$xref->node_name();
	}

	/**
	 * get_object_personne : Charge un objet personne depuis le fichier XML
	 *
	 * @param  objet domxml   : Element XML qui doit etre charge
	 * @return objet personne : Objet personne correcpondant aux infos chargees
	 */
	function get_object_personne($xref)
	{
		$result = array();
		$contacts   = 0;
		$xcontacts  = NULL;
		$adhesions  = 0;
		$xadhesions = NULL;
		/* Creation d'un nouvel objet Personne pour y placer les donnees lues   */
		$personne = new Personne();

		/* Lecture dans le XML de l'attribut numero de personne                 */
		$memnum = $xref->get_attribute('memnum');
		/* Si l'attribut existe et est non-nul                                  */
		if ($memnum != NULL)
			/* Alors, placement de cette valeur dans l'objet resultat           */
			$personne->set_num($memnum);
		
		/* Pour chacun sous element de l'entree 'personne'                      */
		foreach ($xref->child_nodes() as $node)
		{
			/* Extraction du nom du sous-element                                */
			$param = $node->node_name();
			/* Extraction des donnees (s'il y en a) constituant le sous-element */
			$content = $node->get_content();
			if ($param == 'nom')
				$personne->set_nom($content);
			else if ($param == 'prenom')
				$personne->set_prenom($content);
			else if ($param == 'surnom')
				$personne->set_surnom($content);
			else if ($param == 'civilite')
				$personne->set_civilite($node->get_attribute('code'));
			else if ($param == 'sexe')
				$personne->set_sexe($content);
			else if ($param == 'adhesion1')
				$personne->set_adhesion1($content);
			else if ($param == 'adresse1')
				$personne->set_adresse($content, NULL);
			else if ($param == 'adresse2')
				$personne->set_adresse(NULL, $content);
			else if ($param == 'cp')
				$personne->set_adresse(NULL, NULL, $content);
			else if ($param == 'ville')
				$personne->set_adresse(NULL, NULL, NULL, $content);
			else if ($param == 'naissance')
				$personne->set_naissance($content);
			else if ($param == 'maj')
				$personne->set_maj($content);
			else if ($param == 'observations')
				$personne->set_observations($content);
			else if ($param == 'contacts')
			{
				$xcontacts = $node;
				$contacts = count($node->child_nodes());
			}
			else if ($param == 'adhesions')
			{
       			$xadhesions = $node;
				$adhesions = count($node->child_nodes());
			}
			else
				print $node->node_name() ."<br>\n";
		}
		/* Place un status indiquant une fin sans erreur dans le resultat    */
		$result['status'] = 0;
		/* Place l'objet personne dans le resultat                           */
		$result['object'] = $personne;
		/* Place le nombre de contacts trouves dans le resultat              */
		$result['contacts'] = $contacts;
		/* Place la reference de la section contacts dans le resultat        */
		$result['xcontacts'] = $xcontacts;
		/* Place le nombre d'adhesions trouvees dans le resultat             */
		$result['adhesions'] = $adhesions;
		/* Place la reference de la section adhesions dans le resultat       */
		$result['xadhesions'] = $xadhesions;
		/* Retour a l'appelant en renvoyant l'enssemble resultat             */
		return($result);
	}
  
	/**
	 * get_object_personne : Charge un objet personne depuis le fichier XML
	 *
	 * @param  objet domxml   : Element XML qui doit etre charge
	 * @return objet personne : Objet personne correcpondant aux infos chargees
	 */
	function get_object_adhesion($xref)
	{
		$result = array();
		$newAdhesion = new Adhesion;

		$newAdhesion->adhNum = $xref->get_attribute('code');
		$newAdhesion->addReglement();
		$result['status'] = 0;
		$result['object'] = $newAdhesion;

		return($result);
	}

	/**
	 * Ajout d'un objet dans le document XML
	 *
	 * @param $obj    : Objet a ajouter
	 * @param $racine : Element racine dans le document XML pour l'ajout
	 */
	function addObject($obj, $racine = NULL)
	{
		if (get_class($obj) == "personne")
			$this->addObjectPersonne($obj);
		else if (get_class($obj) == "civilite")
			$this->addObjectCivilite($obj);
		else if (get_class($obj) == "adhesion")
			$this->addObjectAdhesion($obj);
		else
			echo "Unknown ". get_class($obj);
	}
	
	/**
	 * Ajout des informations d'un objet de type personne au XML
	 *
	 * @param  $obj : Objet Personne a ajouter.
	 * @return      : Identifiant du nouvel element dans le XML
	 */
	function addObjectPersonne($obj)
	{
 		/* Recherche les entree portant le nom "personnes" dans le XML       */
		$root_pers = $this->doc->get_elements_by_tagname("personnes");
		/* Si aucun n'est trouve, alors il faut le creer                     */
		if (empty($root_pers))
		{
			/* Ajout d'un element "personnes" au document XML                */
			$root_pers = $this->doc->create_element("personnes");
			/* Placement de cet element juste apres la racine                */
			$root_pers = $this->root->append_child($root_pers);
		}
		/* Au moins un element "personnes" a ete trouve (normalement 1 seul) */
		else
			/**/
			$root_pers = $root_pers[0];
		$xpersonne = $this->doc->create_element("personne");
		$xpersonne = $root_pers->append_child($xpersonne);
		$xpersonne->set_attribute("memnum", $obj->get_num());
		
		/* Enregistrement du nom de la personne                              */
		$xpersNom = $this->doc->create_element("nom");
		$xpersNom = $xpersonne->append_child($xpersNom);
		$xpersNom->set_content($obj->getNom());
		
		/* Enregistrement du prenom de la personne                           */
		$xpersPrenom = $this->doc->create_element("prenom");
		$xpersPrenom = $xpersonne->append_child($xpersPrenom);
		$xpersPrenom->set_content($obj->getPrenom());
		
		/* Civilite */
		
		/* Enregistrement du surnom si la personne en possede un            */
		$surnom = $obj->getSurnom();
		if ($surnom != "")
		{
			$xpersSurnom = $this->doc->create_element("surnom");
			$xpersSurnom = $xpersonne->append_child($xpersSurnom);
			$xpersSurnom->set_content($surnom);
		}
		
		/* */
		$xpersNaissance = $this->doc->create_element("DateNaissance");
		$xpersNaissance = $xpersonne->append_child($xpersNaissance);
		$d = $obj->getNaissance();
		$xpersNaissance->set_content($d->format("%d/%m/%Y"));
		
		/* Enregistrement du sexe de la personne                             */
		$xpersSexe = $this->doc->create_element("sexe");
		$xpersSexe = $xpersonne->append_child($xpersSexe);
		$xpersSexe->set_content($obj->getSexe());
		
		/* Enregistrement de la date de premiere adhesion                    */
		$xpersAdh1 = $this->doc->create_element("adhesion1");
		$xpersAdh1 = $xpersonne->append_child($xpersAdh1);
		$xpersAdh1->set_content($obj->getAdhesion1());
		
		$adresse = $obj->getAdresse();
		
		/* Adresse1 */
		$xpersAdr1 = $this->doc->create_element("adresse1");
		$xpersAdr1 = $xpersonne->append_child($xpersAdr1);
		$xpersAdr1->set_content($adresse['adresse1']);

		/* Adresse2 */
		$xpersAdr2 = $this->doc->create_element("adresse2");
		$xpersAdr2 = $xpersonne->append_child($xpersAdr2);
		$xpersAdr2->set_content($adresse['adresse2']);

		/* CP */
		$xpersCP = $this->doc->create_element("cp");
		$xpersCP = $xpersonne->append_child($xpersCP);
		$xpersCP->set_content($adresse['cp']);

		/* Ville */
		$xpersVille = $this->doc->create_element("ville");
		$xpersVille = $xpersonne->append_child($xpersVille);
		$xpersVille->set_content($adresse['ville']);

		/* Contacts */
		
		/* Enregistremnt des observations si la personne en possede          */
		$Observations = $obj->getObservations();
		if ($Observations != "")
		{
			$xpersObs = $this->doc->create_element("observations");
			$xpersObs = $xpersonne->append_child($xpersObs);
			$xpersObs->set_content($Observations);
		}
		
		return ($xpersonne);
	}
	
	function addObjectCivilite($obj)
	{
 		/* Recherche les entree portant le nom "personnes" dans le XML       */
		$root_pers = $this->doc->get_elements_by_tagname("personnes");
		/* Si aucun n'est trouve, alors il faut le creer                     */
		if (empty($root_pers))
		{
			/* Ajout d'un element "personnes" au document XML                */
			$root_pers = $this->doc->create_element("personnes");
			/* Placement de cet element juste apres la racine                */
			$root_pers = $this->root->append_child($root_pers);
		}
		/* Au moins un element "personnes" a ete trouve (normalement 1 seul) */
		else
			/**/
			$root_pers = $root_pers[0];

		/* Ajout d'un element "personnes" au document XML                */
		$root_civ = $this->doc->create_element("civilites");
		/* Placement de cet element juste apres la racine                */
		$root_pers = $root_pers->append_child($root_civ);
		
		$xciv = $this->doc->create_element("civilite");
		$xciv = $root_civ->append_child($xciv);
		$xciv->set_attribute("civcode", $obj->code);
		
		$xcivcode = $this->doc->create_element("code");
		$xcivcode = $xciv->append_child($xcivcode);
		$xcivcode->set_content($obj->code);
		
		$xcivcode = $this->doc->create_element("abr");
		$xcivcode = $xciv->append_child($xcivcode);
		$xcivcode->set_content($obj->abr);
		
		$xcivcode = $this->doc->create_element("libelle");
		$xcivcode = $xciv->append_child($xcivcode);
		$xcivcode->set_content($obj->libelle);
		
	}
	
	function addObjectAdhesion($obj)
	{
		/**/
	}
}
/* EOF */
?>
