<?php
/*
 * Gasell: Copyright 2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Lionel Corgnet (leon) <leon@gmx.fr>
 *          Saint-Genest Gwenael (Hooligan0) <gwen@hooligan0.net>
 *
 * description du fichier gasell/login.php :
 *  appele par base.php, ce fichier permet de :
 *  - verifier qu'un utilisateur est authentifie,
 *  - le cas echeant, demander une authentification,
 *  - verifier une authentification.
 *
 *
 * Historique :
 *  - 09 juillet 2005 : Utilisation d'un objet GasellPage pour l'affichage
 *      de la page de login.
 *  - 19 mai 2005 :
 *    . creation du fichier
 *  - 16 juin 2005 :
 *    . utilisation de la version objet de lib/Authentication.php
 *
 */

@define('GASELL_LOGIN', dirname(__FILE__));

require_once GASELL_LOGIN . "/lib/base.php";
require_once GASELL_BASE . "/lib/Authentication.php";

/* initialisation de l'authentification, par rapport a la session si possible */
$auth = new Authentication (true);

/* si deconnexion a la demande de l'utilisateur (avec reason=logout) */
if (isset($_GET['reason']) && $_GET['reason'] == "logout") {
	/* deconnexion */
	$auth->setRequired(true);
}

/* si l'authentification n'est pas faite */
if (!$auth->isOk()) {
	/* une authentification est necessaire */

	if ($_SERVER['REQUEST_METHOD'] == "POST" &&
			isset($_POST['user']) && isset($_POST['pass'])) {
		/* l'appel provient du formulaire d'authentification */

		/* enregistrement du login */
		$auth->setProposedUser($_POST['user']);

		/* si le mot de passe est correct */
		if ($auth->checkPassword($_POST['pass'])) {

			/* l'authentification est confirmee */
			$auth->validateUser();

			/* enregistrement des l'authentification en session */
			$auth->sessionWrite();

			/* redirection vers la page d'accueil */
			$ref_url = $_SERVER['HTTP_REFERER'];
			$url_racine = substr($ref_url, 0, strrpos($ref_url, "/")+1);
			header("Location: ".$url_racine);
			exit;
		}
		/* sinon */
		else {
			/* l'authentification a echoue */
			$auth->setRefused();
			$auth->sessionWrite();
		}
	}
	else {
		/* il faut appeler le formulaire d'authentification */
		$auth->setVerifying();
		$auth->sessionWrite();
		@define('GASELL_TEMPLATES', GASELL_LOGIN.'/templates');
		include_once GASELL_TEMPLATES . '/_pages-defs.php';
		$Gasell = new Gasell();
		$Gasell->PageInit();
		$Gasell->Page->Load('_LOGIN_', NULL, NULL);
		exit;
	}
}
?>
