<?php

/**
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @file setup/_ext_database.php
 *
 * @brief Ce script regroupe les primitives utilisees dans les fichiers de 
 * configuration pour tester les fonctionnalites de la base de donnees
 *
 * Le choix est fait de nommer les primitives en lettres majuscules pour eviter
 * des possibles erreurs d'interpretation dans le fichier de config XML liees 
 * au choix de codage du charset
 *
 * @date 13-10-2005
 * @author GrdScarabe <gasell-dev@grdscarabe.net>
 **/

/* ==================== INITIALISATION DE L'EXTENSION ==================== */

@define('GASELL_BASE', dirname(__FILE__)."/..");

// Function that load a file and check if it exists
function include_from($dir, $file) {
  /* check if the file exists */
  if(file_exists($dir."/".$file)) {
    include_once($dir."/".$file);
    return true;
  } else {
    return false;
  }
}

/* check first if the Bdd.php file is in the current directory */
if(!include_from(".","Bdd.php")) {
  if(!include_from("../lib","Bdd.php")) {
    // arf ... I download it then !
    if(!download_file("lib/Bdd.php",".")) {
      $msg = "<b>ERREUR</b> : Je ne peux pas trouver le fichier Bdd.php n&eacute;cessaire
      &agrave; mon execution. Merci de placer un lien symbolique dans le r&eacute;pertoire
      d'installation vers ce fichier, ou bien de le copier directement.";
      error_log($msg, 0);

      // cannot continue :'(
      exit;
    } else
      include_once("Bdd.php");
  }
}


/* ============= LES FONCTIONS DE TESTS DE LA BASE DE DONNEES ============ */


/**--------------------------------------------------
 * Fonction qui permet d'afficher un formulaire pour
 * renseigner les donnees de connexion a la base de
 * donnees
 *
 * @param $install objet contenant les informations 
 * de configuration de l'install
 *
 * @result none
 *---------------------------------------------------*/
function DRAW_DB_FORMULAR_CONNECTION(&$install)
{
  $lg = $install->langue;
  $numpg = $install->getCurrentPage()+1;

  /* on cree le template contenant les champs de
     renseignement des donnees de connexion */
  $template  = "\t<h2>".$install->getSentence(25)."</h2>";
  $template .= "\t<form name='db_connexion' method='post' action='";
  $template .= $install->giveURL($numpg)."'>\n";
  $template .= "\t\t<span class='form_label'>".$install->getSentence(26);
  $template .= "</span><input name='DB_HOST' type='text'><br />\n";
  $template .= "\t\t<span class='form_label'>".$install->getSentence(27);
  $template .= "</span><input name='DB_USER' type='text'><br />\n";
  $template .= "\t\t<span class='form_label'>".$install->getSentence(29);
  $template .= "</span><input name='DB_PASSWD' type='password'><br />\n";
  $template .= "\t\t<span class='form_label'>".$install->getSentence(30);
  $template .= "</span><input name='DB_NAME' type='text'><br />\n";
  $template .= "\t\t<input type='submit' class='button_submit' value='";
  $template .= $install->getSentence(18)."'>\n";
  $template .= "\t</form>\n";

  /* on l'affiche */
  $install->addInformations($template);
}



/**--------------------------------------------------
 * Fonction qui teste la connexion a la base de donnees
 * a partir des renseignements fournis
 *
 * @param $install l'objet contenant toutes les donnees de
 * configuration de l'installation
 *
 * @result retourne un booleen indiquant l'echec ou
 * la reussite du test
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> fichier MDB non present !! DEPRECATED
 *   2 -> serveur inaccessible
 *   3 -> donnees de connexion incorrectes
 *---------------------------------------------------*/
function CHECK_CONNECT_DB(&$install)
{
  /* on tente une connexion */
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      // connexion a la base
      $db_link = new Bdd($urlcon);
      
      // on teste le resultat de la connexion
      if(! $db_link->reconnectDB(DEFAULT_REACHABLE_DB))
	{
	  /* erreur de connexion */
	  $res = false;
	  $msg_tmp = $install->getSentence(4);
	  $install->pageencours->updateLog("CHECK_CONNECT_DB",2,$msg_tmp);
	}
      else
	{
	  $res = true;
	  $msg_tmp = $install->getSentence(32);
	  $install->pageencours->updateLog("CHECK_CONNECT_DB",0,$msg_tmp);
	}
    }
  else
    /* sinon on considere la base inaccessible */
    {
      $res = false;
      $msg_tmp = $install->getSentence(31);
      $install->pageencours->updateLog("CHECK_CONNECT_DB",1,$msg_tmp);
    }

  // on renvoie le test
  return $res;
}




/**--------------------------------------------------
 * Fonction qui teste la presence d'une base de 
 * donnees sur le serveur qui correspond
 *
 * PRECONDITION : le test de la connexion a la base
 * a fonctionne et la fonction a sauvegarde les donnees
 * de connexion
 *
 * @param $install l'objet contenant toutes les donnees
 * de configuration de l'installation
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> connection au serveur impossible
 *   2 -> connection a la base impossible
 *
 * @result booleen indiquant l'echec ou la reussite
 * du test
 *---------------------------------------------------*/
function CHECK_DATABASE(&$install)
{
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      // connexion a la base
      $db_link = new Bdd($urlcon);
      
      // on teste le resultat de la connexion
      if(! $db_link->reconnectDB($install->getParameter("DB_NAME")))
      {
	/* erreur de connexion */
	/* comme les donnees de connexion sont enregistrees on considere */
	/* que la connexion est ok, ie c'est la base qui est erronnee */
	$res = false;
	$msg_tmp = $install->getSentence(5);
	$install->pageencours->updateLog("CHECK_DATABASE",2,$msg_tmp);

      } else {
	
	/* connexion ok */
	$res = true;
	$msg_tmp = $install->getSentence(37);
	$install->pageencours->updateLog("CHECK_DATABASE",0,$msg_tmp);
      }
      
    } else {
      /* la connexion a la base n'a pas ete teste */
      $res = false;
      $msg_tmp = $install->getSentence(37);
      $install->pageencours->updateLog("CHECK_DATABASE",1,$msg_tmp);
    }
  
  // on renvoie le resultat
  return $res;
}


/**--------------------------------------------------
 * Fonction qui teste l'envoie de requetes SQL a une
 * base de donnees
 *
 * PRECONDITION : la base doit etre accessible
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> connection a la base impossible
 *   2 -> creation de table impossible
 *   3 -> insertion de donnees impossible
 *   4 -> suppression de table impossible
 *
 * @result booleen indiquant l'echec ou la reussite
 * du test
 *---------------------------------------------------*/
function CHECK_MODIFY_DATABASE(&$install)
{
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      /* connexion a la base */
      $db_link = new Bdd($urlcon);
      
      // on teste le resultat de la connexion
      if(! $db_link->reconnectDB($install->getParameter("DB_NAME")))
	{
	  /* la connexion a echoue */
	  $res = false;
	  $msg_tmp = $install->getSentence(6);
	  $install->pageencours->updateLog("CHECK_MODIFY_DATABASE",1,$msg_tmp);
	  
	} else {
	  /* on envoie les 3 types de requetes de base */
	  
	  // creation de table
	  $cols = array (
			 "col1" => "varchar(5) not null primary key",
			 "col2" => "varchar(2) not null");

	  if(! $db_link->TableCreate("bachibouzouk", $cols))
	    {
	      /* envoie de la requete impossible */
	      $res = false;
	      $msg_tmp = $install->getSentence(7);
	      $install->pageencours->updateLog("CHECK_MODIFY_DATABASE",2,$msg_tmp);

	    } else {
	      // insertion de donnees
	      $cols = array(
			    "col1" => 'colon',
			    "col2" => 'ni');

	      if(! $db_link->Insert('bachibouzouk', $cols))
		{
		  $res = false;
		  $msg_tmp = $install->getSentence(8);
		  $install->pageencours->updateLog("CHECK_MODIFY_DATABASE",
						   3,$msg_tmp);
		  
		} else {
		  // suppression de table
		  if(! $db_link->TableDrop('bachibouzouk'))
		    {
		      $res = false;
		      $msg_tmp = $install->getSentence(9);
		      $install->pageencours->updateLog("CHECK_MODIFY_DATABASE",
						       4,$msg_tmp);

		    } else {
		      $res = true;
		      $msg_tmp = $install->getSentence(33);
		      $install->pageencours->updateLog("CHECK_MODIFY_DATABASE",
						       0,$msg_tmp);
		    }
		}
	    }
	}

    } else {
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("CHECK_CONNECT_DB",1,$msg_tmp);
    }
  
  // on renvoie le resultat
  return $res;
}


/**
 * @func CREATE_TABLES_FROM_XML
 *
 * @brief fonction permettant de creer toutes les tables
 * specifiees dans le fichier XML
 *
 * @param $install l'objet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function CREATE_TABLES_FROM_XML(&$install)
{
  $err = ""; /* les erreurs !! */
  
  /* on recupere le nombre de relations a creer */
  $nb = $install->db->getNbRelation();

  /* on lance chacune des requetes pour creer toutes les relations */
  $nb_err = 0;
  $err    = "";
  for($i=0 ; $i<$nb ; $i++) {
    /* on recupere la requete SQL correspondante */
    if(! $install->db->tables[$i]->sendByBdd()) {
      $nb_err++;
      $err .= $install->db->tables[$i]->toSQL() . "<br />";
    }
  }
  
  /* on fait un bilan des erreurs */
  if($nb_err > 0) { 
    $install->pageencours->updateLog("CREATE_TABLE_FROM_XML",0,
				     $install->getSentence(38));
  } else { 
    $install->pageencours->updateLog("CREATE_TABLE_FROM_XML",$nb_err, $err);
  }
}



/**--------------------------------------------------
 * Fonction qui permet de balancer une requete de
 * creation de table au serveur de base de donnees
 *
 * @param $name        le nom de la table a creer
 * @param $colonnes    tableau des colonnes
 * @param $constraints tableau des contraintes
 * @param $install     l'objet install
 *
 * $error_nbr
 *      0 -> tout est ok
 *      1 -> manque d'info pour la connexion
 *      2 -> erreur dans la requete
 *
 * @result renvoie vrai si la requete a fonctionne
 *-------------------------------------------------*/
function DB_CREATE_TABLE($name, $colonnes, $constraints, &$install) {

  $res = false;

  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      /* connexion a la base */
      $db_link = new Bdd($urlcon);

      /* creation de la table et ajout des contraintes */
      if($db_link->TableCreate($name, $colonnes)) {
	if(count($constraints) > 0) {
	  /* detection des erreurs */
	  $flg = false;
	  $err = "";

	  /* insertion des contraintes */
	  foreach($constraints as $const) {
	    if(! $db_link->AddConstraint($name, $const)) {
	      $flg = false;
	      $err = "$const on $name<br />";
	    }
	  }
	  
	  if(! $flg) {
	    /* tout semble OK */
	    $res = true;
	    $msg_tmp = $install->getSentence(36);
	    $install->pageencours->updateLog("DB_CREATE_TABLE",0,$msg_tmp);
	  } else {
	    $res = false;
	    $msg_tmp = $install->getSentence(13) . "<br />".$err;
	    $install->pageencours->updateLog("DB_CREATE_TABLE",2,$msg_tmp);
	  }
	} // end constraints

      } else {
	$res = false;
	$msg_tmp = $install->getSentence(13);
	$install->pageencours->updateLog("DB_CREATE_TABLE",2,$msg_tmp);
      }
      
    } else {
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("DB_CREATE_TABLE",1,$msg_tmp);
    }
  
  // on renvoie le resultat
  return $res;
}

/**
 * Fonction qui vide completement la base de donnees des tables
 * afin de mettre en place les nouvelles tables
 *
 * @param $install l'objet contenant toutes les donnees liees
 * a l'installation
 *
 * @result renvoie vrai si la suppression a fonctionnee
 **/
function DB_DROP_ALL_TABLES(&$install) 
{
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
   
      /* creation du lien */
      $db_link = new Bdd($urlcon);

      /* on recupere le nombre de relations a supprimer */
      $nb = $install->db->getNbRelation();
      
      /* on lance chacune des requetes pour creer toutes les relations */
      $nb_err = 0;
      $err    = "";
      for($i=0 ; $i<$nb ; $i++) {
	if($db_link->TableDrop($install->db->tables[$i]->name, true)) {
	  $nb_err++;
	  $err .= $install->db->tables[$i]->name . " non supprim&eacute;e<br />";
	}
      }

      /* on genere les erreurs en fonction */
      if($nb_err == 0) {
	$res = true;
	$msg_tmp = $install->getSentence(36);
	$install->pageencours->updateLog("DB_DROP_ALL_TABLES",0,$msg_tmp);
      } else {
	$res = false;
	$msg_tmp = $install->getSentence(13) . "<br />".$err;
	$install->pageencours->updateLog("DB_DROP_ALL_TABLES",2,$msg_tmp);
      }
    } else {
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("DB_DROP_ALL_TABLES",1,$msg_tmp);
    }

  return $res;
}


/**--------------------------------------------------
 * Fonction qui permet de balancer une requete SQL 
 * au serveur de base de donnees
 *
 * @param $sql la requete SQL a envoyer au serveur
 * @param $install l'objet contenant toutes les donnees
 * de configuration de l'install
 *
 * $error_nbr
 *      0 -> tout est ok
 *      1 -> manque d'info pour la connexion
 *      2 -> connexion impossible
 *
 * @result renvoie vrai si la requete a fonctionne
 *-------------------------------------------------*/
function SEND_SQL_REQUEST($sql,&$install)
{
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      // connexion a la base
      $db_link = new Bdd($urlcon);
      
      // on teste le resultat de la connexion
      if(! $db_link->reconnectDB($install->getParameter("DB_NAME")))
	{
	  $res = false;
	  $msg_tmp = $install->getSentence(4);
	  $install->pageencours->updateLog("SEND_SQL_REQUEST",2,$msg_tmp);
	}
      else
	{
	  // on balance la requete
	  if(($db_link->Requette($sql)) == NULL)
	    {
	      /* apparemment ya boulette dans la requete */
	      $res = false;
	      $msg_tmp = $install->getSentence(13);
	      $install->pageencours->updateLog("SEND_SQL_REQUEST",3,$msg_tmp);
	    }
	  else
	    {
	      $res = true;
	      $msg_tmp = $install->getSentence(36);
	      $install->pageencours->updateLog("SEND_SQL_REQUEST",0,$msg_tmp);
	    }

	  /* on ferme la connexion */
	  //$db_link->disconnect();
	}
    }
  else
    {
      /* le profil de connexion n'est pas enregistre */
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("SEND_SQL_REQUEST",1,$msg_tmp);
    }

  return $res;
}



?>
