<?php

/**
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @file setup/_ext_dependencies.php
 *
 * @brief Ce script regroupe les primitives utilisees dans les fichiers de 
 * configuration pour tester les differentes dependances qui peuvent etre
 * rencontrees pour installer Gasell.
 *
 * Le choix est fait de nommer les primitives en lettres majuscules pour eviter
 * des possibles erreurs d'interpretation dans le fichier de config XML liees 
 * au choix de codage du charset
 *
 * @date 13-10-2005
 * @author GrdScarabe <gasell-dev@grdscarabe.net>
 **/





/*****************************************************************************/
/******************** FONCTIONS DE TEST DES DEPENDANCES **********************/
/*****************************************************************************/




/* ============== LES FONCTIONS DE TESTS DES MODULES PEARS  ============== */

/**--------------------------------------------------
 * Fonction qui teste la presence du module pear MDB 
 *
 * @param $install l'objet contenant toute la 
 * configuration de l'installation
 *
 * @result retourne un booleen indiquant l'echec ou
 * la reussite du test (true pour reussite)
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> fichier present mais corrompu
 *   2 -> fichier non present
 *---------------------------------------------------*/
function CHECK_PEAR_MDB(&$install)
{
  /* on tente de charger le fichier MDB */  
  if(@include_once("MDB.php"))
    {
      /* on verifie la valeur d'une variable definie 
         dans le fichier de MDB                     */
      if(MDB_OK == 1) 
	{
	  // tout est OK
	  $res = true;
	  $tmp_msg = $install->getSentence(0)."PEAR MDB".$install->getSentence(1);
	  $install->pageencours->updateLog("CHECK_PEAR_MDB",0,$tmp_msg);
	}
      else
	{
	  // le fichier est accessible mais ne semble pas correspondre
	  $res = false;
	  $tmp_msg = $install->getSentence(0)."PEAR MDB".$install->getSentence(2);
	  $install->pageencours->updateLog("CHECK_PEAR_MDB",1,$tmp_msg);
	}
    }
  else
    {
      /* si le fichier ne s'est pas charge, erreur */
      $res = false;
      $tmp_msg = $install->getSentence(0)."PEAR MDB".$install->getSentence(3);
      $install->pageencours->updateLog("CHECK_PEAR_MDB",2,$tmp_msg);
    }

  /* on renvoie le resultat du test */
  return($res);
}





/**--------------------------------------------------
 * Fonction qui teste la presence du module pear 
 * XML/Tree
 *
 * @param $install l'objet contenant toute la 
 * configuration de l'installation
 *
 * @result retourne un booleen indiquant l'echec ou
 * la reussite du test (true pour reussite)
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   2 -> fichier non present
 *---------------------------------------------------*/
function CHECK_PEAR_XMLTREE(&$install)
{

  /* on tente de charger le fichier XML/Tree */  
  if(@include_once("XML/Tree.php"))
    {
      /* TODO : faire un test sur le contenu du fichier */
      $res = true;
      $msg_tmp = $install->getSentence(0)."PEAR XML/Tree".$install->getSentence(1);
      $install->pageencours->updateLog("CHECK_PEAR_XMLTREE",0,$msg_tmp);
    }
  else
    {
      /* si le fichier ne s'est pas charge, erreur */
      $res = false;
      $msg_tmp = $install->getSentence(0)."PEAR XML/Tree".$install->getSentence(3);
      $install->pageencours->updateLog("CHECK_PEAR_XMLTREE",2,$msg_tmp);
    }

  /* on renvoie le resultat du test */
  return($res);
}




/**--------------------------------------------------
 * Fonction qui teste la presence du module pear Date
 * 
 * @param $install l'objet contenant toute la 
 * configuration de l'installation
 *
 * @result retourne un booleen indiquant l'echec ou
 * la reussite du test (true pour reussite)
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> fichier present mais corrompu
 *   2 -> fichier non present
 *---------------------------------------------------*/
function CHECK_PEAR_DATE(&$install)
{
  /* on tente de charger le fichier Date */  
  if(@include_once("Date.php"))
    {
      /* on verifie la valeur d'une variable definie 
         dans le fichier de Date                     */
      if(DATE_FORMAT_ISO == 1)
	{
	  // tout est ok
	  $res = true;
	  $msg_tmp = $install->getSentence(0)."PEAR DATE".$install->getSentence(1);
	  $install->pageencours->updateLog("CHECK_PEAR_DATE",0,$msg_tmp);
	}
      else
	{
	  /* fichier corrompu ? */
	  $res = false;
	  $msg_tmp = $install->getSentence(0)."PEAR DATE".$install->getSentence(2);
	  $install->pageencours->updateLog("CHECK_PEAR_DATE",1,$msg_tmp);
	}
    }
  else
    {
      /* si le fichier ne s'est pas charge, erreur */
      $res = false;
      $msg_tmp = $install->getSentence(0)."PEAR DATE".$install->getSentence(3);
      $install->pageencours->updateLog("CHECK_PEAR_DATE",2,$msg_tmp);
    }

  /* on renvoie le resultat du test */
  return($res);
}





/**--------------------------------------------------
 * Fonction qui teste la presence du module pear Log
 * 
 * @param $install l'objet contenant toute la 
 * configuration de l'installation
 *
 * @result retourne un booleen indiquant l'echec ou
 * la reussite du test (true pour reussite)
 *
 * $error_nbr :
 *   0 -> tout est OK
 *   1 -> le fichier est present mais corrompu
 *   2 -> fichier non present
 *---------------------------------------------------*/
function CHECK_PEAR_LOG(&$install)
{
/* on tente de charger le fichier Log */  
  if(@include_once("Log.php"))
    {
      /* on verifie la valeur d'une variable definie 
         dans le fichier de Date                     */
      if(PEAR_LOG_EMERG == 0)
	{
	  // tout est ok
	  $res = true;
	  $msg_tmp = $install->getSentence(0)."PEAR LOG".$install->getSentence(1);
	  $install->pageencours->updateLog("CHECk_PEAR_LOG",0,$msg_tmp);
	}
      else
	{
	  /* fichier corrompu ? */
	  $res = false;
	  $msg_tmp = $install->getSentence(0)."PEAR LOG".$install->getSentence(2);
	  $install->pageencours->updateLog("CHECK_PEAR_LOG",1,$msg_tmp);
	}
    }
  else
    {
      /* si le fichier ne s'est pas charge, erreur */
      $res = false;
      $msg_tmp = $install->getSentence(0)."PEAR LOG".$install->getSentence(3);
      $install->pageencours->updateLog("CHECK_PEAR_LOG",2,$msg_tmp);
    }

  /* on renvoie le resultat du test */
  return($res);
}

?>
