<?php

/**
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @file setup/_ext_gasell.php
 *
 * @brief Ce script regroupe les primitives utilisees dans les fichiers de 
 * configuration pour acceder aux fonctionnalites du core de Gasell
 *
 * Le choix est fait de nommer les primitives en lettres majuscules pour eviter
 * des possibles erreurs d'interpretation dans le fichier de config XML liees 
 * au choix de codage du charset
 *
 * @date 13-10-2005
 * @author GrdScarabe <gasell-dev@grdscarabe.net>
 **/



/* ======== LES FONCTIONS RELATIVES AU CORE DE GASELL ======== */



/**
 * @func DRAW_ADD_USER
 *
 * @brief fonction permettant d'afficher le formulaire permettant
 * d'ajouter un utilisateur et une association a l'application
 *
 * @param $install l'onjet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function DRAW_ADD_USER_ASSO(&$install)
{
  $lg = $install->langue;
  $numpg = $install->getCurrentPage()+1;

  /* on cree le template contenant les champs de
     renseignement des donnees de connexion */
  $template  = "\t<h2>".$install->getSentence(43)."</h2>";
  $template .= "\t<form name='add_user' method='post' action='";
  $template .= $install->giveURL($numpg)."'>\n";
  $template .= "<span class='form_label'>Login : </span>";
  $template .= "<input name='USER_LOGIN' type='text'><br />\n";
  $template .= "<span class='form_label'>Secret : </span>";
  $template .= "<input name='USER_PASSWORD' type='password'><br />\n";
  $template .= "\t<h2>".$install->getSentence(46)."</h2>";
  $template .= "<span class='form_label'>Son association : </span>";
  $template .= "<input name='ASSO' type='text'><br />\n";
  $template .= "\t\t<input type='submit' class='button_submit' value='";
  $template .= $install->getSentence(18)."'>\n";
  $template .= "\t</form>\n";

  /* on l'affiche */
  $install->addInformations($template);
}

/**
 * @func ADD_USER_ASSO
 *
 * @brief fonction permettant d'ajouter un
 * utilisateur et une asso dans la base
 *
 * @param $install l'onjet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function ADD_USER_ASSO(&$install)
{
  // On verifie les parametres de connexion
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      // connexion a la base
      $db_link = new Bdd($urlcon);
      
      // on prepare les colonnes de la table gasell_users
      $cols1 = array(
		     "user_uid"  => $install->getParameter("USER_LOGIN"),
		     "user_pass" => md5($install->getParameter("USER_PASSWORD")));

      // on ajoute la personne
      $cols3 = array(
		     "memnom" => $install->getParameter("USER_LOGIN"));
      
      // on prepare les colonnes de la table comptes_gasell
      $cols2 = array(
		     "user_uid" => $install->getParameter("USER_LOGIN"),
		     "memnum"   => 1,
		     "level"    => 4,
		     "db"       => $install->getParameter("DB_NAME"),
		     "asso"     => $install->getParameter("ASSO"));


      // on balance la requete d'ajout
      if($db_link->Insert("gasell_users", $cols1) &&
	 $db_link->Insert("personne", $cols3) &&
	 $db_link->Insert("comptes_gasell",$cols2)) {
	$res = true;
	$msg_tmp = $install->getSentence(44);
	$install->pageencours->updateLog("ADD_USER",0,$msg_tmp);
      } else {
	/* il y a eu une erreur */
	$res = false;
	$msg_tmp = $install->getSentence(45);
	$install->pageencours->updateLog("ADD_USER",1,$msg_tmp);
      }
    }
  else
    {
      /* le profil de connexion n'est pas enregistre */
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("ADD_USER",1,$msg_tmp);
    }

  return $res;
}


/**
 * @func INSERT_MINI_CONFIG
 *
 * @brief permet d'inserer les donnees de
 * configuration minimale dans la base
 *
 * @param $install l'onjet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function INSERT_MINI_CONFIG(&$install)
{
  if( ($install->getParameter("DB_TYPE") != false) &&
      ($install->getParameter("DB_HOST") != false) &&
      ($install->getParameter("DB_USER") != false) &&
      ($install->getParameter("DB_PASSWD") != false) &&
      ($install->getParameter("DB_NAME") != false) )
    {
      /* URL de connexion a la base, de la forme :
            type://login:pass@server/database        */
      $urlcon = $install->getParameter("DB_TYPE") . "://" .
	$install->getParameter("DB_USER")  . ":" . 
	$install->getParameter("DB_PASSWD") . "@" .
	$install->getParameter("DB_HOST") . "/" .
	$install->getParameter("DB_NAME");
      
      // connexion a la base
      $db_link = new Bdd($urlcon);

      // preparation des variables
      $nb_err  = 0;

      // on met en place un exercice
      $cols = array(
		    "exenum"    => 0);
      if(! $db_link->Insert("exercice", $cols)) {
	$nb_err++;
      }

      // on insere les donnees dans config
      $cols = array(
		    "exenum"     => 0,
		    "cfgsection" => "TARIF",
		    "cfgparam"   => "Params",
		    "cfgvaleur"  => "");
      if(! $db_link->Insert("config", $cols)) {
	$nb_err++;
      }
  

      // encore un peu de config
      $cols = array(
		    "exenum"     => 0,
		    "cfgsection" => "PERSONNES",
		    "cfgparam"   => "CheckHomonymes",
		    "cfgvaleur"  => "OUI");
      if(! $db_link->Insert("config", $cols)) {
	$nb_err++;
      }
      
      // et encore de la config
      $cols = array(
		    "cfgsection" => "ASSO",
		    "cfgparam"   => "Exercice",
		    "cfgvaleur"  => "0");
      if(! $db_link->Insert("config", $cols)) {
	$nb_err++;
      }

      // et encore un peu
      $cols = array(
		    "cfgsection" => "ASSO",
		    "cfgparam"   => "ExerciceStatus",
		    "cfgvaleur"  => "Ferme");
      if(! $db_link->Insert("config", $cols)) {
	$nb_err++;
      }

      // insertion des contacts
      $cols = array(
		    "moyconcode" => 1,
		    "moyconlib"  => "Telephone");
      if(! $db_link->Insert("moyencontact", $cols)){
	$nb_err++;
      }

      $cols = array(
		    "moyconcode" => 2,
		    "moyconlib"  => "Fax");
      if(! $db_link->Insert("moyencontact", $cols)) {
	$nb_err++;
      }

      $cols = array(
		    "moyconcode" => 3,
		    "moyconlib"  => "Mail");
      if(! $db_link->Insert("moyencontact", $cols)) {
	$nb_err++;
      }

      $cols = array(
		    "moyconcode" => 4,
		    "moyconlib"  => "GSM");
      if(! $db_link->Insert("moyencontact", $cols)) {
	$nb_err++;
      }

      // mise en place des civilites par defaut
      $cols =  array(
		     "civlib"  => "Monsieur",
		     "civabr"  => "M.");
      if(! $db_link->Insert("civilite", $cols)) {
	$nb_err++;
      }

      $cols = array(
		    "civlib"  => "Madame",
		    "civabr"  => "Mme.");
      if(! $db_link->Insert("civilite", $cols)){
	$nb_err++;
      }

      $cols = array(
		    "civlib"  => "Mademoiselle",
		    "civabr"  => "Mlle.");
      if(! $db_link->Insert("civilite", $cols)) {
	$nb_err++;
      }
      
      /* verification que tout est ok */
      if ($nb_err == 0) {
	$res = true;
	$msg_tmp = $install->getSentence(47);
	$install->pageencours->updateLog("INSERT_MINI_CONFIG",0,$msg_tmp);
      } else {
	/* il y a eu une erreur */
	$res = false;
	$msg_tmp = $install->getSentence(48);
	$install->pageencours->updateLog("INSERT_MINI_CONFIG",1,$msg_tmp);
      }
    }
  else
    {
      /* le profil de connexion n'est pas enregistre */
      $res = false;
      $msg_tmp = $install->getSentence(6);
      $install->pageencours->updateLog("INSERT_MINI_CONFIG",2,$msg_tmp);
    }

  /* renvoie l'etat de l'execution */
  return $res;
}


/**
 * @func DRAW_ROOT_GASELL_FORM
 *
 * @brief fonction permettant d'afficher le formulaire permettant
 * de recuperer le repertoire d'installation de Gasell
 *
 * @param $install l'onjet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function DRAW_ROOT_GASELL_FORM(&$install)
{
	$lg = $install->langue;
	$numpg = $install->getCurrentPage()+1;

	/* try to guess the root directory */
	if( preg_match('@(.+)/[^/]*@', realpath(__FILE__), $m) )
		if(count($m) == 2)
			$dirguess = realpath($m[1].'/../');
		else
			$dirguess = '/';
	else
		$dirguess = '/var/www/';

  /* on cree le template contenant les champs de
     renseignement des donnees de connexion */
  $template  = "\t<h2>".$install->getSentence(40)."</h2>";
  $template .= "\t<form name='db_root_directory' method='post' action='";
  $template .= $install->giveURL($numpg)."'>\n";
  $template .= "<input name=\"INSTALL_ROOT_DIRECTORY\" value=\"$dirguess\" size=\"50\">";
  $template .= "<br/>\n\t\t<input type='submit' class='button_submit' value='";
  $template .= $install->getSentence(18)."'>\n";
  $template .= "\t</form>\n";

  /* on l'affiche */
  $install->addInformations($template);
}



/**
 * @func CREATE_CONFIG_FILE
 *
 * @brief fonction permettant de creer le fichier de configuration de Gasell
 * a partir des donnees recuperees durant l'installation
 *
 * @param $install l'onjet contenant toute la configuration
 * chargee a partir du fichier XML
 **/
function CREATE_CONFIG_FILE(&$install)
{
  /* on prepare les donnees a inscrire dans le fichier de configuration */
  $config  = "?php\n\$conf['bdd']['type'] = 'pgsql';\n";
  $config .= "\$conf['bdd']['host'] = '".$install->getParameter("DB_HOST")."';\n";
  $config .= "\$conf['bdd']['user'] = '".$install->getParameter("DB_USER")."';\n";
  $config .= "\$conf['bdd']['pass'] = '".$install->getParameter("DB_PASSWD")."';\n";
  $config .= "\$conf['bdd']['db']   = '".$install->getParameter("DB_NAME")."';\n";
  $config .= "\$conf['users']['js'] = false;\n";
  $config .= "\$conf['debug_level'] = E_ALL;\n";
  $config .= "\$conf['max_exec_time'] = 0;\n";
  $config .= "\$conf['umask'] = 077;\n?>";

  /* on ouvre le fichier de configuration en ecriture */
  $fic_conf = $install->getParameter("INSTALL_ROOT_DIRECTORY")
    ."/config/conf.php";

  if(! $hdle = @fopen($fic_conf,'w'))
    {
      /* impossible d'ouvrir le fichier, explication */
      $msg_tmp  = $install->getSentence(42);
      $msg_tmp .= " ".$fic_conf ." : <br/>";
      $msg_tmp .= "<pre>&lt;".$config."</pre>";
      $install->pageencours->updateLog("CREATE_CONFIG_FILE",1,$msg_tmp);

      $res = false;
    }
  else
    {
      /* on y inscrit les donnees */
      fwrite($hdle, "<".$config);

      /* on ferme le fichier */
      fclose($hdle);

      /* tout semble ok */
      $msg_tmp = $install->getSentence(41);
      $install->pageencours->updateLog("CREATE_CONFIG_FILE",0,$msg_tmp);

      $res = true;
    }
  return($res);
}




?>
