<?php

/**
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @file setup/_ext_php.php
 *
 * @brief Ce script regroupe les primitives utilisees dans les fichiers de 
 * configuration pour tester les fonctionnalites de php
 *
 * Le choix est fait de nommer les primitives en lettres majuscules pour eviter
 * des possibles erreurs d'interpretation dans le fichier de config XML liees 
 * au choix de codage du charset
 *
 * @date 13-10-2005
 * @author GrdScarabe <gasell-dev@grdscarabe.net>
 **/



/* ======== LES FONCTIONS DE DETECTION DE LA CONFIGURATION DE PHP ======== */

/**--------------------------------------------------
 * Fonction qui teste la presence du support de la base
 * de donnees PostgreSQL dans PHP
 *
 * $error_nbr
 *       0 -> tout est ok
 *       1 -> impossible de charger le module
 *       2 -> utilisation impossible du module
 * @result booleen indiquant l'echec ou la reussite
 * du test (true pour reussite)
 *---------------------------------------------------*/
function CHECK_PGSQL_PHPSUPPORT(&$install)
{
  /* on test si l'extension est chargee */
  if(!extension_loaded('pgsql'))
    {
      /* si ce n'est pas le cas, on risque un chargement dynamique */
      if(!dl('pgsql'))
	/* impossible d'avoir acces au module */
	{ 
	  $res = false;
	  $msg_tmp = $install->getSentence(10);
	  $install->pageencours->updateLog("CHECK_PGSQL_PHPSUPPORT",1,$msg_tmp);
	}
      else
	/* chargement du module semble OK
	     verification tout de meme      */
	{
	  
	  if(extension_loaded('pgsql'))
	    { 
	      $res = true;
	      $msg_tmp = $install->getSentence(34);
	      $install->pageencours->updateLog("CHECK_PGSQL_PHPSUPPORT",0,$msg_tmp);
	    }
	  else
	    { 
	      $res = false;
	      $msg_tmp = $install->getSentence(11);
	      $install->pageencours->updateLog("CHECK_PGSQL_PHPSUPPORT",2,$msg_tmp);
	    }
	} // fin test chargement dynamique
    }
  else
    /* l'extension est chargee */
    { 
      $res = true;
      $msg_tmp = $install->getSentence(34);
      $install->pageencours->updateLog("CHECK_PGSQL_PHPSUPPORT",0,$msg_tmp);
    }

  /* on renvoie le resultat */
  return($res);
}



/**--------------------------------------------------
 * Fonction qui teste la presence du support du xml
 *
 * $error_nbr
 *     0 -> ok
 *     1 -> impossible de charger le module
 *     2 -> utilisation impossible
 *
 * @param $install l'objet de configuration
 * @param $log activer ou non le log
 *
 * @result booleen indiquant l'echec ou la reussite
 * du test (true pour reussite)
 *---------------------------------------------------*/
function CHECK_XML_PHPSUPPORT(&$install,$log)
{
  /* on test si l'extension est chargee */
  if(!extension_loaded('xml'))
    {
      /* si ce n'est pas le cas, on risque un chargement dynamique */
      if(!dl('xml'))
	/* impossible d'avoir acces au module */
	{ 
	  $res = false;
	  if($log)
	    {
	      $msg_tmp = $install->getSentence(12);
	      $install->pageencours->updateLog("CHECK_XML_PHPSUPPORT",1,$msg_tmp);
	    }
	}
      else
	{
	  /* chargement du module semble OK
	     verification tout de meme      */	  
	  if(extension_loaded('xml'))
	    { 
	      $res = true;
	      if($log)
		{
		  $msg_tmp = $install->getSentence(35);
		  $install->pageencours->updateLog("CHECK_XML_PHPSUPPORT",0,$msg_tmp);
		}
	    }
	  else
	    { 
	      $res = false;
	      if($log)
		{
		  $msg_tmp = $install->getSentence(11);
		  $install->pageencours->updateLog("CHECK_XML_PHPSUPPORT",2,$msg_tmp);
		}
	    }
	} // fin test chargement dynamique
    }
  else
    /* l'extension est chargee */
    { 
      $res = true;
      if($log)
	{
	  $msg_tmp = $install->getSentence(35);
	  $install->pageencours->updateLog("CHECK_XML_PHPSUPPORT",0,$msg_tmp);
	}
    }
  
  /* on renvoie le resultat */
  return($res);
}

?>