<?php

/**
 * Gasell: Copyright 2004-2005 Linux-Nantes Association <contact@gasell.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * @file setup/install.php
 *
 * @brief Ce script est le squelette permettant de gerer toutes les primitives qui
 * peuvent etre appelees lors de l'installation. Il contient un corps principal
 * qui appel les fonctions de parsing de l'installeur, afin d'analyser le 
 * fichier de configuration et realiser ainsi une installation totalement 
 * personnalisee.
 *
 * Le choix est fait de nommer les primitives en lettres majuscules pour eviter
 * des possibles erreurs d'interpretation dans le fichier de config XML liees 
 * au choix de codage du charset
 *
 * @date 03-02-2005
 * @author GrdScarabe <gasell-dev@grdscarabe.net>
 **/

/** Breve explication du fonctionnement de l'installation
 * 
 * L'installation s'effectue a partir du present fichier de script
 * et d'un fichier de configuration qui commande le fichier de script.
 * 
 * Lors de l'appel, le script test tout d'abord si l'installation s'effectue
 * en mode graphique ou en mode texte et en fonction selectionne le type
 * d'installation dans le fichier de configuration.
 *
 * Il charge ensuite les informations contenues dans la partie global et qui 
 * concernent les configurations supportees par le logiciel. Tout ceci est 
 * sauvegarde dans un tableau "dependances". Il charge egalement le titre
 * de l'installeur et l'URI des sources auxquelles il pourrait faire reference
 *
 * Ensuite il traite la partie qui lui correspond selon le type d'installation
 * et selectionne la page en cours dans le fichier de configuration...
 * 
 * Pour chaque page des informations sur la presentation son chargees et 
 * permettent de mettre en place le template de la page ! Ensuite les actions
 * sont traitees. 
 *
 */

/**
 * REMARQUES A PROPOS DES FORMULAIRES
 *
 * Toutes les donnees recuperees par des formulaires sont captees
 * automatiquement par le script et stockees dans l'attribut post_args.
 * Elles sont automatiquement retransmises de page en page.
 * Pour cette raison :
 *    -> TOUS LES FORMULAIRES DOIVENT UTILISER LA METHODE POST SOUS PEINE
 *      DE NE PAS ETRE PRIS EN COMPTE
 *    -> LES NOMS DES CHAMPS DOIVENT TOUS ETRE DIFERENTS SOUS PEINE
 *      D'ECRASER DES VALEURS DE VARIABLES PRECEDENTES
 **/
    
    /* Toute les variables ci-dessous definissent les differentes
       options qui apparaissent tout au long de l'installation    */

/* constante contenant l'adresse relative du fichier de config */
/* On regarde d'abord si le parametre est transmit dans l'URL */
if(isset($_GET['config_file']) && !empty($_GET['config_file']))
 {
	define("CONFIG_FILE", $_GET['config_file']);
 }
/* Ensuite on le cherche dans les parametres du script */
else if($_SERVER["argc"] > 0)
{
	/* On debute a l'index 1 car 0 contient le nom de la commande */
	for($i = 1; $i < $_SERVER["argc"]; $i++)
	 {
		/* On decompose chaque chaine "param1=value1&param2=value2" 
			dans un tableau */
		parse_str($_SERVER["argv"][$i], $parsedArray);
		/* On cherche l'index 'config_file' */
		if(isset($parsedArray['config_file']))
		 {
			define("CONFIG_FILE", $parsedArray['config_file']);
		 }
	 }
 }

if (!defined("CONFIG_FILE"))
 {
	define("CONFIG_FILE", "config.xml");
 }
/* definition du type de base de donnees par defaut */
define("DEFAULT_DB_TYPE","pgsql");
/* definition du mode d'installation par defaut */
define("DEFAULT_INSTALL_MODE","graphic");
/* definition de la langue par defaut */
define("DEFAULT_LANGUAGE","fr");
/* la base de donnee toujours accessible pour le SGBDR */
define("DEFAULT_REACHABLE_DB","template1");
/* l'adresse a laquelle il est possible de recuperer des dependances */
define("PATH_DOWNLOAD","http://www.gasell.org/download/:http://svn.gna.org/viewcvs/*checkout*/gasell/trunk/gasell/");
/* repertoires dans lesquels on peut trouver des fichiers de langues */
define("PATH_LANGUAGES",".");
/* les dependances qui sont necessaires au deroulement 
   de l'install, separees par un ':' */
define("DEP_MANDATORY","php");



/*****************************************************************************/
/***************** CLASSES UTILISEES DANS L'INSTALLATION *********************/
/*****************************************************************************/


/**================ CLASSE CLASSSOURCE =================
 * @class ClassSource
 *
 * @brief la classe source permet de stocker une source
 * de donnees et sa priorite
 **====================================================*/
class ClassSource
{
  var $type;      // le type de la source
  var $priority;  // la priorite de la source
  var $address;   // l'adresse de la source

  /**
   * @function ClassSource::setType
   *
   * @brief  methode qui enregistre le type de la source
   * @param $tp le type (net ou file)
   **/
  function setType($tp)
    {
      if(($tp == 'net') or ($tp == 'file'))
	{
	  $this->type = $tp;
	  $res = true;
	}
      else
	$res = false;
      
      return $res;	   
    }

  /**
   * @function ClassSource::setPriority
   *
   * @brief methode qui enregistre la priorite
   * @param $p la priorite de la source (nbre positif) 
   **/
  function setPriority($p)
    {
      if($p >= 0)
	{
	  $this->priority = $p;
	  $res = true;
	}
      else
	$res = false;
      
      return $res;	   
    }

  /**
   * @function ClassSource::setAddress
   *
   * @brief methode qui enregistre l'adresse 
   * @param $a l'adresse de la source (relative ou absolue)
   **/
  function setAddress($a)
    { $this->address = $a; }

}/*EOC ClassSource*/



/**============ CLASSE CLASSSOURCES ======================
 * @class ClassSources
 * 
 * @brief la classe sources permet de stocker les sources 
 * et de les tester par ordre de priorite.
 **=======================================================*/
class ClassSources
{
  // !!ATTENTION, le tableau des sources commence a 1!!
  var $arr;  // le tableau des sources, il est trie par priorites decroissantes
  var $nb;   // le nombre de sources enregistrees
  var $curr; // l'index de la source courante non testee

  /**
   * @function ClassSources::ClassSources
   *
   * @brief le constructeur
   **/
  function ClassSources()
    { $this->nb = 0; $this->curr = 0; }

  /**
   * @function ClassSources::next
   *
   * @brief fonction qui recherche la prochaine source interressante 
   * par ordre de priorite
   * @result renvoie TRUE si une autre source est dispo, FALSE sinon
   **/
  function next()
    { 
      if($this->curr == $this->nb)
	$res = false;
      else
	{
	  $this->curr++;
	  $res = true;
	}

      return $res;
    }

  /**
   * @function ClassSources::addSource
   *
   * @brief ajout d'une source
   * @param $src un objet derive de la classe ClassSource
   **/
  function addSource($src)
    {
      /* on isole la place de la source */
      
      $this->nb++;
      $this->arr[$this->nb] = $src;
      
      if($this->curr == 0)
	/* si c'est la premiere on amorce */
	$this->next();
      else
	/* sinon on remet a 0 */
	$this->reset();
    }
  

  /**
   * @function ClassSources::addSource
   *
   * @brief prend en parametre un fichier si le fichier 
   * se trouve dans la source courante vrai est renvoye,
   * sinon on renvoie faux
   * @param $tst la source a ajouter
   **/
  function testSource($tst)
    {
      // construction de la cible
      $add = $this->arr[$this->curr]->address.$tst;

      return (file_exists($add));
    }

  /**
   * @function ClassSources::reset
   *
   * @brief fonction qui reinitialise les tests 
   **/
  function reset()
    {
      $this->curr = 0;
      $this->next();
    }

  /**
   * @function ClassSources::getAddress
   *
   * @brief fonction qui renvoie l'adresse
   * a partir d'une adresse relative et de la
   * source courante
   * @param $add l'adresse relative
   **/
  function getAddress($add)
    { return $this->arr[$this->curr]->address . $add; }

}/*EOC ClassSources*/



/**=================== CLASSE CLASSWEBSERVERDEP ================
 * @class ClassWebServerDep
 *
 * @brief la classe WebServerDep permet de stocker les serveurs 
 * webs compatibles avec Gasell                           
 **=============================================================*/
class ClassWebServerDep
{
  /* tableau indexe par le nom du serveur, chaque version est enregistree
   * dans un indice
   *   $arr[<nom>][0] -> le nombre de versions enregistrees
   *   $arr[<nom>][X] -> la Xe version enregistree */
  var $arr;

  /**
   * @function ClassWebServerDep::addVersion
   *
   * @brief methode qui ajoute une version
   * @param $name le nom du serveur
   * @param $vers la version
   */
  function addVersion($name,$vers)
    {
      if (is_array($this->arr))
        {
      	  // on verifie si le nom est present
      	  if($this->arr[$name][0] > 0)
	    { 
	      // on recupere l'indice
	      $i = $this->arr[$name][0];
	      $this->arr[$name][++$i] = $vers;
	      // on met le nombre de versions a jour
	      $this->arr[$name][0] = $i;
	   }
	 }
      else
	{
	  $this->arr[$name][0] = 1;
	  $this->arr[$name][1] = $vers;
	}
    }

  /**
   * @function ClassWebServerDep::testVersion
   *
   * @brief methode qui permet de tester si le 
   * server en parametre est supporte
   * @param $name le nom du serveur
   * @param $vers sa version
   */
  function testVersion($name,$vers)
    {
      $res = false;
      // on recherche le nom
      if($this->arr[$name][0] > 0)
	{
	  // on recherche la version
	  foreach($this->arr[$name] as $v)
	    $res = $res or ($v == $vers);
	}

      return $res;
    }

}/*EOC ClassWebServerDep*/



/**====================== CLASSE CLASSDBDEP ================
 * @class ClassDBDep
 *
 * @brief la classe ClassDBDep permet de stocker les SGBD 
 * compatibles avec Gasell 
 **=========================================================*/
class ClassDBDep
{
  /* tableau indexe par le nom du serveur, chaque version est enregistree
   * dans un indice
   *   $arr[<nom>][0] -> le nombre de versions enregistrees
   *   $arr[<nom>][X] -> la Xe version enregistree */
  var $arr;

  /**
   * @function ClassDBDep::addVersion
   *
   * @brief methode qui ajoute une version
   * @param $name le nom du serveur
   * @param $vers la version
   */
  function addVersion($name,$vers)
    {
      // on verifie si le nom est present
      if($this->arr[$name][0] > 0)
	{ 
	  // on recupere l'indice
	  $i = $this->arr[$name][0];
	  $this->arr[$name][++$i] = $vers;
	  // on met le nombre de versions a jour
	  $this->arr[$name][0] = $i;
	}
      else
	{
	  $this->arr[$name][0] = 1;
	  $this->arr[$name][1] = $vers;
	}
    }

  /**
   * @function ClassDBDep::testVersion
   *
   * @brief methode qui permet de tester si le 
   * server en parametre est supporte
   * @param $name le nom du serveur
   * @param $vers sa version
   */
  function testVersion($name,$vers)
    {
      $res = false;
      // on recherche le nom
      if($this->arr[$name][0] > 0)
	{
	  // on recherche la version
	  foreach($this->arr[$name] as $v)
	    $res = $res or ($v == $vers);
	}

      return $res;
    }

}/*EOC ClassDBDep*/




/**==================== CLASSE CLASSPAGE =====================
 * @class ClassPage
 *
 * @brief la classe page permet de construire la page courante 
 **===========================================================*/
class ClassPage
{
  var $title;    // le titre de la page
  var $logo;     // le logo
  var $txt;      // le texte descriptif
  var $infos;    // les warnings et autres erreurs
  var $ajout;    // les ajouts

  var $errors = false;   // flag indiquant si il y a eu des erreurs

  var $page_num; // le numero de la page
  var $page_tot; // le nombre total de page

  // les types de boutons (booleens indiquant leur presence)
  var $bCancel;
  var $bNext;
  var $bPrevious;
  var $bFinish;

  /**
   * @func ClassPage::setPageTot
   *
   * @brief update the argument of
   * the number total of pages
   *
   * @param $tot the total number 
   * of pages
   **/
  function setPageTot($tot){
    $this->page_tot = $tot;
  }

  /**
   * @func ClassPage::setPageNumber
   *
   * @brief update the argument of
   * the page number
   *
   * @param $num the page number
   **/
  function setPageNumber($num){
    $this->page_num = $num;
  }

  /**
   * @func ClassPage::ajouteTexte
   *
   * @brief fonction permettant d'ajouter des rensignements
   * a la page
   *
   * @param $val les renseignements a rajouter
   **/
  function ajouteTexte($val)
    { $this->ajout .= $val; }


  /**
   * @function ClassPage::updateLog
   *
   * @brief fonction qui permet de mettre a jour
   * les logs sur l'execution des differentes
   * primitives
   *
   * @arg primname nom de la primitive dont les
   * logs sont a mettre a jour
   * @arg code le code de l'erreur si il y a, 0 sinon
   * @arg value la valeur du message de log
   **/
  function updateLog($primname, $code, $value)
    {
      // on met a jour les logs
      $GLOBALS["error_nbr"][$primname] = $code;
      $GLOBALS["error_msg"][$primname] = $value;
      // check si il y a eu une erreure
      if($code != 0)
	$this->errors = true;
    }


  /**
   * @function ClassPage::afficher
   *
   * @brief construit et affiche la page
   * a l'ecran en fonction des informations
   * renseignees
   *
   * @param flg permet de specifier si l'on veut afficher
   * les erreurs ou non
   **/
  function afficher($flg = true)
    {
      /* construction du template */
      $template = "<html>\n<head>\n\t<title>".$this->title."</title>\n";
      $template .= "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-15\" />\n";
      $template .= $GLOBALS['STYLE']."</head>\n<body>\n";
      $template .= "<div class='conteneur'>\n";

      /* on cherche le logo dans les sources */
      while(! $GLOBALS['INSTALL']->testerSource($this->logo)) 
	{ 
	  if(! $GLOBALS['INSTALL']->nextSource()) 
	    break; 
	}

      /* si on ne trouve pas le fichier, on n'en renvoie aucun */
      if($GLOBALS['INSTALL']->testerSource($this->logo))
	{
	  $template .= "\t<div class='logo'><img src='";
	  $template .= $GLOBALS['INSTALL']->getSource($this->logo)."'>";
	}
	  
      $template .= "</div>\n\t<div class='droite'>\n\t\t";
      $template .= "\t<H1>".$this->title."</H1>\n<div class='text'>\n";
      $template .= nl2br($this->txt)."\n\t\t</div>\n";

      /* si quelque chose est ajoute on le met */
      if(isset($this->ajout) && !(empty($this->ajout)))
	  $template .= $this->ajout;

      /* on affiche ici tous les messages d'erreur, si il y en a plus de 1
       qui est du a l'initialisation avec main */
      if(count($GLOBALS['error_msg']) > 1)
      {
	$template .= "\t\t<div class='errors'>\n";
	foreach($GLOBALS['error_msg'] as $k=>$v)
	  {
	    if($k != "main")
	      {
		if($GLOBALS['error_nbr'][$k] == 0)
		  {
		    $template .= "\t\t\t<div class='noerror'>";
		    $template .= "<img src='images/ok.png' alt='ok'>".$v."</div>\n";
		  }
		else
		  {
		    $template .= "\t\t\t<div class='error'>";
		    $template .= "<img src='images/ko.png' alt='ko'>".$v."</div>\n";
		  }
	      }
	  }
	$template .= "\t\t</div>\n";
      }

      $template .= "\t\t<div class='buttons'>\n";

      /* mise en place des boutons */
      if($this->bCancel) 
	{
	  $template .= "\t\t\t<span class='button'>\n\t\t\t\t";
	  $template .= "<a href='".$GLOBALS['INSTALL']->giveURL(-1)."'>";
	  $template .= $GLOBALS['INSTALL']->getSentence(16)."</a>\n\t\t\t</span>\n";
	}
      if($this->bPrevious) 
	{
	  $numpg = intval($GLOBALS['INSTALL']->getCurrentPage())-1;
	  $template .= "\t\t\t<span class='button'>\n";
	  $template .= "\t\t\t\t<a href='".$GLOBALS['INSTALL']->giveURL($numpg)."'>";
	  $template .= $GLOBALS['INSTALL']->getSentence(17)."</a>\n\t\t\t</span>\n";
	}
      if($this->bNext && !$this->errors) // si error, pas de suivant
	{
	  $numpg = intval($GLOBALS['INSTALL']->getCurrentPage())+1;
	  $template .= "\t\t\t<span class='button'>\n";
	  $template .= "\t\t\t\t<a href='".$GLOBALS['INSTALL']->giveURL($numpg)."'>";
	  $template .= $GLOBALS['INSTALL']->getSentence(18)."</a>\n\t\t\t</span>\n";
	}
      if($this->bFinish) 
	{
	  $template .= "\t\t\t<span class='button'>\n";
	  $template .= "\t\t\t\t<a href='".$GLOBALS['INSTALL']->giveURL(-2)."'>";
	  $template .= $GLOBALS['INSTALL']->getSentence(19)."</a>";
	  $template .= "\n\t\t\t</span>\n";
	}


      $template .= "\t\t</div>\n\t</div>\n</div>\n";

      /* on ajoute le compteur de page */
      $template .= "<div class='compteur'>";
      $template .= $GLOBALS['INSTALL']->getSentence(49)." ";
      $template .= $this->page_num."/".$this->page_tot;
      $template .= "</div>";

      $template .= "</body>\n</html>";
     
      /* affichage */
      echo $template;
    }
}/*EOC ClassPage*/


/**================ CLASSE CLASSTABLE ===================
 * @class ClassTable
 *
 * @brief permet de stocker des renseignements sur une
 * relation d'une base de donnees
 **======================================================*/
class ClassTable
{
  /* ------ le schema definissant la relation ------ */
  var $name;         /* le nom de la relation */
  var $columns;      /* les differentes colonnes de la relation */
  var $constraints;  /* les differentes contraintes de la relation */

  /* ------ les methodes ------ */
  
  /**
   * @func ClassTable::toSQL
   *
   * @brief convertit la definition de la table
   * en requete SQL
   **/
  function toSQL()
    {
      $sql = "CREATE TABLE ".$this->name." (";
      
      /* definitions des colonnes */
      for($i=0 ; $i<(count($this->columns)-1) ; $i++)
	$sql .= $this->columns[$i].", ";
      $sql .= $this->columns[count($this->columns)-1]." )";

      /* inscription des contraintes */
      for($i=0 ; $i<(count($this->constraints)-1) ; $i++)
	$sql .= $this->constraints[$i].", ";
      $sql .= $this->constraints[count($this->constraints)-1].";";

      return $sql;
    }

  /**
   * @func ClassTable::sendByBdd()
   *
   * @brief envoie la requete grace a l'utilisation
   * de la librairie Bdd qui permet une abstraction
   * de la classe
   **/
  function sendByBdd() {
    /* creation du tableau des colonnes */
    foreach($this->columns as $col) {
      /* first word is the name */
      preg_match("/([^ ]+) (.+)/", $col, $matches);
      $cols[$matches[1]] = $matches[2];
    }

    /* mise en place des contraintes sur la table */
    if(count($this->constraints) > 0) {
      foreach($this->constraints as $cons) {
	/* first word is the name */
	if( preg_match('/^([^ ]*)(.+)([^ ]*)$/', $cons, $matches) )
		$const[] = $matches[2];
	else
		$const[] = $cons;
      }
    } else {
      $const = array();
    }
    
    return DB_CREATE_TABLE($this->name, $cols, $const, $GLOBALS["INSTALL"]);
  }

  /**
   * @func ClassTable::addColumn
   *
   * @brief permet d'ajouter une colone a la table
   *
   * @param $col la colonne
   **/
  function addColumn($col)
    { 
      $this->columns[] = $col;
    }

  /**
   * @func ClassTable::addConstraint
   *
   * @brief permet d'ajouter une contrainte a
   * la definition de la relation
   *
   * @param $cst la constrainte
   **/
  function addConstraint($cst)
    {
      /* on ne commite que si la contrainte est valide */
      if(preg_match("/[a-zA-Z]/", $cst)) {
	$this->constraints[] = $cst;
      }
    }

} /* EOC : ClassTable */


/**================ CLASSE CLASSDATABASE ====================
 * @class ClassDatabase
 *
 * @brief classe permettant de contenir une base de donnees
 * entiere et de la creer sur le serveur
 **==========================================================*/
class ClassDatabase
{
  var $tables; /* les differentes tables de la base 
		  representees sous forme d'objets de ClassTable */
  var $nb=0;   /* nombre de tables presentes */
  
  /* ------ les methodes ------ */

  /**
   * @func ClassDatabase::addTable
   *
   * @brief permet d'ajouter une table
   * a la base sous forme d'objet de type
   * ClassTable
   *
   * @param $tbl la table a jouter
   **/
  function addTable($tbl)
    {
      $this->tables[$this->nb] = $tbl;
      $this->nb += 1;
    }

  /**
   * @func ClassDatabase::getNbRelation
   *
   * @brief permet d'obtenir le nombre
   * de relations presentes dans la base
   **/
  function getNbRelation()
    { return $this->nb; }

  /**
   * @func getSQLRequest
   *
   * @brief permet d'obtenir la requete SQL de
   * creation de la relation passee en parametre
   * 
   * @param $num le numero de la relation a creer,
   * il doit etre dans les bornes
   **/
  function getSQLRequest($num)
    {
      if( ($num < $this->getNbRelation()) && ($num >= 0) )
	return $this->tables[$num]->toSQL();
      else
	return NULL;
    }

}/*EOC : ClassDatabase*/



/**==================== CLASSE INSTALLCONFIG =====================
 * @class InstallConfig
 *
 * @brief permet de stocker toutes les donnees necessaires
 * a la realisation de l'install et evite d'utiliser des
 * variables globales...paske c'est moche les variables globales
 * et sinon on serait oblige de balancer 50 parametres aux
 * fonctions...donc voila !!
 * cela permet entre autres de regrouper toutes les fonctions
 * importantes et utiles sous une meme entite...simplifiant au
 * maximum l'ecriture d'extensions pour le script
 **===============================================================*/
class InstallConfig
{
  /* ------------- la configuration globale de l'install ------------ */
  var $sourcesdispos;  /* les sources disponibles */
  var $webservdep;     /* les dependances sur les serveurs web */
  var $dbdep;          /* les dependances sur le SGBDR */
  var $db;             /* la base de donnee representee par les relations */
  var $langues_dispos; /* tableau des indices de langues disponibles */
  var $totalpages;     /* nombre de page pour chaque mode */

  var $pageencours;    /* la page en cours de creation */
  var $installTitle;   /* le titre de l'installeur */
  var $page_courante;  /* la page courante de l'installation */
  var $mode;           /* le mode d'installation */
  var $langue;         /* la langue de l'installation */
  var $form_args;      /* les arguments des formulaires et d'un peu tout */


  /* ------------- le constructeur de la classe ------------ */
  /**
   * @func InstallConfig::InstallConfig
   *
   * @brief constructeur, permet de mettre en
   * place les valeurs trouvees ou bien celles par
   * defaut
   **/
  function InstallConfig()
    {
      /** initialisation des variables globales **/
      $GLOBALS['currentscriptType'] = false;
      $GLOBALS['currentpageok']     = false;

      /* Initialisation des instances de classe */
      $this->sourcesdispos = new ClassSources();
      $this->webservdep    = new ClassWebServerDep();
      $this->dbdep         = new ClassDBDep();
      $this->db            = new ClassDatabase();
      $this->pageencours   = new ClassPage();

      /* on determine la page en cours */
      if(! isset($_GET['page']))
	{ $this->page_courante = 1; }
      else if(! empty($_GET['page']))
	{ $this->page_courante = $_GET['page']; }
      else
	{ $this->page_courante = 1; }
      /* on met a jour la page courante */
      $this->pageencours->setPageNumber($this->page_courante);

      /* detection du type d'installation (graphic/text) */
      if(isset($_GET['type']) || isset($_SERVER['argv']))
	{
	  if(empty($_GET['type']) && empty($_SERVER['argv']))
	    { // on choisit le mode par defaut
	      $this->mode = DEFAULT_INSTALL_MODE; }
	  else
	    {
	    	if(!empty($_GET['type']))
	    		$this->mode = $_GET['type'];
		/* L'argument 1 contient le type d'install : php -f install.php text */
		if(!empty($_SERVER['argv'][1]))
			$this->mode = $_SERVER['argv'][1];
	    }
	}else
	  { $this->mode = DEFAULT_INSTALL_MODE; }

      /* on determine la langue a utiliser */
      if(isset($_GET['lang']))
	{
	  if(empty($_GET['lang']))
	    {
	      // on charge la langue par defaut
	      $this->langue = DEFAULT_LANGUAGE;
	    }else{
	      // on charge la langue en parametre
	      $this->langue = $_GET['lang'];
	      
	      // on verifie que cette langue existe
	      if(! isset($dico[$this->langue]))
		{ // on charge la langue par defaut
		  $this->langue = DEFAULT_LANGUAGE; }
	    }
	}else{
	  $this->langue = DEFAULT_LANGUAGE; 
	}

      /* par defaut on utilise PostgreSQL */
      $this->addFormArg("DB_TYPE",DEFAULT_DB_TYPE);
    }


  /* --- Fonctions permettant de fixer les valeurs des attributs --- */

  /**
   * @func InstallConfig::setInstallTitle
   *
   * @brief fonction permettant de fixer la valeur 
   * du titre de l'installer
   *
   * @param $title le titre a fixer
   **/
  function setInstallTitle($title)
    { $this->installTitle = $title; }


  /**
   * @func InstallConfig::setPageTitle
   *
   * @brief fonction permettant de fixer la valeur
   * du titre de la page en cours
   *
   * @param title le titre a afficher
   **/
  function setPageTitle($title)
    { $this->pageencours->title = $title; }


  /**
   * @func InstallConfig::setPageLogo
   *
   * @brief permet de fixer le logo qui va
   * aparaitre sur la page
   *
   * @param $logo le nom du fichier du logo
   * sans l'adresse
   **/
  function setPageLogo($logo)
    { $this->pageencours->logo = $logo; }

  /**
   * @func InstallConfig::setPageText
   *
   * @brief permet de fixer le texte qui va
   * aparaitre sur la page
   *
   * @param $txt le text a inclure
   **/
  function setPageText($txt)
    { $this->pageencours->txt .= $txt; }

  /**
   * @func InstallConfig::addInformations
   *
   * @brief permet d'ajouter des donnees
   * a afficher dans la page
   *
   * @param $val les donnees a ajouter
   **/
  function addInformations($val)
    { $this->pageencours->ajouteTexte($val); }

  /**
   * @func InstallConfig::putCancel
   *
   * @brief fonction permettant d'ajouter un bouton cancel
   * a la page en cours de creation
   **/
  function putCancel()
    { $this->pageencours->bCancel = true; }

  /**
   * @func InstallConfig::putPrevious
   *
   * @brief fonction permettant d'ajouter un bouton previous
   * a la page en cours de creation
   **/
  function putPrevious()
    { $this->pageencours->bPrevious = true; }

  /**
   * @func InstallConfig::putNext
   *
   * @brief fonction permettant d'ajouter un bouton
   * next a la page en cours
   **/
  function putNext()
    { $this->pageencours->bNext = true; }

  /**
   * @func InstallConfig::putFinish
   *
   * @brief fonction permettant d'ajouter un bouton
   * finish a la page en cours
   **/
  function putFinish()
    { $this->pageencours->bFinish = true; }

  /**
   * @func InstallConfig::addFormArg
   *
   * @brief permet d'ajouter une variable obtenue
   * par l'intermediaire d'un formulaire
   *
   * @param $name le nom de la variable
   * @param $arg la valeur
   **/
  function addFormArg($name, $arg){
    $narg = urldecode($arg);
    $narg = stripslashes($narg);
    switch($name) {
      /* modification de la langue */
    case "lang":
      if($this->isRegisteredLanguage($narg))
	$this->langue = $narg;
      break;
    case "page":
      $this->page_courante = $narg;
      break;
      /* ajout des donnees de formulaire */
    default:
      $this->form_args[$name] = $narg;
    }
  }

  /* ------------- Fonctions d'acces aux attributs ------------ */

  /**
   * @func InstallConfig::addPage
   *
   * @brief fonction permettant d'ajouter
   * une nouvelle page au mode en parametre
   *
   * @param mode le mode auquel ajouter la page
   **/
  function addPage($mode){
    if(isset($this->totalpages[$mode])) {
      if(! empty($this->totalpages[$mode])){
	$this->totalpages[$mode] = $this->totalpages[$mode] + 1;
	/* on met a jour la page en cours */
	$this->pageencours->setPageTot($this->getTotalPage($this->getMode()));
      } else {
	$this->totalpages[$mode] = 1;
	/* on met a jour la page en cours */
	$this->pageencours->setPageTot($this->getTotalPage($this->getMode()));
      } 
    } else {
      $this->totalpages[$mode] = 1;
      /* on met a jour la page en cours */
      $this->pageencours->setPageTot($this->getTotalPage($this->getMode()));
    }
  }

  /**
   * @func getTotalPage
   *
   * @brief permet d'obtenir le nombre total
   * de page pour le mode en parametre
   *
   * @param mode le mode concerne   **/
  function getTotalPage($mode){
    if(isset($this->totalpages[$mode]))
      if(! empty($this->totalpages[$mode]))
	return $this->totalpages[$mode];
      else
	return 0;
    else
      return 0;
  }

  /**
   * @func InstallConfig::addLanguage
   *
   * @brief fonction permettant d'ajouter
   * un langage
   *
   * @param $ind     l'indice la langue (fr, en, ...)
   * @param $libelle le nom de la langue
   **/
  function addLanguage($ind, $libelle)
   { $this->langues_dispos[$ind] = $libelle; }

  /**
   * @func InstallConfig::isRegisteredLanguage
   *
   * @brief fonction permettant de determiner
   * si un language existe ou non
   *
   * @param $ind indice de la la langue a tester
   **/
  function isRegisteredLanguage($ind)
    { return isset($this->langues_dispos[$ind]); }

  /**
   * @func InstallConfig::getListeLangues
   *
   * @brief retourne un tableau contenant les indices
   * des langues
   **/
  function getListeLangues()
    { return $this->langues_dispos; }


  /**
   * @func InstallConfig::getCurrentPage
   *
   * @brief retourne la page courante de
   * l'install ie la page a construire
   **/
  function getCurrentPage()
    { 
      if(isset($this->page_courante))
	return $this->page_courante;
      else
	return 1;
    }

  /**
   * @func InstallConfig::getMode
   *
   * @brief retourne le mode de
   * l'installation
   **/
  function getMode()
    {
      if(isset($this->mode))
	return $this->mode;
      else
	return "graphic";
    }


  /**
   * @func InstallConfig::isDebug
   *
   * @brief permet de definir si le mode
   * debug est active ou non
   **/
  function isDebug()
    {
      return $GET["debug"] == 1;
    }

  /**
   * @func InstallConfig::getParameter
   *
   * @brief permet d'obtenir un parametre
   * recupere par GET
   *
   * @param $name   le nom du parametre a recuperer
   * @param $escp determine si on a besoin d'echaper
   *   les quotes dans le parametre ou non
   **/
  function getParameter($name, $escp=true)
    { 
      if(isset($this->form_args[$name]))
	{
	  if($escp)
	    {
	      /* escape only unescaped quotes */
	      $patt = "/(?<!\\\)\'/";
	      $repl = "\\'";
	      $res = preg_replace($patt, $repl, $this->form_args[$name]);
	    } else {
	      $res = $this->form_args[$name];
	    }
	}
      else
	$res = false;

      /* retourne le resultat */
      return $res;
    }


  /**
   * @func InstallConfig::genDebug
   *
   * @brief permet de generer les informations de
   * debuggage de l'installer
   **/
  function genDebug()
    {
      /* on inclut toutes les donnees sur l'environnement serveur */
      $debug = "************** ENVIRONNEMENT SERVEUR **************<br/>";
      foreach($_SERVER as $k=>$v)
	$debug .= "$k : $v<br/>";
      
      /* inclusion des informations d'Apache */
      $arr_apache = apache_get_modules();
      $debug .= "************** CONFIGURATION APACHE **************<br/>";
      $debug .= "Version : ".apache_get_version()."<br/>";
      foreach($arr_apache as $k=>$v)
	$debug .= "$k : $v<br/>";
      
      /* on inclut les informations sur le systeme d'exploitation */
      /* TODO */

      /* on inclut les informations sur l'installer */
      $debug .= "************** ETAT INSTALLER **************<br/>";
      $debug .= "Requete : ".$_SERVER['REQUEST_URI']." - ".$_SERVER['QUERY_STRING'];
      $debug .= "<br/> URL : ".$PHP_SELF."<br/>";
      foreach($GLOBALS["INSTALL"]->form_args as $k=>$v)
	$debug .= "$k : $v<br/>";

      /* on indique la totalite du fichier de config de l'installer */
      $debug .= "************** FICHIER DE CONFIG **************<br/>";
      $tmp = file(CONFIG_FILE);
      foreach($tmp as $l)
	$debug .= $tmp."<br/>";

      return $debug;
    }

  /* ------------- Encapsulation des fonctions des autres classes ------------ */

  /**
   * @func InstallConfig::giveURL
   *
   * @brief permet de recuperer l'URL de base
   * de l'installation, comportant entre autres la langue
   * ainsi que les arguments a passer. Le numero de page
   * n'y est pas indique par defaut, il doit l'etre
   * en parametre
   *
   * @param $page le numero de la page voulue
   **/
  function giveURL($page)
    {
      /* mise en place de l'url de base */
      $url  = "install.php?page=".$page."&amp;lang=".$this->langue;
      $url .= "&amp;mode=".$this->mode;
      if(count($this->form_args) > 0)
	{
	  /* on ajoute les parametres un par un */
	  foreach($this->form_args as $k=>$v)
	    {
	      if( ($k != "lang") && ($k != "page") )
		{
		  $tmp = urlencode($this->getParameter($k, false));
		  $tmp = preg_replace('/\+/', "%20", $tmp); // bug urlencode
		  $url .= "&amp;$k=".$tmp;
		}
	    }
	}

      /* on renvoie le resultat */
      return $url;
    }

  /**
   * @func InstallConfig::getSentence
   *
   * @brief permet d'obtenir automatiquement
   * la phrase voulue traduite en fonction 
   * de la langue detectee
   **/
  function getSentence($num) { 
    return $GLOBALS['dico'][$this->langue][$num]; 
  }


  /**
   * @func InstallConfig::getRootDirectory
   *
   * @brief permet d'obtenir l'adresse du repertoire
   * d'installation de l'application
   **/
  function getRootDirectory()
    {
      return $this->getParameter('INSTALL_ROOT_DIRECTORY');
    }
  
  /**
   * @func InstallConfig::addWebServDep
   *
   * @brief fonction permettant d'ajouter une dependance sur
   * le serveur web
   *
   * @param webserv le nom du webserver
   * @param vers la version du webserver
   **/
  function addWebServDep($webserv,$vers)
    { return $this->webservdep->addVersion($webserv,$vers); }

  /**
   * @func InstallConfig::addDBServDep
   *
   * @brief fonction qui permet de rajouter une dependance
   * sur le serveur de base de donnees
   *
   * @param $dbserv le nom du serveur
   * @param $vers la version
   **/
  function addDBServDep($dbserv,$vers)
    { return $this->dbdep->addVersion($dbserv,$vers); }


  /**
   * @func InstallConfig::addSource
   *
   * @brief fonction qui permet d'ajouter
   * une nouvelle source de donnees
   *
   * @param $src la source
   **/
  function addSource($src)
    { return $this->sourcesdispos->addSource($src); }


  /**
   * @func InstallConfig::addTable
   *
   * @brief permet d'ajouter une relation a la
   * l'installation
   *
   * @param $rel la relation
   **/
  function addTable($rel)
    {
      /* on ajoute la requete de creation */
      $this->db->addTable($rel);
    }


  /**
   * @func InstallConfig::afficherPage
   *
   * @brief fonction permettant d'afficher
   * la page construite
   **/
  function afficherPage()
    { $this->pageencours->afficher(); }

  /**
   * @func InstallConfig::testerSource
   *
   * @brief permet de tester un fichier
   * pour une source de donnees
   *
   * @param $fic le fichier a tester
   **/
  function testerSource($fic)
    { return $this->sourcesdispos->testSource($fic); }

  /**
   * @func InstallConfig::nextSource
   *
   * @brief permet de tester la source
   * suivante dans la liste de priorite
   **/
  function nextSource()
    { return $this->sourcesdispos->next(); }

  /**
   * @func InstallConfig::getSource
   *
   * @brief permet d'obtenir l'adresse
   * d'un fichier en parametre a partir 
   * de la source courante
   *
   * @param $fic le fichier dont on veut
   * connaitre l'adresse
   **/
  function getSource($fic)
    { return $this->sourcesdispos->getAddress($fic); }

  /**
   * @func InstallConfig::resetSources
   *
   * @brief permet de recommencer le
   * parcours des sources disponibles
   * depuis le debut
   *
   * @noparam
   **/
  function resetSources() {
    $this->sourcesdispos->reset();
  }


  /**
   * @func getSQLRequest
   *
   * @brief renvoie la requete SQL de creation
   * de la relation
   *
   * @param $n le numero de la requete
   **/
  function getSQLRequest($n)
    {
      return $this->db->getSQLRequest($n);
    }

}/* EOC InstallConfig */



/*****************************************************************************/
/****************** VARIABLES GLOBALES DE L'INSTALLATION *********************/
/*****************************************************************************/

/* =========== LES VARIABLES D'ENVIRONNEMENT DE L'INSTALL ============ */

    /* La variable ci-dessous determine le style de l'installation */
global $STYLE;
$STYLE  = "\t<style type='text/css'>\n";
$STYLE .= "\t\t.conteneur{margin-right: 0;margin-left: 0; padding 0;}\n";
$STYLE .= "\t\tA{color:#ffffff}\n";
$STYLE .= "\t\tBODY{background-color:#0e5158;}\n";
$STYLE .= "\t\tH1{color:#ff9d1b;weight:700;text-align:center; padding-bottom:2ex;}\n";
$STYLE .= "\t\tH2{color:#ffad2b;text-decoration: underline; text-align:center;";
$STYLE .= "font-size:120%;}\n";
$STYLE .= "\t\tFORM INPUT{background-color:#0e6168; margin-left: 2ex;";
$STYLE .= "border-color:#ff9d1b; border-width:1px; margin-bottom: 1ex;";
$STYLE .= "position: absolute; right: 10%; text-align:center; color:#ffffff}\n";
$STYLE .= "\t\tFORM INPUT centre{background-color:#0e6168; border-color:#ff9d1b;";
$STYLE .= " border-width:1px; margin-bottom: 1ex; text-align:center; ";
$STYLE .= "color:#ffffff}\n";
$STYLE .= "\t\t.droite{margin-left: 31%; width:67%; background-color:#0e5158; opacity: 0.9}\n";
$STYLE .= "\t\t.msg{color:#ff9d1b;width:80%;background-color:green}\n";
$STYLE .= "\t\t.nt{color:#ffAd2b;font-size:x-small;margin-top:2ex}\n";
$STYLE .= "\t\t.text{color:#ff9d1b;text-align:justify;}\n";
$STYLE .= "\t\t.logo{width:30%; float:left;}\n";
$STYLE .= "\t\t.buttons{text-align: center;margin-top:2ex;}\n";
$STYLE .= "\t\t.button{background-color: #ffAb2b; margin-left: 2ex; ";
$STYLE .= "margin-right: 2ex; padding-right: 1ex; padding-left: 1ex; ";
$STYLE .= "padding-top: 3px; padding-bottom: 3px; border-style: dotted; ";
$STYLE .= "border-color: #ffffff; border-width: 1px; margin-top: 2ex;}\n";
$STYLE .= "\t\t.button_submit{background-color: #ffAb2b; margin-right: 2ex;";
$STYLE .= " padding-right: 1ex; padding-left: 1ex; padding-top: 3px;";
$STYLE .= " padding-bottom: 3px; border-style: dotted; border-color: #ffffff;";
$STYLE .= " border-width: 1px; color: #ffffff; text-decoration:underline;";
$STYLE .= " font-size: 100%; margin-top: 2ex; position:absolute; right: 20%;}\n";
$STYLE .= "\t\t.form_label{color:#ffad2b;}\n";
$STYLE .= "\t\t.errors{background-color: white; margin-top: 2ex;";
$STYLE .= " padding: 1ex 1ex 1ex 1ex; font-align:center;}\n";
$STYLE .= "\t\t.error{color: red; margin-bottom: 1ex;}\n";
$STYLE .= "\t\tPRE {color: blue; background-color: #BBBBBB; padding: 1em 1em 1em 1em;";
$STYLE .= " border: 1px dashed white;}";
$STYLE .= "\t\t.noerror{color: green; margin-bottom: 1ex;}\n";
$STYLE .= "\t\t.compteur{color: #cecee1; width: 100%; text-align:center; margin-top: 30px;}\n";
$STYLE .= "\t</style>\n";



/* ------------------- les globals pour le XML -------------------- */
/**
 * le tag courant et le precedent
 * le precedent est necessaire a cause de title..de plus il permet de tester
 * que le fichier XML est fiable 
 **/
global $currentTag, $previousTag, $deep;
$GLOBALS['deep'] = 0;

global $deptype;       // le type de la dependance en cours
global $webservername; // le nom des serveur en cours d'enregistrement
global $dbservername;

/* la source en cours de completion */
global $sourcecours; 
$GLOBALS['sourcecours'] = new ClassSource();

/* la relation en cours de construction */
global $tablecours; 
$GLOBALS["tablecours"] = new ClassTable();

global $currentscriptType; // le type de script d'install en cours
global $currentpageok;     // determine si on se trouve dans la bonne page




/* ============== LES VARIABLES DE GESTION DES ERREURS  ============== */



    /* Toutes les variables ci-dessus doivent etre completees
       au fur et a mesure de l'execution des differents fonctions
       car elles sont un moyen de verifier leur bon deroulement   */

// $error_nbr est un tableau associatif qui associe a un nom de 
// fonction un code d'erreur correspondant a la derniere execution
// par defaut on associe a main le code d'execution OK
global $error_nbr;
$error_nbr['main'] = 0;

// $error_msg est un tableau associatif qui associe a un nom de fonction
// un message d'erreur correspondant a la derniere execution de la fonction
// par defaut on associe a main le message OK
global $error_msg;
$error_msg['main'] = 'ok';




/* ================ MOTEUR DE LANCEMENT DES PRIMITIVES ================ */
/**
 * @function lancer_primitive
 *
 * @brief fonction qui permet de balancer une primitive et de 
 * gerer le retour d'erreur
 * @param $prim la primitive a lancer
 * @param $install l'objet contenant le configuration de l'install
 *
 * @return boolean le resultat de la primitive
 **/
function lancer_primitive($prim, &$install)
{
  /* Pour fonctionner correctement, les primitives doivent respecter
     un certain nombre de regles, notamment :
            - tout nom de primitive doit etre ecrit en masjuscule
            - les parametres, si il y en a, doivent etre encadres de majuscule
            et separes par des virgules. Si un des parametres contient plus d'un
            mot il doit etre place entre guillemets.
            - tout caractere precede par \ est rajoute sans prendre en compte
            son eventuel sens special                                           */

  /* on recupere le nom de la primitive : on copie tous les caracteres jusqu'a
    la premiere parenthese ouvrante .*/
  $prim_name = "";
  for($i=0 ; $i<strlen($prim) ; $i++)
    {
      if($prim[$i] != "(")
	$prim_name .= $prim[$i];
      else
	break;
    }

  /* on recupere les arguments dans un tableau, chaque argument est
   stocke dans une case de $args en partant de 1                    */
  $args = array(); $nb_quote=0; $tmp = "";
  if($i < strlen($prim)){
    if($prim[$i] == "(")
      {
	/* on parcourt la chaine jusqu'a trouver des virgules */
	for($j=$i+1 ; $j<strlen($prim) ; $j++)
	  {

	    switch ($prim[$j])
	      {
	      case '"':
		/* on prend en compte les guillemets */
		if($nb_quote != 0)
		  $nb_quote = 0;
		else
		  $nb_quote = 1;
		$tmp .= $prim[$j];
		break;

	      case "\\":
		/* on ajoute le caractere et le suivant sans en tenir compte */
		$tmp .= $prim[$j];
		if($j+1 < strlen($prim)) 
		  $tmp.= $prim[++$j];
		break;
		
	      case ',':
		/* on tombe sur un separateur d'args, on regarde si il
	       est protege par une paire de guillemets ou non */
		if($nb_quote == 0)
		  {
		    /* nouvel argument */
		    $args[] = $tmp;
		    $tmp = "";
		  }
		else
		  {
		    /* ce n'est pas un nouvel argument */
		    $tmp .= $prim[$j];
		  }
		break;
		
	      default:
		$tmp .= $prim[$j];
		break;
	      }/* end switch */
	    
	  }/* end for */
	
	/* on insere le dernier argument */
	$args[] = $tmp;
      }
  }
  /* sinon il n'y a pas d'arguments */

  /* on lance la fonction */
  if(function_exists($prim_name)) {
    /* build arguments string */
    $params = "";
    for($i=0 ; $i<count($args) ; $i++){
      $params .= $args[$i] . ",";
    }

    /* launch the function */
    $params .= "\$install";

    return eval("$prim_name($params);");
  }
}


/* ======================================================================= */
/* ======= LES FONCTIONS DE PARSING DES FICHIERS DE CONFIGURATION  ======= */
/* ======================================================================= */


/* =========== TRAITEMENT DES ELEMENTS GLOBAUX =========== */

/**
 * @func TreatGlobalElement_Gasell
 *
 * @brief element racine du fichier de configuration
 * xml
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatGlobalElement_Gasell($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      /* ouverture du tag */
      $GLOBALS['currentTag'] = 'gasell';
      $GLOBALS['previousTag'] = 'root';
    }
  else
    {
      /* fermeture du tag */
      $GLOBALS['currentTag'] = 'root';
    }
}

/**
 * @func TreatGlobalElement_Doc
 *
 * @brief element de documentation du xml
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatGlobalElement_Doc($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      /* ouverture du tag */
      $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
      $GLOBALS['currentTag'] = 'doc';
    }
  else
    {
      /* fermeture du tag */
      if(strcasecmp($GLOBALS['currentTag'],'doc') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];

	  /* on determine le precedent en fonction de la place */
	  switch($GLOBALS['currentTag'])
	    {
	    case 'gasell':
	      $GLOBALS['previousTag'] = 'root';
	      break;

	    case 'table':
	      $GLOBALS['previousTag'] = 'database';
	      break;

	    case 'column':
	      $GLOBALS['previousTag'] = 'table';
	      break;

	    case 'default':
	      $install->pageencours->updateLog("PARSING",1,"XML parsing error 1");
	      break;
	    }
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 1"); }
    }
}

/* =========== TRAITEMENT DES ELEMENTS DE DATABASE =========== */


/**
 * @func TreatDatabaseElement_Database
 *
 * @brief element racine de la declaration de la base
 * de donnees
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatDatabaseElement_Database($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      if(strcasecmp($GLOBALS['currentTag'],'gasell') == 0)
	{
	  /* ouverture du tag */
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'database';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 2"); }
    }
  else
    {
      /* fermeture du tag */
      if(strcasecmp($GLOBALS['currentTag'],'database') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'root';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 3"); }
    }
}


/**
 * @func TreatDatabaseElement_Table
 *
 * @brief element de declaration d'une relation
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatDatabaseElement_Table($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      if(strcasecmp($GLOBALS['currentTag'],'database') == 0)
	{
	  /* ouverture du tag */
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'table';

	  /* on recupere le nom de la table */
	  $GLOBALS["tablecours"]->name = $attarr["NAME"];
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 4"); }
    }
  else
    {
      /* fermeture du tag */
      if(strcasecmp($GLOBALS['currentTag'],'table') == 0)
	{
	  /* on ferme le tag */
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'gasell';

	  /* on ajoute la table a la base */
	  $install->addTable($GLOBALS["tablecours"]);

	  /* on vide la table */
	  unset($GLOBALS["tablecours"]);

	  /* ... et on prepare la prochaine */
	  $GLOBALS["tablecours"] = new ClassTable();
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 5"); }
    }
}


/**
 * @func TreatDatabaseElement_Column
 *
 * @brief element de declaration d'une colonne de relation
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatDatabaseElement_Column($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      if(strcasecmp($GLOBALS['currentTag'],'table') == 0)
	{
	  /* ouverture du tag */
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'column';

	  /* preparation de la colonne */
	  $col = $attarr["NAME"]." ".$attarr["TYPE"];

	  /* on genere la contrainte si besoin */
	  if(isset($attarr["CONSTRAINT"]) && !empty($attarr["CONSTRAINT"]))
	    {
	      switch ($attarr["CONSTRAINT"])
		{
		case "PRIMARY":
		case "primary":
		  $col .= " PRIMARY KEY";
		  break;

		case "REFERENCES":
		case "references":
		  /* on verifie que la contrainte est valide */
		  if(isset($attarr["REFERENCES"]) && !empty($attarr["REFERENCES"]))
		    { $col .= " REFERENCES ".$attarr["REFERENCES"]; }
		  else
		    { 
		      /* par securite on efface la requete*/
		      $col = ""; 
		      $install->pageencours->updateLog("PARSING",2,"XML corrupted");
		    }
		  break;

		case "UNIQUE":
		case "unique":
		  $col .= " UNIQUE";
		  break;
		}
	    }

	  /* enregistrement des donnees */
	  if($col != "")
	    $GLOBALS["tablecours"]->addColumn($col);
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 6"); }
    }
  else
    {
      /* fermeture du tag */
      if(strcasecmp($GLOBALS['currentTag'],'column') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'database';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 7"); }
    }
}


/**
 * @func TreatDatabaseElement_Constraint
 *
 * @brief element de declaration d'une contrainte sur une table
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode  de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la config de l'install
 **/
function TreatDatabaseElement_Constraint($attarr,$md,&$install)
{
  /* on verifie dans quel contexte la fonction est appelee */
  if($md == 'begin')
    {
      if(strcasecmp($GLOBALS['currentTag'],'table') == 0)
	{
	  /* ouverture du tag */
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'constraint';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 8"); }
    }
  else
    {
      /* fermeture du tag */
      if(strcasecmp($GLOBALS['currentTag'],'constraint') == 0)
	{
	  /* on commite la nouvelle contrainte */
	  $const = $GLOBALS['currentconstraint'];
	  $GLOBALS['tablecours']->addConstraint($const);
	  $GLOBALS['currentconstraint'] = "";

	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'database';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 9"); }
    }
}

/* =========== TRAITEMENT DES ELEMENTS DE L'INSTALL ========== */

/**--------------------------------------------------
 * @func TreatInstallElement_Install
 *
 * Fonction qui gere l'ouverture d'un tag Install
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Install($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag install 
	 on verifie que le tag courant est le root */
      if(strcasecmp($GLOBALS['currentTag'],'gasell') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'install';
	  /* ce tag n'a aucun attribut et aucun autre contenu */
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
      
      // on retourne le deroulement de la fonction
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'install') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'root';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 10"); }
    }
}



/**--------------------------------------------------
 * @func TreatInstallElement_Global
 *
 * Fonction qui gere l'ouverture d'un tag Global
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Global($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag global
	 on verifie que le tag courant est install */
      if(strcasecmp($GLOBALS['currentTag'],'install') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'global';
	  
	  /* ce tag n'a aucun attribut et aucun autre contenu */
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,"XML parsing error 11");
	  $res = false;
	}

      // on retourne le deroulement de la focntion
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'global') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'gasell';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 12"); }
    }
}


/**--------------------------------------------------
 * @func TreatInstallElement_System
 *
 * Fonction qui gere l'ouverture d'un tag System
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_System($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag system
	 on verifie que le tag courant est global */
      if(strcasecmp($GLOBALS['currentTag'],'global') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'system';
	  
	  /* ce tag n'a aucun attribut et aucun autre contenu */
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}

      // on retourne le deroulement de la focntion
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'system') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'install';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,"XML parsing error 13"); }
    }
}

/**--------------------------------------------------
 * @func TreatInstallElement_Dep
 *
 * Fonction qui gere l'ouverture d'un tag dep
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Dep($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag dep
	 on verifie que le tag courant est le system */
      if(strcasecmp($GLOBALS['currentTag'],'system') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'dep';
	  
	  /* 2 attributs : le type et le nom */
	  switch ($attarr['TYPE'])
	    {
	    case 'DB':
	      $GLOBALS['dbservername'] = $attarr['NAME'];
	      $GLOBALS['deptype'] = 'DB';
	      break;

	    case 'Web':
	      $GLOBALS['webservername'] = $attarr['NAME'];
	      $GLOBALS['deptype'] = 'Web';
	      break;
	    }

	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}

      // on retourne le deroulement de la fonction
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'dep') == 0)
	{
	  $GLOBALS['deptype'] = '';
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'global';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**--------------------------------------------------
 * @func TreatInstallElement_Version
 *
 * Fonction qui gere l'ouverture d'un tag version
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Version($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag version
	 on verifie que le tag courant est le dep */
      if(strcasecmp($GLOBALS['currentTag'],'dep') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'version';

	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
      
      // on retourne le deroulement de la fonction
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'version') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'system';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**--------------------------------------------------
 * @func TreatInstallElement_TitleG
 *
 * Fonction qui gere l'ouverture d'un tag title
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_TitleG($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag title (g)
	 on verifie que le tag courant est le system */
      if(strcasecmp($GLOBALS['currentTag'],'global') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'title';
	  
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
      
      // on retourne le deroulement de la focntion
      return $res;
    }
  else /* fermeture du tag */
    {
      if( (strcasecmp($GLOBALS['currentTag'],'title') == 0) && 
	  (strcasecmp($GLOBALS['previousTag'],'global') == 0) )
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'install';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**--------------------------------------------------
 * @func TreatInstallElement_Sources
 *
 * Fonction qui gere l'ouverture d'un tag sources
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Sources($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag sources
	 on verifie que le tag courant est le system */
      if(strcasecmp($GLOBALS['currentTag'],'global') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'sources';

	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}

      // on retourne le deroulement de la focntion
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'sources') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'install';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}


/**--------------------------------------------------
 * @func TreatInstallElement_Source
 *
 * Fonction qui gere l'ouverture d'un tag source
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Source($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag source
	 on verifie que le tag courant est le sources */
      if(strcasecmp($GLOBALS['currentTag'],'sources') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'source';

	  /* on sauvegarde des donnees de la source */
	  if (isset($attarr['TYPE']))
	  {
	  	$GLOBALS['sourcecours']->setType($attarr['TYPE']);
	  }
	  if (isset($attarr['PRIORITY']))
	  {
	  	$GLOBALS['sourcecours']->setPriority($attarr['PRIORITY']);
	  }
	  
	  /* on recupere les tags */
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
      
      // on retourne le deroulement de la focntion
      return $res;
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'source') == 0)
	{
	  /* on ajoute la source a la liste de sources */
	  $install->addSource($GLOBALS['sourcecours']);
	  
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'global';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}


/**--------------------------------------------------
 * @func TreatInstallElement_Extension
 *
 * Fonction qui gere le tage extension
 *
 * @param $attarr tableau contenant les attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 *---------------------------------------------------*/
function TreatInstallElement_Extension($attarr, $md, &$install){
  /* entree dans le tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag extension
	 on verifie que le tag courant est le global */
      if(strcasecmp($GLOBALS['currentTag'],'global') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'extension';

	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
  } else { /* sortie du tag */
    if(strcasecmp($GLOBALS['currentTag'],'extension') == 0)
	{
	  /* on replace les tags courants */
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'install';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));}
  }
}


/**
 * @function TreatInstallElement_Script
 *
 * @brief gestion de l'entree dans le
 * tag script
 * @param $attarr l'ensemble des attributs
 * @param $md le mode de traitement du tag : ouverture ou fermeture
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Script($attarr,$md,&$install)
{
  /* entree dans le tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag script
	 on verifie que le tag courant est le install */
      if(strcasecmp($GLOBALS['currentTag'],'install') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'script';
	  
	  /* on recupere les tags */
	  if(strcasecmp($attarr['TYPE'],$install->getMode()) == 0)
	    /* on indique que l'on est dans le bon script type */
	    $GLOBALS['currentscriptType'] = true;
	  
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* sortie du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'script') == 0)
	{
	  /* on replace les tags courants */
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'gasell';

	  /* si le currentscript etait positionne...on le retire */
	  $GLOBALS['currentscriptType'] = false;
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));}
    }
}


/**
 * @function TreatInstallElement_Page
 *
 * @brief gestion de l'entree dans le
 * tag page
 * @param $attarr l'ensemble des attributs
 * @param $md le mode d'installation : ouverture ou fermeture du tag
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Page($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag page
	 on verifie que le tag courant est le script */
      if(strcasecmp($GLOBALS['currentTag'],'script') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'page';

	  /* on augmente le nombre de page disponible */
	  $install->addPage($install->getMode());
	 
	  /* on determine si la page est la bonne */
	  if($attarr['NUMBER'] == $install->getCurrentPage())
	    {
	      /* on determine que l'on est dans la bonne page */
	      $GLOBALS['currentpageok'] = true;
	    }
	  
	  /* tout s'est bien passe */
	  $res = true;
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'install';

	  $GLOBALS['currentpageok'] = false;
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Logo
 *
 * @brief permet de traiter le tag logo
 * @param $attarr tableau des attributs du tag
 * @param $md le mode d'utilisation : ouverture ou fermeture du tag
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Logo($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag logo
	 on verifie que le tag courant est le page */
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'logo';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'logo') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'script';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_TitleNG
 *
 * @brief permet de traiter le tag title non globale
 * @param $attarr tableau des attributs du tag
 * @param $md le mode d'utilisation : ouverture ou fermeture du tag
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_TitleNG($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag title (ng)
	 on verifie que le tag courant est le page */
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'title';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if( (strcasecmp($GLOBALS['currentTag'],'title') == 0) &&
	  (strcasecmp($GLOBALS['previousTag'],'page') == 0) )
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'script';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Text
 *
 * @brief traitement du tag text
 * @param $attarr liste des attributs du tag
 * @param $md le mode de traitement du tag (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Text($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag text
	 on verifie que le tag courant est le page */
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'text';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'text') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'script';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Buttons
 *
 * @brief traitement du tag button
 * @param $attarr tableau des attributs du tag
 * @param $md le mode dans lequel on travail (ouverture/fermeture)
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Buttons($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag buttons
	 on verifie que le tag courant est le page */
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'buttons';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'buttons') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'script';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Button
 *
 * @brief traitement du tag Button
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Button($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag button
	 on verifie que le tag courant est le buttons */
      if(strcasecmp($GLOBALS['currentTag'],'buttons') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'button';
	  
	  /* on verifie que la page en cours est la bonne */
	  if($GLOBALS['currentpageok'] && $GLOBALS['currentscriptType'])
	    {
	      /* on fixe le bon bouton */
	      switch ($attarr['TYPE'])
		{
		case 'cancel':
		  $install->putCancel();
		  break;

		case 'previous':
		  $install->putPrevious();
		  break;

		case 'next':
		  $install->putNext();
		  break;

		case 'finish':
		  $install->putFinish();
		  break;
		}
	    }
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'button') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'page';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Actions
 *
 * @brief traitement du tag Actions
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Actions($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag actions
	 on verifie que le tag courant est le page */
      if(strcasecmp($GLOBALS['currentTag'],'page') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'actions';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'actions') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'script';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));}
    }
}


/**
 * @function TreatInstallElement_Action
 *
 * @brief traitement du tag Action
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Action($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag action
	 on verifie que le tag courant est le actions */
      if(strcasecmp($GLOBALS['currentTag'],'actions') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'action';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'action') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'page';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));}
    }
}

/**
 * @function TreatInstallElement_Primitive
 *
 * @brief traitement du tag Primitive
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 **/
function TreatInstallElement_Primitive($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* lorsque l'on entre dans un tag Primitive
	 on verifie que le tag courant est le action */
      if(strcasecmp($GLOBALS['currentTag'],'action') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'primitive';
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'primitive') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'actions';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Optionnal
 *
 * @brief traitement du tag Optionnal
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 */
function TreatInstallElement_Optionnal($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* on verifie la bonne configuration du fichier */
      if(strcasecmp($GLOBALS['currentTag'],'action') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'optionnal';

	  /* not yet implemented */
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'optionnal') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'actions';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16)); }
    }
}

/**
 * @function TreatInstallElement_Required
 *
 * @brief traitement du tag Required
 * @param $attarr liste des attributs du tag
 * @param $md le mode d'utilisation
 * @param $install l'objet contenant toute la configuration
 * de l'installation en cours
 */
function TreatInstallElement_Required($attarr,$md,&$install)
{
  /* ouverture du tag */
  if($md == 'begin')
    {
      /* on verifie la bonne configuration du fichier */
      if(strcasecmp($GLOBALS['currentTag'],'action') == 0)
	{
	  $GLOBALS['previousTag'] = $GLOBALS['currentTag'];
	  $GLOBALS['currentTag'] = 'required';

	  /* not yet implemented */
	}
      else
	{
	  /* dans ce cas, il y a une erreur au niveau 
	     de la structure du fichier de conf       */
	  $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));
	  $res = false;
	}
    }
  else /* fermeture du tag */
    {
      if(strcasecmp($GLOBALS['currentTag'],'required') == 0)
	{
	  $GLOBALS['currentTag'] = $GLOBALS['previousTag'];
	  $GLOBALS['previousTag'] = 'actions';
	}
      else
	{ $install->pageencours->updateLog("PARSING",1,$install->getSentence(16));}
    }
}


/**--------------------------------------------------
 * Fonction qui permet d'effectuer le traitement 
 * lorsque le parser detecte l'ouverture d'un tag
 *
 * @param $parser le parser qui a appele la fonction
 * @param $tagname le tag dont le parser vient de 
 *                 detecter l'ouverture
 * @param $attarr tableau regroupant les attributs du tag
 *---------------------------------------------------*/
function beginTag($parser,$tagname,$attarr)
{
  $GLOBALS['deep']++;

  /* chacun des cas doit sauvegarder le nom du tag precedent 
     ds la variable $GLOBALS['previousTag'] et sauvegarder
     le tag courant ds la variable $GLOBALS['currentTag'] 
     seulement si le precedent est parent du courant.
     Le principe d'induction permet alors d'assurer que la
     hierarchie des tags sera controlee et respectee*/

  /* le tag 'root' est l'ancetre transcendental... il est
     le pere de tous les tags racines */

  /* on traite selon sa valeur */
  switch ($tagname)
    {
      /* =================================================== */
      /* =========== TRAITEMENT DES TAGS SPECIAUX ========== */
      /* =================================================== */
      /* tag root */
    case 'GASELL':
    case 'gasell':
      TreatGlobalElement_Gasell($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag intermediaire */
    case 'DOC':
    case 'doc':
      TreatGlobalElement_Doc($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* =================================================== */
      /* ==== TRAITEMENT DU SCHEMA DES BASES DE DONNEES ==== */
      /* =================================================== */
      /* tag racine */
    case 'DATABASE':
    case 'database':
      TreatDatabaseElement_Database($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'TABLE':
    case 'table':
      TreatDatabaseElement_Table($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 2 */
    case 'COLUMN':
    case 'column':
      TreatDatabaseElement_Column($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 2 */
    case 'CONSTRAINT':
    case 'constraint':
      TreatDatabaseElement_Constraint($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* =================================================== */
      /* == TRAITEMENT DE LA CONFIGURATION DE L'INSTALLER == */
      /* =================================================== */
      /* tag racine */
    case 'INSTALL': 
    case 'install':
      TreatInstallElement_Install($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'GLOBAL': 
    case 'global':
      TreatInstallElement_Global($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.1 */
    case 'SYSTEM':
    case 'system': 
      TreatInstallElement_System($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.system.1 */
    case 'DEP': 
    case 'dep':
      TreatInstallElement_Dep($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.system.dep.1 */
    case 'VERSION': 
    case 'version':
      TreatInstallElement_Version($attarr,'begin',$GLOBALS['INSTALL']);
      break;
      
      /* tag niveau global.1 */
    case 'TITLE':
    case 'title':
      if(($GLOBALS['currentTag'] == 'global')
	 ||($GLOBALS['currentTag'] == 'GLOBAL'))
	{ TreatInstallElement_TitleG($attarr,'begin',$GLOBALS['INSTALL']); }
      else
	{ TreatInstallElement_TitleNG($attarr,'begin',$GLOBALS['INSTALL']); }
      break;

      /* tag niveau global.1 */
    case 'SOURCES':
    case 'sources': 
      TreatInstallElement_Sources($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.sources.1 */
    case 'SOURCE':
    case 'source': 
      TreatInstallElement_Source($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.1 */
    case 'EXTENSION':
    case 'extension':
      TreatInstallElement_Extension($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'SCRIPT':
    case 'script': 
      TreatInstallElement_Script($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.1 */
    case 'PAGE':
    case 'page': 
      TreatInstallElement_Page($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.1 */
    case 'LOGO':
    case 'logo': 
      TreatInstallElement_Logo($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    case 'TEXT':
    case 'text': 
      TreatInstallElement_Text($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    case 'BUTTONS':
    case 'buttons': 
      TreatInstallElement_Buttons($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.buttons.1 */
    case 'BUTTON':
    case 'button': 
      TreatInstallElement_Button($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    case 'ACTIONS':
    case 'actions': 
      TreatInstallElement_Actions($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.actions.1 */
    case 'ACTION':
    case 'action': 
      TreatInstallElement_Action($attarr,'begin',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.actions.action.1 */
    case 'PRIMITIVE':
    case 'primitive': 
      TreatInstallElement_Primitive($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    case 'OPTIONNAL':
    case 'optionnal': 
      TreatInstallElement_Optionnal($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    case 'REQUIRED':
    case 'required': 
      TreatInstallElement_Required($attarr,'begin',$GLOBALS['INSTALL']);
      break;
    default:
      $GLOBALS['INSTALL']->getSentence(20);
    }
  /* si c'est autre chose, on ne s'en occupe pas ! */
}

/**--------------------------------------------------
 * Fonction qui gere le contenu des tags XML
 *
 * @param $parser le parser qui a appele la fonction
 * @param $cdata le contenu du tag xml
 *---------------------------------------------------*/
function dataTreatment($parser,$cdata)
{
  switch ($GLOBALS['currentTag'])
    {
      /* =================================================== */
      /* ==== TRAITEMENT DU SCHEMA DES BASES DE DONNEES ==== */
      /* =================================================== */

      /* =================================================== */
      /* == TRAITEMENT DE LA CONFIGURATION DE L'INSTALLER == */
      /* =================================================== */
      /* on enregistre les versions */
    case 'VERSION':
    case 'version':
      /* en fonction de la dependance en cours, on ajoute */
      switch ($GLOBALS['deptype'])
	{
	case 'Web':
	  $GLOBALS['INSTALL']->addWebServDep($GLOBALS['webservername'],$cdata);
	  break;
	case 'DB':
	  $GLOBALS['INSTALL']->addDBServDep($GLOBALS['dbservername'],$cdata);
	  break;	  
	}
      break;
      
      /* on enregistre le titre */
    case 'TITLE':
    case 'title':
      if(strcasecmp($GLOBALS['previousTag'],'global') == 0)
	{ $GLOBALS['INSTALL']->setInstallTitle($cdata); }
      else
	{
	  /* previousTag : page */
	  if($GLOBALS['currentpageok'] && $GLOBALS['currentscriptType'])
	    { $GLOBALS['INSTALL']->setPageTitle($cdata); }
	}
      break;

      /* on enregistre l'adresse de la source */
    case 'SOURCE':
    case 'source':
      /*$GLOBALS['sourcecours']->setAddress($cdata);*/
      /* nothing to do in theory */
      break;

      /* on enregistre le logo si la page est bonne */
    case 'LOGO':
    case 'logo':
      /* on verifie que la page est la bonne */
      if($GLOBALS['currentpageok'] && $GLOBALS['currentscriptType'])
	{
	  /* on peut ajouter le logo */
	  $GLOBALS['INSTALL']->setPageLogo($cdata);
	}
      break;

      /* on enregistre le texte de presentation si la page est bonne */
    case 'TEXT':
    case 'text':
      /* on verifie que la page est bonne */
      if($GLOBALS['currentpageok'] && $GLOBALS['currentscriptType'])
	{
	  /* on peut ajouter le texte */
	  $GLOBALS['INSTALL']->setPageText($cdata);
	}
      break;

      /* on charge l'extension */
    case 'EXTENSION':
    case 'extension':
      loading_dependencies($cdata);
      break;

      /* on balance la primitive */
    case 'PRIMITIVE':
    case 'primitive':
      /* on verifie que la page est la bonne */
      if($GLOBALS['currentpageok'] && $GLOBALS['currentscriptType'])
	{
	  /* on lance la primitive */
	  lancer_primitive($cdata,$GLOBALS['INSTALL']);
	}
      break;

      /* on ajoute la contrainte */
    case 'CONSTRAINT':
    case 'constraint':
      if(! isset($GLOBALS['currentconstraint']))
	$GLOBALS['currentconstraint'] = "";
      $GLOBALS['currentconstraint'] .= $cdata;
      break;
    }
}

/**--------------------------------------------------
 * Fonction qui permet d'effectuer le traitement 
 * lorsque le parser detecte la fermeture d'un tag
 *
 * @param $parser le parser qui a appele la fonction
 * @param $tagname le tag dont le parser vient de 
 *                 detecter l'ouverture
 *---------------------------------------------------*/
function endTag($parser,$tagname)
{
  $GLOBALS['deep']--;

  $attarr = array();

  switch ($tagname)
    {
      /* =================================================== */
      /* =========== TRAITEMENT DES TAGS SPECIAUX ========== */
      /* =================================================== */
      /* tag root */
    case 'GASELL':
    case 'gasell':
      TreatGlobalElement_Gasell($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* tag intermediaire */
    case 'DOC':
    case 'doc':
      TreatGlobalElement_Doc($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* =================================================== */
      /* ==== TRAITEMENT DU SCHEMA DES BASES DE DONNEES ==== */
      /* =================================================== */
      /* tag racine */
    case 'DATABASE':
    case 'database':
      TreatDatabaseElement_Database($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'TABLE':
    case 'table':
      TreatDatabaseElement_Table($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 2 */
    case 'COLUMN':
    case 'column':
      TreatDatabaseElement_Column($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 2 */
    case 'CONSTRAINT':
    case 'constraint':
      TreatDatabaseElement_Constraint($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* =================================================== */
      /* == TRAITEMENT DE LA CONFIGURATION DE L'INSTALLER == */
      /* =================================================== */
      /* tag racine */
    case 'INSTALL':
    case 'install': 
      TreatInstallElement_Install(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'GLOBAL':
    case 'global':
      TreatInstallElement_Global(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.1 */
    case 'SYSTEM':
    case 'system':
      TreatInstallElement_System(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.system.1 */
    case 'DEP':
    case 'dep':
      TreatInstallElement_Dep(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.system.dep.1 */
    case 'VERSION':
    case 'version': 
      TreatInstallElement_Version(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.1 */
    case 'TITLE':
    case 'title':
      if(($GLOBALS['previousTag'] == 'global')
	 ||($GLOBALS['previousTag'] == 'GLOBAL'))
	{ TreatInstallElement_TitleG(array(),'end',$GLOBALS['INSTALL']); }
      else
	{ TreatInstallElement_TitleNG(array(),'end',$GLOBALS['INSTALL']); }
      break;

      /* tag niveau global.1 */
    case 'SOURCES':
    case 'sources':
      TreatInstallElement_Sources(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.sources.1 */
    case 'SOURCE':
    case 'source':
      TreatInstallElement_Source(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau global.1 */
    case 'EXTENSION':
    case 'extension':
      TreatInstallElement_Extension(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau 1 */
    case 'SCRIPT':
    case 'script':
      TreatInstallElement_Script(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.1 */
    case 'PAGE':
    case 'page':
      TreatInstallElement_Page(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.1 */
    case 'LOGO':
    case 'logo':
      TreatInstallElement_Logo(array(),'end',$GLOBALS['INSTALL']);
      break;
    case 'TEXT':
    case 'text':
      TreatInstallElement_Text(array(),'end',$GLOBALS['INSTALL']);
      break;
    case 'BUTTONS':
    case 'buttons':
      TreatInstallElement_Buttons(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.buttons.1 */
    case 'BUTTON':
    case 'button':
      TreatInstallElement_Button(array(),'end',$GLOBALS['INSTALL']);
      break;
    case 'ACTIONS':
    case 'actions':
      TreatInstallElement_Actions(array(),'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.actions.1 */
    case 'ACTION':
    case 'action':
      TreatInstallElement_Action($attarr,'end',$GLOBALS['INSTALL']);
      break;

      /* tag niveau script.page.actions.action.1 */
    case 'PRIMITIVE':
    case 'primitive':
      TreatInstallElement_Primitive($attarr,'end',$GLOBALS['INSTALL']);
      break;
    case 'OPTIONNAL':
    case 'optionnal': 
      TreatInstallElement_Optionnal($attarr,'end',$GLOBALS['INSTALL']);
      break;
    case 'REQUIRED':
    case 'required': 
      TreatInstallElement_Required($attarr,'end',$GLOBALS['INSTALL']);
      break;
    }
  /* si c'est autre chose, on ne s'en occupe pas ! */
}

/**--------------------------------------------------
 * Fonction qui met en place le parser du fichier de 
 * configuration
 *
 * @param $file le fichier a parser
 *---------------------------------------------------*/
function parsing($file)
{
  /* Mise en place du parser */
  $parser = xml_parser_create('ISO-8859-1');

  /* definition des fonctions de traitement */
  xml_set_element_handler($parser,"beginTag","endTag");
  xml_set_character_data_handler($parser,"dataTreatment");
  
  /* initialisation du tag courant */
  $GLOBALS['currentTag'] = 'root';

  /* ouverture du fichier et lecture*/
  $fp = fopen($file,'r');
  while ($data=fread($fp,4096)){
    xml_parse($parser,$data,feof($fp)) 
      || die(sprintf('XML Error : %s at line %d', 
		     xml_error_string(xml_get_error_code($parser)),
		     xml_get_current_line_number($parser)));
  }
  
  /* on libere  la memoire utilisee par le parser */
  xml_parser_free($parser);
}

/* ===== LES FONCTIONS DE MISE EN PLACE DES VARIABLES D'INSTALLATION ===== */

/* == LES FONCTIONS DE DEPLOIEMENT DES FICHIERS DE CONFIGURATION GASELL == */


/* ======================================================================= */
/* =================== FONCTIONS PROPRES A L'INSTALLER =================== */
/* ======================================================================= */

/**---------------------------------------------------
 * Fonction qui permet de charger les fichiers de 
 * langage pour l'installer
 **---------------------------------------------------*/
function loading_languages(){
  /* on recupere tous les repertoire du path */
  $ldirs = explode(":",PATH_LANGUAGES);

  /* et on les charge */
  foreach($ldirs as $dir){
    if ($handle = opendir($dir)) {

      while (false !== ($file = readdir($handle))) {
	if(ereg("^_lang_(.*)\.php$",$file,$regs)){
	  /* chargement du fichier */
	  if(include_once($file)){
	    /* ajout de la langue */
	    $hdle = fopen($file,"r");
	    $tmp = fgets($hdle); $tmp = fgets($hdle);
	    fclose($hdle);

	    $lng = preg_replace("@//@", "", $tmp);
	    $GLOBALS['INSTALL']->addLanguage($regs[1], $lng);
	  }
	}
      }
    
      closedir($handle);
    } else {
      error_log("Impossible to browse the directory $dir");
    }
  }
}

/**---------------------------------------------------
 * Fonction qui permet de telecharger des fichiers 
 * manquant a partir des adresses specifiees dans la
 * constante PATH_DOWNLOAD
 *
 * @param address l'adresse du fichier a telecharger
 * @param dest    la destination
 **---------------------------------------------------*/
function download_file($file, $dest) {
  // TODO : check we can download ;)
  // TODO : consider text and binay downloads

  /* prepare the URLs to download the file */
  $urls = explode(":",PATH_DOWNLOAD);
  $j=0; $is_dl = false;
  
  /* on teste toutes les urls du path */
  while((!$is_dl) && ($j<count($urls))){
    
    if($whdl = fopen($urls[$j]."/".$file,"r")){
      /* le fichier est accessible, on cree la copie locale */
      if($fhdle = fopen($dest."/".$file,"w")){
	/* copie du fichier en local */
	while(!feof($whdl)){
	  $tmp = fgets($whdl);
	  if((fwrite($fhdle, $tmp)) === false){
	    error_log("Failed to download $file from ".$urls[$j]."<br/>");
	  }
	  
	  close($fhdle);
	} /* fichier downloade */
      } else {
	error_log("Impossible to write in the current directory");
      }
      
      /* on ferme le fichier */
      close($whdl);
      $is_dl = true;
    }
  }
  
  // is it a success ?
  return $is_dl;
}

/**---------------------------------------------------
 * Fonction qui permet de charger une ou des depedances
 * a partir du parametre
 *
 * @param deps string representant une dependance ou bien
 * tableau de dependances
 **---------------------------------------------------*/
function loading_dependencies($dep){

  /* traitement du parametre selon son type */
  if(is_string($dep)){

    /* on charge le fichier si il est dans le repertoire courant */
    if(file_exists("_ext_".$dep.".php")){
      require_once("_ext_".$dep.".php");
    } else {
      /* on le telecharge */
      /* on charge le fichier si il a ete telecharge */
      if(!download_file("_ext_$dep.php", ".")){
	error_log("Impossible to resolve the dependencie");
      } else {
	require_once("_ext_$dep.php");
      }
    }
  } elseif(is_array($dep)) {
    foreach($dep as $d)
      if(is_string($d) || is_array($d))
	loading_dependencies($d);
  }
}


/* ======================================================================= */
/* ================= DIVERSES FONCTIONS DE PRESENTATION ================== */
/* ======================================================================= */

/**--------------------------------------------------
 * Fonction affiche a l'ecran une erreur fatale de 
 * l'installaton
 *
 * @param $txt la description de l'erreur
 * @param $install objet contenant les donnees de
 * configuratiio sur l'installation en cours
 *---------------------------------------------------*/
function erreur_fatale($txt,&$install)
{
  // on agit differemment selon qu'il s'agisse d'une
  // installation graphique ou en mode texte
  if($install->getMode() == "graphic")
    {
      // installation en mode graphique...donc visible
      $template = "<html>\n".$GLOBALS['STYLE'];
      $template .= "<body bgcolor='#0e5158'>\n";
      $template .= "\t<h1>".$install->getSentence(14)."</h1>\n";
      $template .= "\t<div class='msg'>".$txt."</div>\n";
      $template .= "\t<div class='nt'>L'&eacute;quipe de d&eacute;veloppement";
      $template .= " GASELL vous prie de l'excuser pour ce d&eacute;sagrement:";
      $template .= "<a href='http://www.gasell.org'>www.gasell.org</a></div>\n";
      $template .= "\t<div class='msg'>Pour aider au debuggage, merci d'inclure";
      $template .= " les informations ci-dessous :</div>\n";
      $template .= "\t<div class='error_debug'>".$install->genDebug()."</div>\n";
      $template .= "</body>\n</html>";

      echo $template;
    }
  else
    {
      // installation en mode texte...simple message
      die($txt);
    }
}


/*===========================================================================*/
/*====== LE CORPS DE L'INSTALLATION : DEROULEMENT DE L'INSTALLATION =========*/
/*===========================================================================*/

/* test des options passees au script d'install par la methode GET */
/* +sieurs types :                                                 */
/*  - choix du type d'installation : $_GET['type'] == graphic|text */
/*  par defaut, l'installation d'effectue en mode graphique        */
/*  - page en cours de traitement : $_GET['page'] == [0-9]+        */
/*  en mode texte il n'y a pas de pages                            */
/*  - choix de la langue d'installation : $_GET['lang'] == ....    */

/* on charge la configuration de l'installation */
$GLOBALS['INSTALL'] = new InstallConfig();

/**
 * on charge les langues
 **/
loading_languages();

/**
 * INSERTION d'une SECURITE: on verifie qu'il n'y a pas de lock dans
 * le repertoire de l'installation !
 **/
$CDIR = dirname(__FILE__);
if (file_exists("$CDIR/lock")) {
	// redirection vers l'index de l'application
	echo "<html>\n<head>\n";
	echo "<meta http-equiv=\"refresh\" content=\"2;url=../index.php\">\n";
	echo "</head>\n<body>\n<center><b>";
	echo "The installation is locked<br/>\n";
	echo "You have to remove the lock file to launch the installation\n";
	echo "</b></center>\n</body>\n</html>";
	exit();
}

/**
 * on recupere toutes les donnees qui sont postees
 **/
if($_SERVER["REQUEST_METHOD"] == 'POST')
{
  /* on recupere chacune des variables */
  foreach($_POST as $k=>$v)
    {
      /* as the parameter come from user ... we encode it ! */
      $GLOBALS['INSTALL']->addFormArg($k,$v);
    }
}

/**
 * on recupere toutes les donnees recuperees precedemment
 * et passees dans l'adresse
 **/
foreach($_GET as $k=>$v) {
  $GLOBALS['INSTALL']->addFormArg($k,$v);
}
    
/**
 * On recupere egalement les donnees passees en parametre
 * par la ligne de commande (contenus $_SERVER['argv'])
 **/
if($_SERVER["argc"] > 0)
{
  /* On debute a l'index 1 car 0 contient le nom de la commande */
  for($i = 1; $i < $_SERVER["argc"]; $i++)
    {
      /* On decompose chaque chaine "param1=value1&param2=value2" 
	      dans un tableau */
      parse_str($_SERVER["argv"][$i], $parsedArray);
      /* On ajoute tout ca dans les parametres du formulaire */
      foreach($parsedArray as $k=>$v)
	$GLOBALS['INSTALL']->addFormArg($k, $v);
    }
}


/* on teste si il s'agit d'une page suite a un cancel ou un termine */
if($GLOBALS['INSTALL']->getCurrentPage() == -1)
{
  /* on cree une nouvelle page */
  $pagecancel = new ClassPage();
  
  /* on la remplie */
  $pagecancel->title     = $GLOBALS['INSTALL']->getSentence(21);
  $pagecancel->logo      = "";
  $pagecancel->txt       = $GLOBALS['INSTALL']->getSentence(22);
  $pagecancel->bCancel   = false;
  $pagecancel->bNext     = false;
  $pagecancel->bPrevious = false;
  $pagecancel->bFinish   = false;;
  
  /* on l'affiche */
  $pagecancel->afficher(false);
}
else if($GLOBALS['INSTALL']->getCurrentPage() == -2)
{
  /* delete the session values if there are some (bug #4832)*/
  session_start();
  if(isset($_SESSION['gasell_auth']))
    session_destroy();

	/* we create the lock to prevent access to the setup */
	if($fh = @fopen("$CDIR/lock", "w")) {
		fwrite($fh, "REMOVE THIS FILE TO UNLOCK THE INSTALLATION\n");
		fclose($fh);
		$txt = "";
	} else {
		$txt  = "<br/><br/><div class=\"errors\">\n";
		$txt .= "<div class=\"error\">Impossible to create the lock ... ";
		$txt .= "please create a file named lock in $CDIR in order";
		$txt .= "to prevent forward installations.</div></div>";
	}

  /* on cree une nouvelle page */
  $pagefinish = new ClassPage();
  
  /* on la remplie */
  $pagefinish->title     = $GLOBALS['INSTALL']->getSentence(23);
  $pagefinish->logo      = "";
  $pagefinish->txt       = $GLOBALS['INSTALL']->getSentence(24) . $txt;
  $pagefinish->bCancel   = false;
  $pagefinish->bNext     = false;
  $pagefinish->bPrevious = false;
  $pagefinish->bFinish   = false;
  
  /* on l'affiche */
  $pagefinish->afficher();
}
/*TODO a supprimer plus tard car pose des problemes de securite*/
else if($GLOBALS['INSTALL']->getCurrentPage() == "hack")
{
  /* on recupere la primitive a lancer */
  $prim = $GLOBALS['INSTALL']->getParameter("primitive");
  if($prim != false) {
    parsing(CONFIG_FILE);

    $msg = "Execute primitive : ".$prim."<br />";
    lancer_primitive($prim, $GLOBALS["INSTALL"]);
  } else {
    $msg = "<br />No primitive specified !!";
  }

  /* on cree une nouvelle page */
  $pagehack = new ClassPage();
  $pagehack->title     = "INSTALLER HACKING PAGE";
  $pagehack->txt       = "BEWARE !! Use this page ONLY if you ";
  $pagehack->txt      .= "know what you're doing !".$msg;
  $pagehack->bCancel   = false;
  $pagehack->bNext     = false;
  $pagehack->bPrevious = false;
  $pagehack->bFinish   = false;
  /* on l'affiche */
  $pagehack->afficher();
}
else
{

  /**
   * on charge les extensions obligatoires
   **/
  $exts = explode(":",DEP_MANDATORY);
  loading_dependencies($exts);

  /**
   * test de la presence du fichier de configuration de l'install 
   **/
  if(file_exists(CONFIG_FILE))
    {
      /* test des capacites xml du serveur */
      if(CHECK_XML_PHPSUPPORT($GLOBALS["INSTALL"],false))
	{
	  /* on parse le fichier de config */
	  parsing(CONFIG_FILE);

	/* Mode graphique => affichage de la page HTML */
	if ($GLOBALS['INSTALL']->getMode() == "graphic")
	  {
	  	/* on affiche la page construite */
	  	$GLOBALS['INSTALL']->afficherPage();
	  }
	}
      else
	{
	  // on affiche une erreur fatale
	  erreur_fatale($error_msg['CHECK_XML_PHPSUPPORT'],$GLOBALS['INSTALL']);
	}
    }else{
      // fichier de configuration introuvable...c'est pas bon
      erreur_fatale($install->getSentence(15),$GLOBALS['INSTALL']->getMode(),
		    $GLOBALS['INSTALL']);
    }
}
?>
      
